@@ -4,11 +4,13 @@
 #include "bus/nscsi/hd.h"
 #include "imagedev/harddriv.h"
 
+#include "multibyte.h"
+
 #define LOG_COMMAND     (1U << 1)
 #define LOG_DATA        (1U << 2)
 #define LOG_UNSUPPORTED (1U << 3)
 
-#define VERBOSE 0
+//#define VERBOSE 3
 
 #include "logmacro.h"
 
@@ -22,6 +24,15 @@ nscsi_harddisk_device::nscsi_harddisk_device(const machine_config &mconfig, cons
 nscsi_harddisk_device::nscsi_harddisk_device(const machine_config &mconfig, device_type type, const char *tag, device_t *owner, uint32_t clock) :
 	nscsi_full_device(mconfig, type, tag, owner, clock), image(*this, "image"), lba(0), cur_lba(0), blocks(0), bytes_per_sector(0)
 {
+	m_default_model_name =
+		" SEAGATE"
+		"          ST225N"
+		"1.00";
+}
+
+void nscsi_harddisk_device::set_default_model_name(const std::string_view &model)
+{
+	m_default_model_name = model;
 }
 
 void nscsi_harddisk_device::device_start()
@@ -50,7 +61,7 @@ void nscsi_harddisk_device::device_reset()
 
 void nscsi_harddisk_device::device_add_mconfig(machine_config &config)
 {
-	HARDDISK(config, image).set_interface("scsi_hdd");
+	HARDDISK(config, image).set_interface("scsi_hdd,hdd");
 }
 
 uint8_t nscsi_harddisk_device::scsi_get_data(int id, int pos)
@@ -59,9 +70,7 @@ uint8_t nscsi_harddisk_device::scsi_get_data(int id, int pos)
 	if(id != 2)
 	{
 		data = nscsi_full_device::scsi_get_data(id, pos);
-	}
-	else
-	{
+	} else {
 		int clba = lba + pos / bytes_per_sector;
 		if(clba != cur_lba) {
 			cur_lba = clba;
@@ -108,7 +117,7 @@ void nscsi_harddisk_device::scsi_command()
 		break;
 
 	case SC_READ_6:
-		lba = ((scsi_cmdbuf[1] & 0x1f)<<16) | (scsi_cmdbuf[2]<<8) | scsi_cmdbuf[3];
+		lba = get_u24be(&scsi_cmdbuf[1]) & 0x1fffff;
 		blocks = scsi_cmdbuf[4];
 		if(!blocks)
 			blocks = 256;
@@ -118,24 +127,27 @@ void nscsi_harddisk_device::scsi_command()
 		if(image->read(lba, block)) {
 			scsi_data_in(2, blocks*bytes_per_sector);
 			scsi_status_complete(SS_GOOD);
-		}
-		else
-		{
+		} else {
 			scsi_status_complete(SS_CHECK_CONDITION);
 			sense(false, SK_ILLEGAL_REQUEST, SK_ASC_INVALID_FIELD_IN_CDB);
 		}
 		break;
 
 	case SC_WRITE_6:
-		lba = ((scsi_cmdbuf[1] & 0x1f)<<16) | (scsi_cmdbuf[2]<<8) | scsi_cmdbuf[3];
+		lba = get_u24be(&scsi_cmdbuf[1]) & 0x1fffff;
 		blocks = scsi_cmdbuf[4];
 		if(!blocks)
 			blocks = 256;
 
 		LOG("command WRITE start=%08x blocks=%04x\n", lba, blocks);
 
-		scsi_data_out(2, blocks*bytes_per_sector);
-		scsi_status_complete(SS_GOOD);
+		if(image->write(lba, block)) {
+			scsi_data_out(2, blocks*bytes_per_sector);
+			scsi_status_complete(SS_GOOD);
+		} else {
+			scsi_status_complete(SS_CHECK_CONDITION);
+			sense(false, SK_ILLEGAL_REQUEST, SK_ASC_INVALID_FIELD_IN_CDB);
+		}
 		break;
 
 	case SC_INQUIRY: {
@@ -168,9 +180,11 @@ void nscsi_harddisk_device::scsi_command()
 				LOG("IDNT tag not found in chd metadata, using default inquiry data\n");
 
 				// Apple HD SC setup utility needs to see this
-				strcpy((char *)&scsi_cmdbuf[8], " SEAGATE");
-				strcpy((char *)&scsi_cmdbuf[16], "          ST225N");
-				strcpy((char *)&scsi_cmdbuf[32], "1.00");
+				memset(scsi_cmdbuf+8, ' ', 28);
+				int len = m_default_model_name.size();
+				if(len > 28)
+					len = 28;
+				memcpy(scsi_cmdbuf+8, m_default_model_name.data(), len);
 				scsi_cmdbuf[36] = 0x00; // # of extents high
 				scsi_cmdbuf[37] = 0x08; // # of extents low
 				scsi_cmdbuf[38] = 0x00; // group 0 commands 0-1f
@@ -221,13 +235,11 @@ void nscsi_harddisk_device::scsi_command()
 		uint32_t dsize = info.cylinders * info.heads * info.sectors - 1;
 		scsi_cmdbuf[pos++] = 0x08; // Block descriptor length
 		scsi_cmdbuf[pos++] = 0x00;
-		scsi_cmdbuf[pos++] = (dsize>>16) & 0xff;
-		scsi_cmdbuf[pos++] = (dsize>>8) & 0xff;
-		scsi_cmdbuf[pos++] = (dsize & 0xff);
+		put_u24be(&scsi_cmdbuf[pos], dsize);
+		pos += 3;
 		scsi_cmdbuf[pos++] = 0x00;
-		scsi_cmdbuf[pos++] = (info.sectorbytes>>16)&0xff;
-		scsi_cmdbuf[pos++] = (info.sectorbytes>>8)&0xff;
-		scsi_cmdbuf[pos++] = (info.sectorbytes & 0xff);
+		put_u24be(&scsi_cmdbuf[pos], info.sectorbytes);
+		pos += 3;
 
 		int pmax = page == 0x3f ? 0x3e : page;
 		int pmin = page == 0x3f ? 0x00 : page;
@@ -245,7 +257,7 @@ void nscsi_harddisk_device::scsi_command()
 			case 0x01: // read-write error recovery page
 				scsi_cmdbuf[pos++] = 0x01; // !PS, page id
 				scsi_cmdbuf[pos++] = 0x0a; // page length
-				scsi_cmdbuf[pos++] = 0; // various bits
+				scsi_cmdbuf[pos++] = 0x26; // various bits
 				scsi_cmdbuf[pos++] = 0; // read retry count
 				scsi_cmdbuf[pos++] = 0; // correction span
 				scsi_cmdbuf[pos++] = 0; // head offset count
@@ -279,18 +291,18 @@ void nscsi_harddisk_device::scsi_command()
 			case 0x03:  { // Format parameters page
 				scsi_cmdbuf[pos++] = 0x83; // PS, page id
 				scsi_cmdbuf[pos++] = 0x16; // Page length
-				scsi_cmdbuf[pos++] = (info.cylinders * info.heads) >> 8; // Track/zone
-				scsi_cmdbuf[pos++] = info.cylinders * info.heads;        // Track/zone
+				put_u16be(&scsi_cmdbuf[pos], info.cylinders * info.heads); // Track/zone
+				pos += 2;
 				scsi_cmdbuf[pos++] = 0x00; // Alt sect/zone
 				scsi_cmdbuf[pos++] = 0x00; // Alt sect/zone
 				scsi_cmdbuf[pos++] = 0x00; // Alt track/zone
 				scsi_cmdbuf[pos++] = 0x00; // Alt track/zone
 				scsi_cmdbuf[pos++] = 0x00; // Alt track/volume
 				scsi_cmdbuf[pos++] = 0x00; // Alt track/volume
-				scsi_cmdbuf[pos++] = info.sectors >> 8; // Sectors/track
-				scsi_cmdbuf[pos++] = info.sectors;      // Sectors/track
-				scsi_cmdbuf[pos++] = info.sectorbytes >> 8; // Bytes/sector
-				scsi_cmdbuf[pos++] = info.sectorbytes;      // Bytes/sector
+				put_u16be(&scsi_cmdbuf[pos], info.sectors); // Sectors/track
+				pos += 2;
+				put_u16be(&scsi_cmdbuf[pos], info.sectorbytes); // Bytes/sector
+				pos += 2;
 				scsi_cmdbuf[pos++] = 0x00; // Interleave
 				scsi_cmdbuf[pos++] = 0x00; // Interleave
 				scsi_cmdbuf[pos++] = 0x00; // Track skew
@@ -307,28 +319,27 @@ void nscsi_harddisk_device::scsi_command()
 			case 0x04: { // Rigid drive geometry page
 				scsi_cmdbuf[pos++] = 0x84; // PS, page id
 				scsi_cmdbuf[pos++] = 0x16; // Page length
-				scsi_cmdbuf[pos++] = info.cylinders >> 16; // Cylinders
-				scsi_cmdbuf[pos++] = info.cylinders >> 8;  // Cylinders
-				scsi_cmdbuf[pos++] = info.cylinders;       // Cylinders
-				scsi_cmdbuf[pos++] = info.heads;           // Heads
-				scsi_cmdbuf[pos++] = 0x00;                  // Starting cylinder - write precomp
-				scsi_cmdbuf[pos++] = 0x00;                  // Starting cylinder - write precomp
-				scsi_cmdbuf[pos++] = 0x00;                  // Starting cylinder - write precomp
-				scsi_cmdbuf[pos++] = 0x00;                  // Starting cylinder - reduced write current
-				scsi_cmdbuf[pos++] = 0x00;                  // Starting cylinder - reduced write current
-				scsi_cmdbuf[pos++] = 0x00;                  // Starting cylinder - reduced write current
-				scsi_cmdbuf[pos++] = 0x00;                  // Drive step rate
-				scsi_cmdbuf[pos++] = 0x00;                  // Drive step rate
-				scsi_cmdbuf[pos++] = 0x00;                  // Landing zone cylinder
-				scsi_cmdbuf[pos++] = 0x00;                  // Landing zone cylinder
-				scsi_cmdbuf[pos++] = 0x00;                  // Landing zone cylinder
-				scsi_cmdbuf[pos++] = 0x00;                  // RPL
-				scsi_cmdbuf[pos++] = 0x00;                  // Rotational offset
-				scsi_cmdbuf[pos++] = 0x00;                  // Reserved
-				scsi_cmdbuf[pos++] = uint8_t(10000 >> 8);     // Medium rotation rate
-				scsi_cmdbuf[pos++] = uint8_t(10000);          // Medium rotation rate
-				scsi_cmdbuf[pos++] = 0x00;                  // Reserved
-				scsi_cmdbuf[pos++] = 0x00;                  // Reserved
+				put_u24be(&scsi_cmdbuf[pos], info.cylinders); // Cylinders
+				pos += 3;
+				scsi_cmdbuf[pos++] = info.heads;              // Heads
+				scsi_cmdbuf[pos++] = 0x00;                    // Starting cylinder - write precomp
+				scsi_cmdbuf[pos++] = 0x00;                    // Starting cylinder - write precomp
+				scsi_cmdbuf[pos++] = 0x00;                    // Starting cylinder - write precomp
+				scsi_cmdbuf[pos++] = 0x00;                    // Starting cylinder - reduced write current
+				scsi_cmdbuf[pos++] = 0x00;                    // Starting cylinder - reduced write current
+				scsi_cmdbuf[pos++] = 0x00;                    // Starting cylinder - reduced write current
+				scsi_cmdbuf[pos++] = 0x00;                    // Drive step rate
+				scsi_cmdbuf[pos++] = 0x00;                    // Drive step rate
+				scsi_cmdbuf[pos++] = 0x00;                    // Landing zone cylinder
+				scsi_cmdbuf[pos++] = 0x00;                    // Landing zone cylinder
+				scsi_cmdbuf[pos++] = 0x00;                    // Landing zone cylinder
+				scsi_cmdbuf[pos++] = 0x00;                    // RPL
+				scsi_cmdbuf[pos++] = 0x00;                    // Rotational offset
+				scsi_cmdbuf[pos++] = 0x00;                    // Reserved
+				put_u16be(&scsi_cmdbuf[pos], 10000);          // Medium rotation rate
+				pos += 2;
+				scsi_cmdbuf[pos++] = 0x00;                    // Reserved
+				scsi_cmdbuf[pos++] = 0x00;                    // Reserved
 				break;
 			}
 
@@ -385,7 +396,7 @@ void nscsi_harddisk_device::scsi_command()
 		}
 
 		if (!fail) {
-			scsi_cmdbuf[0] = pos;
+			scsi_cmdbuf[0] = pos - 1;
 			if (pos > size)
 				pos = size;
 
@@ -403,9 +414,9 @@ void nscsi_harddisk_device::scsi_command()
 		scsi_status_complete(SS_GOOD);
 		break;
 
-	case SC_RECIEVE_DIAG_RES: {
-		LOG("command RECIEVE DIAGNOSTICS RESULTS\n");
-		int size = (scsi_cmdbuf[3] << 8) | scsi_cmdbuf[4];
+	case SC_RECEIVE_DIAGNOSTIC_RESULTS: {
+		LOG("command RECEIVE DIAGNOSTIC RESULTS\n");
+		int size = get_u16be(&scsi_cmdbuf[3]);
 		int pos = 0;
 		scsi_cmdbuf[pos++] = 0;
 		scsi_cmdbuf[pos++] = 6;
@@ -421,9 +432,9 @@ void nscsi_harddisk_device::scsi_command()
 		break;
 	}
 
-	case SC_SEND_DIAGNOSTICS: {
-		LOG("command SEND DIAGNOSTICS\n");
-		int size = (scsi_cmdbuf[3] << 8) | scsi_cmdbuf[4];
+	case SC_SEND_DIAGNOSTIC: {
+		LOG("command SEND DIAGNOSTIC\n");
+		int size = get_u16be(&scsi_cmdbuf[3]);
 		if(scsi_cmdbuf[1] & 4) {
 			// Self-test
 			scsi_status_complete(SS_GOOD);
@@ -445,20 +456,19 @@ void nscsi_harddisk_device::scsi_command()
 		break;
 	}
 
+	case SC_SYNCHRONIZE_CACHE:
+		LOG("command SYNCHRONIZE CACHE (10)\n");
+		scsi_status_complete(SS_GOOD);
+		break;
+
 	case SC_READ_CAPACITY: {
 		LOG("command READ CAPACITY\n");
 
 		const auto &info = image->get_info();
 		uint32_t size = info.cylinders * info.heads * info.sectors - 1;
 
-		scsi_cmdbuf[0] = (size>>24) & 0xff;
-		scsi_cmdbuf[1] = (size>>16) & 0xff;
-		scsi_cmdbuf[2] = (size>>8) & 0xff;
-		scsi_cmdbuf[3] = (size & 0xff);
-		scsi_cmdbuf[4] = (info.sectorbytes>>24)&0xff;
-		scsi_cmdbuf[5] = (info.sectorbytes>>16)&0xff;
-		scsi_cmdbuf[6] = (info.sectorbytes>>8)&0xff;
-		scsi_cmdbuf[7] = (info.sectorbytes & 0xff);
+		put_u32be(&scsi_cmdbuf[0], size);
+		put_u32be(&scsi_cmdbuf[4], info.sectorbytes);
 
 		scsi_data_in(0, 8);
 		scsi_status_complete(SS_GOOD);
@@ -466,8 +476,8 @@ void nscsi_harddisk_device::scsi_command()
 	}
 
 	case SC_READ_10:
-		lba = (scsi_cmdbuf[2]<<24) | (scsi_cmdbuf[3]<<16) | (scsi_cmdbuf[4]<<8) | scsi_cmdbuf[5];
-		blocks = (scsi_cmdbuf[7] << 8) | scsi_cmdbuf[8];
+		lba = get_u32be(&scsi_cmdbuf[2]);
+		blocks = get_u16be(&scsi_cmdbuf[7]);
 
 		LOG("command READ EXTENDED start=%08x blocks=%04x\n",lba, blocks);
 
@@ -483,13 +493,20 @@ void nscsi_harddisk_device::scsi_command()
 		break;
 
 	case SC_WRITE_10:
-		lba = (scsi_cmdbuf[2]<<24) | (scsi_cmdbuf[3]<<16) | (scsi_cmdbuf[4]<<8) | scsi_cmdbuf[5];
-		blocks = (scsi_cmdbuf[7] << 8) | scsi_cmdbuf[8];
+		lba = get_u32be(&scsi_cmdbuf[2]);
+		blocks = get_u16be(&scsi_cmdbuf[7]);
 
 		LOG("command WRITE EXTENDED start=%08x blocks=%04x\n", lba, blocks);
 
-		scsi_data_out(2, blocks*bytes_per_sector);
-		scsi_status_complete(SS_GOOD);
+		if(image->write(lba, block)) {
+			scsi_data_out(2, blocks*bytes_per_sector);
+			scsi_status_complete(SS_GOOD);
+		}
+		else
+		{
+			scsi_status_complete(SS_CHECK_CONDITION);
+			sense(false, SK_ILLEGAL_REQUEST, SK_ASC_INVALID_FIELD_IN_CDB);
+		}
 		break;
 
 	case SC_FORMAT_UNIT:
@@ -523,7 +540,7 @@ void nscsi_harddisk_device::scsi_command()
 		scsi_status_complete(SS_GOOD);
 		break;
 
-	case SC_VERIFY:
+	case SC_VERIFY_10:
 		LOG("command VERIFY BytChk %d\n", !!(scsi_cmdbuf[1] & 0x02));
 		if (!(scsi_cmdbuf[1] & 0x02))
 			scsi_status_complete(SS_GOOD);
