@@ -165,14 +165,14 @@ MODRM_TABLE i386_MODRM_table[256];
 
 /*************************************************************************/
 
-uint32_t i386_device::i386_translate(int segment, uint32_t ip, int rwn)
+uint32_t i386_device::i386_translate(int segment, uint32_t ip, int rwn, int size)
 {
 	// TODO: segment limit access size, execution permission, handle exception thrown from exception handler
 	if (PROTECTED_MODE && !V8086_MODE && (rwn != -1))
 	{
 		if (!(m_sreg[segment].valid))
 			FAULT_THROW((segment == SS) ? FAULT_SS : FAULT_GP, 0);
-		if (i386_limit_check(segment, ip))
+		if (i386_limit_check(segment, ip, size))
 			FAULT_THROW((segment == SS) ? FAULT_SS : FAULT_GP, 0);
 		if ((rwn == 0) && ((m_sreg[segment].flags & 8) && !(m_sreg[segment].flags & 2)))
 			FAULT_THROW(FAULT_GP, 0);
@@ -203,7 +203,7 @@ bool i386_device::i386_translate_address(int intention, bool debug, offs_t *addr
 	bool user = (intention & TR_USER) ? true : false;
 	bool write = (intention & TR_WRITE) ? true : false;
 
-	if (!(m_cr[0] & 0x80000000))
+	if (!(m_cr[0] & CR0_PG))
 	{
 		if (entry)
 			*entry = 0x77;
@@ -213,7 +213,7 @@ bool i386_device::i386_translate_address(int intention, bool debug, offs_t *addr
 	uint32_t page_dir = m_program->read_dword(pdbr + directory * 4);
 	if (page_dir & 1)
 	{
-		if ((page_dir & 0x80) && (m_cr[4] & 0x10))
+		if ((page_dir & 0x80) && (m_cr[4] & CR4_PSE))
 		{
 			a = (page_dir & 0xffc00000) | (a & 0x003fffff);
 			if (debug)
@@ -283,7 +283,7 @@ bool i386_device::i386_translate_address(int intention, bool debug, offs_t *addr
 
 bool i386_device::translate_address(int pl, int type, uint32_t *address, uint32_t *error)
 {
-	if (!(m_cr[0] & 0x80000000)) // Some (very few) old OS's won't work with this
+	if (!(m_cr[0] & CR0_PG)) // Some (very few) old OS's won't work with this
 		return true;
 
 	const vtlb_entry *table = vtlb_table();
@@ -1543,7 +1543,7 @@ void i386_device::i386_check_irq_line()
 	if ( (m_irq_state) && m_IF )
 	{
 		m_cycles -= 2;
-		i386_trap(standard_irq_callback(0, m_pc), 1, 0);
+		i386_trap(standard_irq_callback(0, m_pc), 1);
 	}
 }
 
@@ -1592,7 +1592,7 @@ void i386_device::report_invalid_modrm(const char* opcode, uint8_t modrm)
 	for (uint32_t i = 1; i < 16; i++)
 		logerror("  %08X\n", m_opcode_addrs[(m_opcode_addrs_index - i) & 15]);
 #endif
-	i386_trap(6, 0, 0);
+	i386_trap(6, 0);
 }
 
 
@@ -2043,8 +2043,6 @@ void i386_device::i386_common_init()
 
 	machine().save().register_postload(save_prepost_delegate(FUNC(i386_device::i386_postload), this));
 
-	m_smiact.resolve_safe();
-	m_ferr_handler.resolve_safe();
 	m_ferr_handler(0);
 
 	set_icountptr(m_cycles);
@@ -2154,9 +2152,9 @@ void i386_device::register_state_i386_x87()
 {
 	register_state_i386();
 
-	state_add( X87_CTRL,   "x87_CW", m_x87_cw).formatstr("%04X");
-	state_add( X87_STATUS, "x87_SW", m_x87_sw).formatstr("%04X");
-	state_add( X87_TAG,    "x87_TAG", m_x87_tw).formatstr("%04X");
+	state_add(X87_CTRL,  "x87_CW", m_x87_cw).formatstr("%04X");
+	state_add(X87_STATUS,"x87_SW", m_x87_sw).formatstr("%04X");
+	state_add(X87_TAG,  "x87_TAG", m_x87_tw).formatstr("%04X");
 	state_add( X87_ST0,    "ST0", m_debugger_temp ).callexport().formatstr("%15s");
 	state_add( X87_ST1,    "ST1", m_debugger_temp ).callexport().formatstr("%15s");
 	state_add( X87_ST2,    "ST2", m_debugger_temp ).callexport().formatstr("%15s");
@@ -2547,8 +2545,7 @@ void i386_device::enter_smm()
 
 	m_cr[0] &= ~(0x8000000d);
 	set_flags(2);
-	if(!m_smiact.isnull())
-		m_smiact(true);
+	m_smiact(true);
 	m_smm = true;
 	m_smi_latched = false;
 
@@ -2699,8 +2696,7 @@ void i386_device::leave_smm()
 			m_sreg[i].valid = true;
 	}
 
-	if (!m_smiact.isnull())
-		m_smiact(false);
+	m_smiact(false);
 	m_smm = false;
 
 	CHANGE_PC(m_eip);
@@ -2728,7 +2724,7 @@ void i386_device::execute_set_input(int irqline, int state)
 			return;
 		}
 		if ( state )
-			i386_trap(2, 1, 0);
+			i386_trap(2, 1);
 	}
 	else
 	{
@@ -2782,6 +2778,7 @@ void i386_device::execute_run()
 
 	if (m_halted)
 	{
+		debugger_wait_hook();
 		m_tsc += cycles;
 		m_cycles = 0;
 		return;
@@ -2804,7 +2801,7 @@ void i386_device::execute_run()
 				{
 					uint32_t phys_addr = 0;
 					uint32_t error;
-					phys_addr = (m_cr[0] & (1 << 31)) ? translate_address(m_CPL, TR_FETCH, &m_dr[i], &error) : m_dr[i];
+					phys_addr = (m_cr[0] & CR0_PG) ? translate_address(m_CPL, TR_FETCH, &m_dr[i], &error) : m_dr[i];
 					if(breakpoint_length != 0) // Not one byte in length? logerror it, I have no idea how this works on real processors.
 					{
 						LOGMASKED(LOG_INVALID_OPCODE, "i386: Breakpoint length not 1 byte on an instruction breakpoint\n");
@@ -2813,7 +2810,7 @@ void i386_device::execute_run()
 					{
 						// The processor never automatically clears bits in DR6. It only sets them.
 						m_dr[6] |= 1 << i;
-						i386_trap(1,0,0);
+						i386_trap(1,0);
 						break;
 					}
 				}
@@ -2853,7 +2850,7 @@ void i386_device::execute_run()
 				m_prev_eip = m_eip;
 				m_ext = 1;
 				m_dr[6] |= (1 << 14); //Set BS bit of DR6.
-				i386_trap(1,0,0);
+				i386_trap(1,0);
 			}
 			if(m_lock && (m_opcode != 0xf0))
 				m_lock = false;
@@ -3315,8 +3312,23 @@ void pentium2_device::device_reset()
 	m_cpuid_max_input_value_eax = 0x02;
 	m_cpu_version = REG32(EDX);
 
-	// [ 0:0] FPU on chip
-	m_feature_flags = 0x008081bf;       // TODO: enable relevant flags here
+	// [ 0: 0] FPU on chip
+	// [ 1: 1] VME Virtual 8086 Mode Enhancements
+	// [ 2: 2] DE Debugging Extensions
+	// [ 3: 3] PSE Page Size Extension
+	// [ 4: 4] TSC Time Stamp Counter
+	// [ 5: 5] MSR Model Specific Registers
+	// [ 6: 6] PAE Physical Address Extension
+	// [ 7: 7] MCE Machine Check Exception
+	// [ 8: 8] CMPXCHG8B opcode supported
+	// [11:11] SEP SYSENTER and SYSEXIT opcodes
+	// [12:12] MTRR Memory type range register
+	// [13:13] PGE Page Global Enable
+	// [14:14] MCA Machine Check Architecture
+	// [15:15] CMOV Conditional Move instructions
+	// [23:23] MMX instructions
+	//m_feature_flags = 0x0080f9ff;
+	m_feature_flags = 0x008081bf;  // TODO: enable missing flags
 
 	CHANGE_PC(m_eip);
 }
@@ -3399,7 +3411,7 @@ void pentium3_device::opcode_cpuid()
 		{
 			// TODO: lower part of 96 bits s/n for Pentium III processors only (ditched in 4)
 			// (upper 32-bits part is in EAX=1 EAX return)
-			// NB: if this is triggered from an Arcade system then there's a very good chance
+			// NOTE: if this is triggered from an Arcade system then there's a very good chance
 			// that is trying to tie the serial as a form of copy protection cfr. gamecstl
 			LOGMASKED(LOG_MSR, "CPUID with EAX=00000003 (Pentium III PSN?) at %08x!\n", m_eip);
 			REG32(EAX) = 0x00000000;
