@@ -485,7 +485,7 @@ void tms340x0_device::write_pixel_16(offs_t offset, uint32_t data)
 void tms340x0_device::write_pixel_32(offs_t offset, uint32_t data)
 {
 	/* TODO: plane masking */
-	TMS34010_WRMEM_WORD(offset & 0xffffffe0, data);
+	TMS34010_WRMEM_DWORD(offset & 0xffffffe0, data);
 }
 
 /* No Raster Op + Transparency */
@@ -712,8 +712,6 @@ void tms340x0_device::device_start()
 {
 	m_scanline_ind16_cb.resolve();
 	m_scanline_rgb32_cb.resolve();
-	m_output_int_cb.resolve();
-	m_ioreg_pre_write_cb.resolve();
 	m_to_shiftreg_cb.resolve();
 	m_from_shiftreg_cb.resolve();
 
@@ -865,6 +863,7 @@ void tms340x0_device::execute_run()
 	/* Get out if CPU is halted. Absolutely no interrupts must be taken!!! */
 	if (IOREG(REG_HSTCTLH) & 0x8000)
 	{
+		debugger_wait_hook();
 		m_icount = 0;
 		return;
 	}
@@ -878,7 +877,7 @@ void tms340x0_device::execute_run()
 	/* check interrupts first */
 	m_executing = true;
 	check_interrupt();
-	if ((machine().debug_flags & DEBUG_FLAG_ENABLED) == 0)
+	if (!debugger_enabled())
 	{
 		do
 		{
@@ -1244,7 +1243,7 @@ static const char *const ioreg_name[] =
 
 void tms34010_device::io_register_w(offs_t offset, u16 data, u16 mem_mask)
 {
-	if (!m_ioreg_pre_write_cb.isnull())
+	if (!m_ioreg_pre_write_cb.isunset())
 		m_ioreg_pre_write_cb(offset, data, mem_mask);
 
 	int oldreg, newreg;
@@ -1319,15 +1318,9 @@ void tms34010_device::io_register_w(offs_t offset, u16 data, u16 mem_mask)
 
 				/* the TMS34010 can set output interrupt? */
 				if (!(oldreg & 0x0080) && (newreg & 0x0080))
-				{
-					if (!m_output_int_cb.isnull())
-						m_output_int_cb(1);
-				}
+					m_output_int_cb(1);
 				else if ((oldreg & 0x0080) && !(newreg & 0x0080))
-				{
-					if (!m_output_int_cb.isnull())
-						m_output_int_cb(0);
-				}
+					m_output_int_cb(0);
 
 				/* input interrupt? (should really be state-based, but the functions don't exist!) */
 				if (!(oldreg & 0x0008) && (newreg & 0x0008))
@@ -1395,7 +1388,7 @@ static const char *const ioreg020_name[] =
 
 void tms34020_device::io_register_w(offs_t offset, u16 data, u16 mem_mask)
 {
-	if (!m_ioreg_pre_write_cb.isnull())
+	if (!m_ioreg_pre_write_cb.isunset())
 		m_ioreg_pre_write_cb(offset, data, mem_mask);
 
 	int oldreg, newreg;
@@ -1471,15 +1464,9 @@ void tms34020_device::io_register_w(offs_t offset, u16 data, u16 mem_mask)
 
 			/* the TMS34010 can set output interrupt? */
 			if (!(oldreg & 0x0080) && (newreg & 0x0080))
-			{
-				if (!m_output_int_cb.isnull())
-					m_output_int_cb(1);
-			}
+				m_output_int_cb(1);
 			else if ((oldreg & 0x0080) && !(newreg & 0x0080))
-			{
-				if (!m_output_int_cb.isnull())
-					m_output_int_cb(0);
-			}
+				m_output_int_cb(0);
 
 			/* input interrupt? (should really be state-based, but the functions don't exist!) */
 			if (!(oldreg & 0x0008) && (newreg & 0x0008))
@@ -1561,7 +1548,8 @@ u16 tms34010_device::io_register_r(offs_t offset)
 {
 	int result, total;
 
-	LOGCONTROLREGS("%s: read %s\n", machine().describe_context(), ioreg_name[offset]);
+	if (!machine().side_effects_disabled())
+		LOGCONTROLREGS("%s: read %s\n", machine().describe_context(), ioreg_name[offset]);
 
 	switch (offset)
 	{
@@ -1602,7 +1590,8 @@ u16 tms34020_device::io_register_r(offs_t offset)
 {
 	int result, total;
 
-	LOGCONTROLREGS("%s: read %s\n", machine().describe_context(), ioreg020_name[offset]);
+	if (!machine().side_effects_disabled())
+		LOGCONTROLREGS("%s: read %s\n", machine().describe_context(), ioreg020_name[offset]);
 
 	switch (offset)
 	{
@@ -1732,13 +1721,16 @@ u16 tms340x0_device::host_r(offs_t offset)
 			addr = (IOREG(REG_HSTADRH) << 16) | IOREG(REG_HSTADRL);
 			result = TMS34010_RDMEM_WORD(addr & 0xfffffff0);
 
-			/* optional postincrement (it says preincrement, but data is preloaded, so it
-			   is effectively a postincrement */
-			if (IOREG(REG_HSTCTLH) & 0x1000)
+			if (!machine().side_effects_disabled())
 			{
-				addr += 0x10;
-				IOREG(REG_HSTADRH) = addr >> 16;
-				IOREG(REG_HSTADRL) = (uint16_t)addr;
+				/* optional postincrement (it says preincrement, but data is preloaded, so it
+				   is effectively a postincrement */
+				if (IOREG(REG_HSTCTLH) & 0x1000)
+				{
+					addr += 0x10;
+					IOREG(REG_HSTADRH) = addr >> 16;
+					IOREG(REG_HSTADRL) = (uint16_t)addr;
+				}
 			}
 			break;
 
@@ -1749,7 +1741,8 @@ u16 tms340x0_device::host_r(offs_t offset)
 
 		/* error case */
 		default:
-			logerror("tms34010_host_control_r called on invalid register %d\n", reg);
+			if (!machine().side_effects_disabled())
+				logerror("tms34010_host_control_r called on invalid register %d\n", reg);
 			break;
 	}
 
