@@ -25,9 +25,12 @@
 #include <cstdint>
 #include <cstring>
 #include <ctime>
+#include <functional>
+#include <iosfwd>
 #include <initializer_list>
 #include <list>
 #include <memory>
+#include <string_view>
 #include <vector>
 
 
@@ -43,12 +46,13 @@ constexpr ioport_value IP_ACTIVE_LOW = 0xffffffff;
 constexpr int MAX_PLAYERS = 10;
 
 // unicode constants
+constexpr char32_t UCHAR_INVALID = 0xffff;
 constexpr char32_t UCHAR_PRIVATE = 0x100000;
 constexpr char32_t UCHAR_SHIFT_1 = UCHAR_PRIVATE + 0;
 constexpr char32_t UCHAR_SHIFT_2 = UCHAR_PRIVATE + 1;
 constexpr char32_t UCHAR_SHIFT_BEGIN = UCHAR_SHIFT_1;
 constexpr char32_t UCHAR_SHIFT_END = UCHAR_SHIFT_2;
-constexpr char32_t UCHAR_MAMEKEY_BEGIN = UCHAR_PRIVATE + 2;
+constexpr char32_t UCHAR_MAMEKEY_BEGIN = UCHAR_SHIFT_END + 1;
 
 
 // crosshair types
@@ -108,8 +112,8 @@ enum
 	INPUT_STRING_Coin_B,
 //  INPUT_STRING_20C_1C,    //  0.050000
 //  INPUT_STRING_15C_1C,    //  0.066667
-//  INPUT_STRING_10C_1C,    //  0.100000
-#define __input_string_coinage_start INPUT_STRING_9C_1C
+#define __input_string_coinage_start INPUT_STRING_10C_1C
+	INPUT_STRING_10C_1C,    //  0.100000
 	INPUT_STRING_9C_1C,     //  0.111111
 	INPUT_STRING_8C_1C,     //  0.125000
 	INPUT_STRING_7C_1C,     //  0.142857
@@ -126,13 +130,13 @@ enum
 	INPUT_STRING_3C_1C,     //  0.333333
 	INPUT_STRING_8C_3C,     //  0.375000
 //  INPUT_STRING_10C_4C,    //  0.400000
+	INPUT_STRING_5C_2C,     //  0.400000
 //  INPUT_STRING_7C_3C,     //  0.428571
 //  INPUT_STRING_9C_4C,     //  0.444444
 //  INPUT_STRING_10C_5C,    //  0.500000
 //  INPUT_STRING_8C_4C,     //  0.500000
 //  INPUT_STRING_6C_3C,     //  0.500000
 	INPUT_STRING_4C_2C,     //  0.500000
-	INPUT_STRING_5C_2C,     //  0.500000
 	INPUT_STRING_2C_1C,     //  0.500000
 //  INPUT_STRING_9C_5C,     //  0.555556
 //  INPUT_STRING_7C_4C,     //  0.571429
@@ -170,7 +174,6 @@ enum
 //  INPUT_STRING_6C_7C,     //  1.166667
 //  INPUT_STRING_5C_6C,     //  1.200000
 //  INPUT_STRING_8C_10C,    //  1.250000
-	INPUT_STRING_3C_5C,     //  1.250000
 	INPUT_STRING_4C_5C,     //  1.250000
 //  INPUT_STRING_7C_9C,     //  1.285714
 //  INPUT_STRING_6C_8C,     //  1.333333
@@ -182,7 +185,7 @@ enum
 	INPUT_STRING_2C_3C,     //  1.500000
 //  INPUT_STRING_5C_8C,     //  1.600000
 //  INPUT_STRING_6C_10C,    //  1.666667
-//  INPUT_STRING_3C_5C,     //  1.666667
+	INPUT_STRING_3C_5C,     //  1.666667
 	INPUT_STRING_4C_7C,     //  1.750000
 //  INPUT_STRING_5C_9C,     //  1.800000
 //  INPUT_STRING_5C_10C,    //  2.000000
@@ -209,20 +212,20 @@ enum
 	INPUT_STRING_1C_7C,     //  7.000000
 	INPUT_STRING_1C_8C,     //  8.000000
 	INPUT_STRING_1C_9C,     //  9.000000
-#define __input_string_coinage_end INPUT_STRING_1C_9C
-//  INPUT_STRING_1C_10C,    //  10.000000
+	INPUT_STRING_1C_10C,    //  10.000000
 //  INPUT_STRING_1C_11C,    //  11.000000
 //  INPUT_STRING_1C_12C,    //  12.000000
 //  INPUT_STRING_1C_13C,    //  13.000000
 //  INPUT_STRING_1C_14C,    //  14.000000
 //  INPUT_STRING_1C_15C,    //  15.000000
-//  INPUT_STRING_1C_20C,    //  20.000000
-//  INPUT_STRING_1C_25C,    //  25.000000
+	INPUT_STRING_1C_20C,    //  20.000000
+	INPUT_STRING_1C_25C,    //  25.000000
 //  INPUT_STRING_1C_30C,    //  30.000000
 //  INPUT_STRING_1C_40C,    //  40.000000
-//  INPUT_STRING_1C_50C,    //  50.000000
+	INPUT_STRING_1C_50C,    //  50.000000
 //  INPUT_STRING_1C_99C,    //  99.000000
-//  INPUT_STRING_1C_100C,   //  100.000000
+	INPUT_STRING_1C_100C,   //  100.000000
+#define __input_string_coinage_end INPUT_STRING_1C_100C
 //  INPUT_STRING_1C_120C,   //  120.000000
 //  INPUT_STRING_1C_125C,   //  125.000000
 //  INPUT_STRING_1C_150C,   //  150.000000
@@ -325,7 +328,7 @@ enum
 //**************************************************************************
 
 // constructor function pointer
-typedef void(*ioport_constructor)(device_t &owner, ioport_list &portlist, std::string &errorbuf);
+typedef void(*ioport_constructor)(device_t &owner, ioport_list &portlist, std::ostream &errorbuf);
 
 // I/O port callback function delegates
 typedef device_delegate<ioport_value ()> ioport_field_read_delegate;
@@ -470,7 +473,7 @@ public:
 	bool none() const { return (m_condition == ALWAYS); }
 
 	// configuration
-	void reset() { set(ALWAYS, nullptr, 0, 0); }
+	void reset() { set(ALWAYS, "", 0, 0); }
 	void set(condition_t condition, const char *tag, ioport_value mask, ioport_value value)
 	{
 		m_condition = condition;
@@ -485,7 +488,7 @@ public:
 private:
 	// internal state
 	condition_t     m_condition;    // condition to use
-	const char *    m_tag;          // tag of port whose condition is to be tested
+	const char *    m_tag;          // tag of port whose condition is to be tested (must never be nullptr)
 	ioport_port *   m_port;         // reference to the port to be tested
 	ioport_value    m_mask;         // mask to apply to the port
 	ioport_value    m_value;        // value to compare against
@@ -529,7 +532,7 @@ class ioport_diplocation
 {
 public:
 	// construction/destruction
-	ioport_diplocation(const char *name, u8 swnum, bool invert);
+	ioport_diplocation(std::string_view name, u8 swnum, bool invert);
 
 	// getters
 	const char *name() const { return m_name.c_str(); }
@@ -555,14 +558,12 @@ class ioport_field
 	friend class dynamic_field;
 
 	// flags for ioport_fields
-	static inline constexpr u32 FIELD_FLAG_OPTIONAL = 0x0001;    // set if this field is not required but recognized by hw
-	static inline constexpr u32 FIELD_FLAG_COCKTAIL = 0x0002;    // set if this field is relevant only for cocktail cabinets
-	static inline constexpr u32 FIELD_FLAG_TOGGLE =   0x0004;    // set if this field should behave as a toggle
-	static inline constexpr u32 FIELD_FLAG_ROTATED =  0x0008;    // set if this field represents a rotated control
-	static inline constexpr u32 ANALOG_FLAG_REVERSE = 0x0010;    // analog only: reverse the sense of the axis
-	static inline constexpr u32 ANALOG_FLAG_RESET =   0x0020;    // analog only: always preload in->default for relative axes, returning only deltas
-	static inline constexpr u32 ANALOG_FLAG_WRAPS =   0x0040;    // analog only: positional count wraps around
-	static inline constexpr u32 ANALOG_FLAG_INVERT =  0x0080;    // analog only: bitwise invert bits
+	static inline constexpr u32 FIELD_FLAG_COCKTAIL = 0x0001;    // set if this field is relevant only for cocktail cabinets
+	static inline constexpr u32 FIELD_FLAG_TOGGLE =   0x0002;    // set if this field should behave as a toggle
+	static inline constexpr u32 ANALOG_FLAG_REVERSE = 0x0004;    // analog only: reverse the sense of the axis
+	static inline constexpr u32 ANALOG_FLAG_RESET =   0x0008;    // analog only: always preload in->default for relative axes, returning only deltas
+	static inline constexpr u32 ANALOG_FLAG_WRAPS =   0x0010;    // analog only: positional count wraps around
+	static inline constexpr u32 ANALOG_FLAG_INVERT =  0x0020;    // analog only: bitwise invert bits
 
 public:
 	// construction/destruction
@@ -589,10 +590,8 @@ public:
 	void set_value(ioport_value value);
 	void clear_value();
 
-	bool optional() const { return ((m_flags & FIELD_FLAG_OPTIONAL) != 0); }
 	bool cocktail() const { return ((m_flags & FIELD_FLAG_COCKTAIL) != 0); }
 	bool toggle() const { return ((m_flags & FIELD_FLAG_TOGGLE) != 0); }
-	bool rotated() const { return ((m_flags & FIELD_FLAG_ROTATED) != 0); }
 	bool analog_reverse() const { return ((m_flags & ANALOG_FLAG_REVERSE) != 0); }
 	bool analog_reset() const { return ((m_flags & ANALOG_FLAG_RESET) != 0); }
 	bool analog_wraps() const { return ((m_flags & ANALOG_FLAG_WRAPS) != 0); }
@@ -664,7 +663,7 @@ public:
 	void set_user_settings(const user_settings &settings);
 
 private:
-	void expand_diplocation(const char *location, std::string &errorbuf);
+	void expand_diplocation(const char *location, std::ostream &errorbuf);
 
 	// internal state
 	ioport_field *              m_next;             // pointer to next field in sequence
@@ -744,7 +743,7 @@ class ioport_list : public std::map<std::string, std::unique_ptr<ioport_port>>
 public:
 	ioport_list() { }
 
-	void append(device_t &device, std::string &errorbuf);
+	void append(device_t &device, std::ostream &errorbuf);
 };
 
 
@@ -779,13 +778,13 @@ public:
 
 	// other operations
 	ioport_field *field(ioport_value mask) const;
-	void collapse_fields(std::string &errorbuf);
+	void collapse_fields(std::ostream &errorbuf);
 	void frame_update();
 	void init_live_state();
 	void update_defvalue(bool flush_defaults);
 
 private:
-	void insert_field(ioport_field &newfield, ioport_value &disallowedbits, std::string &errorbuf);
+	void insert_field(ioport_field &newfield, ioport_value &disallowedbits, std::ostream &errorbuf);
 
 	// internal state
 	ioport_port *               m_next;         // pointer to next port
@@ -1033,7 +1032,7 @@ class ioport_configurer
 {
 public:
 	// construction/destruction
-	ioport_configurer(device_t &owner, ioport_list &portlist, std::string &errorbuf);
+	ioport_configurer(device_t &owner, ioport_list &portlist, std::ostream &errorbuf);
 
 	// static helpers
 	static const char *string_from_token(const char *string);
@@ -1047,7 +1046,6 @@ public:
 	ioport_configurer& field_add_char(std::initializer_list<char32_t> charlist);
 	ioport_configurer& field_add_code(input_seq_type which, input_code code);
 	ioport_configurer& field_set_way(int way) { m_curfield->m_way = way; return *this; }
-	ioport_configurer& field_set_rotated() { m_curfield->m_flags |= ioport_field::FIELD_FLAG_ROTATED; return *this; }
 	ioport_configurer& field_set_name(const char *name) { assert(m_curfield != nullptr); m_curfield->m_name = string_from_token(name); return *this; }
 	ioport_configurer& field_set_player(int player) { m_curfield->m_player = player - 1; return *this; }
 	ioport_configurer& field_set_cocktail() { m_curfield->m_flags |= ioport_field::FIELD_FLAG_COCKTAIL; field_set_player(2); return *this; }
@@ -1056,7 +1054,6 @@ public:
 	ioport_configurer& field_set_analog_reverse() { m_curfield->m_flags |= ioport_field::ANALOG_FLAG_REVERSE; return *this; }
 	[[deprecated("PORT_RESET is deprecated; manage counter state explicitly")]]
 	ioport_configurer& field_set_analog_reset() { m_curfield->m_flags |= ioport_field::ANALOG_FLAG_RESET; return *this; }
-	ioport_configurer& field_set_optional() { m_curfield->m_flags |= ioport_field::FIELD_FLAG_OPTIONAL; return *this; }
 	ioport_configurer& field_set_min_max(ioport_value minval, ioport_value maxval) { m_curfield->m_min = minval; m_curfield->m_max = maxval; return *this; }
 	ioport_configurer& field_set_sensitivity(s32 sensitivity) { m_curfield->m_sensitivity = sensitivity; return *this; }
 	ioport_configurer& field_set_delta(s32 delta) { m_curfield->m_centerdelta = m_curfield->m_delta = delta; return *this; }
@@ -1070,6 +1067,7 @@ public:
 	ioport_configurer& field_set_dynamic_read(ioport_field_read_delegate delegate) { m_curfield->m_read = delegate; return *this; }
 	ioport_configurer& field_set_dynamic_write(ioport_field_write_delegate delegate, u32 param = 0) { m_curfield->m_write = delegate; m_curfield->m_write_param = param; return *this; }
 	ioport_configurer& field_set_diplocation(const char *location) { m_curfield->expand_diplocation(location, m_errorbuf); return *this; }
+	ioport_configurer& field_set_gm_note(u8 note);
 
 	// setting helpers
 	ioport_configurer& setting_alloc(ioport_value value, const char *name);
@@ -1082,11 +1080,12 @@ private:
 	// internal state
 	device_t &          m_owner;
 	ioport_list &       m_portlist;
-	std::string &       m_errorbuf;
+	std::ostream &      m_errorbuf;
 
 	ioport_port *       m_curport;
 	ioport_field *      m_curfield;
 	ioport_setting *    m_cursetting;
+	int                 m_curshift;
 };
 
 
@@ -1097,10 +1096,6 @@ private:
 
 #define UCHAR_MAMEKEY(code) (UCHAR_MAMEKEY_BEGIN + ITEM_ID_##code)
 
-// macro for a read callback function (PORT_CUSTOM)
-#define CUSTOM_INPUT_MEMBER(name)   ioport_value name()
-#define DECLARE_CUSTOM_INPUT_MEMBER(name)   ioport_value name()
-
 // macro for port write callback functions (PORT_CHANGED)
 #define INPUT_CHANGED_MEMBER(name)  void name(ioport_field &field, u32 param, ioport_value oldval, ioport_value newval)
 #define DECLARE_INPUT_CHANGED_MEMBER(name)  void name(ioport_field &field, u32 param, ioport_value oldval, ioport_value newval)
@@ -1126,7 +1121,7 @@ private:
 
 // start of table
 #define INPUT_PORTS_START(_name) \
-ATTR_COLD void INPUT_PORTS_NAME(_name)(device_t &owner, ioport_list &portlist, std::string &errorbuf) \
+ATTR_COLD void INPUT_PORTS_NAME(_name)(device_t &owner, ioport_list &portlist, std::ostream &errorbuf) \
 { \
 	ioport_configurer configurer(owner, portlist, errorbuf);
 // end of table
@@ -1135,7 +1130,7 @@ ATTR_COLD void INPUT_PORTS_NAME(_name)(device_t &owner, ioport_list &portlist, s
 
 // aliasing
 #define INPUT_PORTS_EXTERN(_name) \
-	extern void INPUT_PORTS_NAME(_name)(device_t &owner, ioport_list &portlist, std::string &errorbuf)
+	extern void INPUT_PORTS_NAME(_name)(device_t &owner, ioport_list &portlist, std::ostream &errorbuf)
 
 // including
 #define PORT_INCLUDE(_name) \
@@ -1177,9 +1172,6 @@ ATTR_COLD void INPUT_PORTS_NAME(_name)(device_t &owner, ioport_list &portlist, s
 #define PORT_16WAY \
 	configurer.field_set_way(16);
 
-#define PORT_ROTATED \
-	configurer.field_set_rotated();
-
 // general flags
 #define PORT_NAME(_name) \
 	configurer.field_set_name(_name);
@@ -1202,8 +1194,8 @@ ATTR_COLD void INPUT_PORTS_NAME(_name)(device_t &owner, ioport_list &portlist, s
 #define PORT_RESET \
 	configurer.field_set_analog_reset();
 
-#define PORT_OPTIONAL \
-	configurer.field_set_optional();
+#define PORT_GM_NOTE(_id) \
+	configurer.field_set_gm_note(_id);
 
 // analog settings
 // if this macro is not used, the minimum defaults to 0 and maximum defaults to the mask value
@@ -1222,11 +1214,10 @@ ATTR_COLD void INPUT_PORTS_NAME(_name)(device_t &owner, ioport_list &portlist, s
 #define PORT_CROSSHAIR(axis, scale, offset, altaxis) \
 	configurer.field_set_crosshair(CROSSHAIR_AXIS_##axis, altaxis, scale, offset);
 
-#define PORT_CROSSHAIR_MAPPER(_callback) \
-	configurer.field_set_crossmapper(ioport_field_crossmap_delegate(_callback, #_callback, DEVICE_SELF, (device_t *)nullptr));
-
-#define PORT_CROSSHAIR_MAPPER_MEMBER(_device, _class, _member) \
-	configurer.field_set_crossmapper(ioport_field_crossmap_delegate(&_class::_member, #_class "::" #_member, _device, (_class *)nullptr));
+#define PORT_CROSSHAIR_MAPPER_MEMBER_IMPL(_device, _funcptr, _name) \
+	configurer.field_set_crossmapper(ioport_field_crossmap_delegate(owner, _device, _funcptr, _name));
+#define PORT_CROSSHAIR_MAPPER_DEVICE_MEMBER(...) PORT_CROSSHAIR_MAPPER_MEMBER_IMPL(__VA_ARGS__)
+#define PORT_CROSSHAIR_MAPPER_MEMBER(...) PORT_CROSSHAIR_MAPPER_MEMBER_IMPL(DEVICE_SELF, __VA_ARGS__)
 
 // how many optical counts for 1 full turn of the control
 #define PORT_FULL_TURN_COUNT(_count) \
@@ -1252,48 +1243,41 @@ ATTR_COLD void INPUT_PORTS_NAME(_name)(device_t &owner, ioport_list &portlist, s
 	configurer.field_set_analog_invert();
 
 // read callbacks
-#define PORT_CUSTOM_MEMBER(_class, _member) \
-	configurer.field_set_dynamic_read(ioport_field_read_delegate(owner, DEVICE_SELF, &_class::_member, #_class "::" #_member));
-#define PORT_CUSTOM_DEVICE_MEMBER(_device, _class, _member) \
-	configurer.field_set_dynamic_read(ioport_field_read_delegate(owner, _device, &_class::_member, #_class "::" #_member));
+#define PORT_CUSTOM_MEMBER_IMPL(_device, _funcptr, _name) \
+	configurer.field_set_dynamic_read(ioport_field_read_delegate(owner, _device, _funcptr, _name));
+#define PORT_CUSTOM_DEVICE_MEMBER(...) PORT_CUSTOM_MEMBER_IMPL(__VA_ARGS__)
+#define PORT_CUSTOM_MEMBER(...) PORT_CUSTOM_MEMBER_IMPL(DEVICE_SELF, __VA_ARGS__);
 
 // write callbacks
-#define PORT_CHANGED_MEMBER(_device, _class, _member, _param) \
-	configurer.field_set_dynamic_write(ioport_field_write_delegate(owner, _device, &_class::_member, #_class "::" #_member), (_param));
+#define PORT_CHANGED_MEMBER_IMPL(_device, _funcptr, _name, _param) \
+	configurer.field_set_dynamic_write(ioport_field_write_delegate(owner, _device, _funcptr, _name), (_param));
+#define PORT_CHANGED_MEMBER(...) PORT_CHANGED_MEMBER_IMPL(__VA_ARGS__)
 
 // input device handler
-#define PORT_READ_LINE_MEMBER(_class, _member) \
-	configurer.field_set_dynamic_read( \
-			ioport_field_read_delegate( \
-				owner, \
-				DEVICE_SELF, \
-				static_cast<ioport_value (*)(_class &)>([] (_class &device) -> ioport_value { return (device._member() & 1) ? ~ioport_value(0) : 0; }), \
-				#_class "::" #_member));
-#define PORT_READ_LINE_DEVICE_MEMBER(_device, _class, _member) \
+#define PORT_READ_LINE_MEMBER_IMPL(_device, _funcptr, _name) \
 	configurer.field_set_dynamic_read( \
 			ioport_field_read_delegate( \
 				owner, \
 				_device, \
-				static_cast<ioport_value (*)(_class &)>([] (_class &device) -> ioport_value { return (device._member() & 1) ? ~ioport_value(0) : 0; }), \
-				#_class "::" #_member));
+				static_cast<ioport_value (*)(emu::detail::rw_delegate_device_class_t<decltype(_funcptr)> &)>( \
+					[] (auto &device) -> ioport_value { return (std::invoke(_funcptr, device) & 1) ? ~ioport_value(0) : 0; }), \
+				_name));
+#define PORT_READ_LINE_DEVICE_MEMBER(...) PORT_READ_LINE_MEMBER_IMPL(__VA_ARGS__)
+#define PORT_READ_LINE_MEMBER(...) PORT_READ_LINE_MEMBER_IMPL(DEVICE_SELF, __VA_ARGS__)
 
 // output device handler
-#define PORT_WRITE_LINE_MEMBER(_class, _member) \
-	configurer.field_set_dynamic_write( \
-			ioport_field_write_delegate( \
-				owner, \
-				DEVICE_SELF, \
-				static_cast<void (*)(_class &, ioport_field &, u32, ioport_value, ioport_value)>([] (_class &device, ioport_field &field, u32 param, ioport_value oldval, ioport_value newval) { device._member(newval); }), \
-				#_class "::" #_member));
-#define PORT_WRITE_LINE_DEVICE_MEMBER(_device, _class, _member) \
+#define PORT_WRITE_LINE_MEMBER_IMPL(_device, _funcptr, _name) \
 	configurer.field_set_dynamic_write( \
 			ioport_field_write_delegate( \
 				owner, \
 				_device, \
-				static_cast<void (*)(_class &, ioport_field &, u32, ioport_value, ioport_value)>([] (_class &device, ioport_field &field, u32 param, ioport_value oldval, ioport_value newval) { device._member(newval); }), \
-				#_class "::" #_member));
+				static_cast<void (*)(emu::detail::rw_delegate_device_class_t<decltype(_funcptr)> &, ioport_field &, u32, ioport_value, ioport_value)>( \
+					[] (auto &device, ioport_field &field, u32 param, ioport_value oldval, ioport_value newval) { std::invoke(_funcptr, device, newval); }), \
+				_name));
+#define PORT_WRITE_LINE_DEVICE_MEMBER(...) PORT_WRITE_LINE_MEMBER_IMPL(__VA_ARGS__)
+#define PORT_WRITE_LINE_MEMBER(...) PORT_WRITE_LINE_MEMBER_IMPL(DEVICE_SELF, __VA_ARGS__)
 
-// dip switch definition
+// DIP switch definition
 #define PORT_DIPNAME(_mask, _default, _name) \
 	configurer.field_alloc(IPT_DIPSWITCH, (_default), (_mask), (_name));
 #define PORT_DIPSETTING(_default, _name) \
@@ -1302,7 +1286,7 @@ ATTR_COLD void INPUT_PORTS_NAME(_name)(device_t &owner, ioport_list &portlist, s
 // note that these are specified LSB-first
 #define PORT_DIPLOCATION(_location) \
 	configurer.field_set_diplocation(_location);
-// conditionals for dip switch settings
+// conditionals for DIP switch settings
 #define PORT_CONDITION(_tag, _mask, _condition, _value) \
 	configurer.set_condition(ioport_condition::_condition, _tag, _mask, _value);
 // analog adjuster definition
@@ -1319,6 +1303,103 @@ ATTR_COLD void INPUT_PORTS_NAME(_name)(device_t &owner, ioport_list &portlist, s
 #define PORT_CHAR(...) \
 	configurer.field_add_char({ __VA_ARGS__ });
 
+// General-midi derived piano notes
+#define PORT_GM_A0   PORT_GM_NOTE( 21)  // Start of 88-key keyboard
+#define PORT_GM_AS0  PORT_GM_NOTE( 22)
+#define PORT_GM_B0   PORT_GM_NOTE( 23)
+
+#define PORT_GM_C1   PORT_GM_NOTE( 24)
+#define PORT_GM_CS1  PORT_GM_NOTE( 25)
+#define PORT_GM_D1   PORT_GM_NOTE( 26)
+#define PORT_GM_DS1  PORT_GM_NOTE( 27)
+#define PORT_GM_E1   PORT_GM_NOTE( 28)  // Start of 76-key keyboard
+#define PORT_GM_F1   PORT_GM_NOTE( 29)
+#define PORT_GM_FS1  PORT_GM_NOTE( 30)
+#define PORT_GM_G1   PORT_GM_NOTE( 31)
+#define PORT_GM_GS1  PORT_GM_NOTE( 32)
+#define PORT_GM_A1   PORT_GM_NOTE( 33)
+#define PORT_GM_AS1  PORT_GM_NOTE( 34)
+#define PORT_GM_B1   PORT_GM_NOTE( 35)
+
+#define PORT_GM_C2   PORT_GM_NOTE( 36)  // Start of 49 and 61-key keyboards
+#define PORT_GM_CS2  PORT_GM_NOTE( 37)
+#define PORT_GM_D2   PORT_GM_NOTE( 38)
+#define PORT_GM_DS2  PORT_GM_NOTE( 39)
+#define PORT_GM_E2   PORT_GM_NOTE( 40)
+#define PORT_GM_F2   PORT_GM_NOTE( 41)
+#define PORT_GM_FS2  PORT_GM_NOTE( 42)
+#define PORT_GM_G2   PORT_GM_NOTE( 43)
+#define PORT_GM_GS2  PORT_GM_NOTE( 44)
+#define PORT_GM_A2   PORT_GM_NOTE( 45)
+#define PORT_GM_AS2  PORT_GM_NOTE( 46)
+#define PORT_GM_B2   PORT_GM_NOTE( 47)
+
+#define PORT_GM_C3   PORT_GM_NOTE( 48)
+#define PORT_GM_CS3  PORT_GM_NOTE( 49)
+#define PORT_GM_D3   PORT_GM_NOTE( 50)
+#define PORT_GM_DS3  PORT_GM_NOTE( 51)
+#define PORT_GM_E3   PORT_GM_NOTE( 52)
+#define PORT_GM_F3   PORT_GM_NOTE( 53)
+#define PORT_GM_FS3  PORT_GM_NOTE( 54)
+#define PORT_GM_G3   PORT_GM_NOTE( 55)
+#define PORT_GM_GS3  PORT_GM_NOTE( 56)
+#define PORT_GM_A3   PORT_GM_NOTE( 57)
+#define PORT_GM_AS3  PORT_GM_NOTE( 58)
+#define PORT_GM_B3   PORT_GM_NOTE( 59)
+
+#define PORT_GM_C4   PORT_GM_NOTE( 60)  // Middle C
+#define PORT_GM_CS4  PORT_GM_NOTE( 61)
+#define PORT_GM_D4   PORT_GM_NOTE( 62)
+#define PORT_GM_DS4  PORT_GM_NOTE( 63)
+#define PORT_GM_E4   PORT_GM_NOTE( 64)
+#define PORT_GM_F4   PORT_GM_NOTE( 65)
+#define PORT_GM_FS4  PORT_GM_NOTE( 66)
+#define PORT_GM_G4   PORT_GM_NOTE( 67)
+#define PORT_GM_GS4  PORT_GM_NOTE( 68)
+#define PORT_GM_A4   PORT_GM_NOTE( 69)
+#define PORT_GM_AS4  PORT_GM_NOTE( 70)
+#define PORT_GM_B4   PORT_GM_NOTE( 71)
+
+#define PORT_GM_C5   PORT_GM_NOTE( 72)
+#define PORT_GM_CS5  PORT_GM_NOTE( 73)
+#define PORT_GM_D5   PORT_GM_NOTE( 74)
+#define PORT_GM_DS5  PORT_GM_NOTE( 75)
+#define PORT_GM_E5   PORT_GM_NOTE( 76)
+#define PORT_GM_F5   PORT_GM_NOTE( 77)
+#define PORT_GM_FS5  PORT_GM_NOTE( 78)
+#define PORT_GM_G5   PORT_GM_NOTE( 79)
+#define PORT_GM_GS5  PORT_GM_NOTE( 80)
+#define PORT_GM_A5   PORT_GM_NOTE( 81)
+#define PORT_GM_AS5  PORT_GM_NOTE( 82)
+#define PORT_GM_B5   PORT_GM_NOTE( 83)
+
+#define PORT_GM_C6   PORT_GM_NOTE( 84)  // End of 49-key keyboard
+#define PORT_GM_CS6  PORT_GM_NOTE( 85)
+#define PORT_GM_D6   PORT_GM_NOTE( 86)
+#define PORT_GM_DS6  PORT_GM_NOTE( 87)
+#define PORT_GM_E6   PORT_GM_NOTE( 88)
+#define PORT_GM_F6   PORT_GM_NOTE( 89)
+#define PORT_GM_FS6  PORT_GM_NOTE( 90)
+#define PORT_GM_G6   PORT_GM_NOTE( 91)
+#define PORT_GM_GS6  PORT_GM_NOTE( 92)
+#define PORT_GM_A6   PORT_GM_NOTE( 93)
+#define PORT_GM_AS6  PORT_GM_NOTE( 94)
+#define PORT_GM_B6   PORT_GM_NOTE( 95)
+
+#define PORT_GM_C7   PORT_GM_NOTE( 96)  // End of 61-key keyboard
+#define PORT_GM_CS7  PORT_GM_NOTE( 97)
+#define PORT_GM_D7   PORT_GM_NOTE( 98)
+#define PORT_GM_DS7  PORT_GM_NOTE( 99)
+#define PORT_GM_E7   PORT_GM_NOTE(100)
+#define PORT_GM_F7   PORT_GM_NOTE(101)
+#define PORT_GM_FS7  PORT_GM_NOTE(102)
+#define PORT_GM_G7   PORT_GM_NOTE(103)  // End of 76-key keyboard
+#define PORT_GM_GS7  PORT_GM_NOTE(104)
+#define PORT_GM_A7   PORT_GM_NOTE(105)
+#define PORT_GM_AS7  PORT_GM_NOTE(106)
+#define PORT_GM_B7   PORT_GM_NOTE(107)
+
+#define PORT_GM_C8   PORT_GM_NOTE(108)  // End of 88-key keyboard
 
 // name of table
 #define DEVICE_INPUT_DEFAULTS_NAME(_name) device_iptdef_##_name
@@ -1362,12 +1443,6 @@ ATTR_COLD void INPUT_PORTS_NAME(_name)(device_t &owner, ioport_list &portlist, s
 #define PORT_SERVICE_NO_TOGGLE(_mask, _default) \
 	PORT_BIT( _mask, _mask & _default, IPT_SERVICE ) PORT_NAME( DEF_STR( Service_Mode ))
 
-#define PORT_VBLANK(_screen) \
-	PORT_READ_LINE_DEVICE_MEMBER(_screen, screen_device, vblank)
-
-#define PORT_HBLANK(_screen) \
-	PORT_READ_LINE_DEVICE_MEMBER(_screen, screen_device, hblank)
-
 //**************************************************************************
 //  INLINE FUNCTIONS
 //**************************************************************************
