@@ -10,6 +10,10 @@
     The keyboard works entirely differently, which is a big deal.
 
     TODO: emulate expansion connector (not wholly Apple II compatible)
+    TODO: emulate joystick port (wired to KBIN and KBOUT)
+    TODO: emulate cassette MOTOR softswitch
+    TODO: emulate dual RESET keys
+    TODO: correct PAL colors
 
     $C05A - banks RAM from c100-ffff
     $C05B - banks ROM from c100-ffff
@@ -68,8 +72,8 @@ public:
 	void tk2000(machine_config &config);
 
 protected:
-	virtual void machine_start() override;
-	virtual void machine_reset() override;
+	virtual void machine_start() override ATTR_COLD;
+	virtual void machine_reset() override ATTR_COLD;
 
 private:
 	required_device<cpu_device> m_maincpu;
@@ -99,7 +103,7 @@ private:
 	uint8_t ram_r(offs_t offset);
 	void ram_w(offs_t offset, uint8_t data);
 
-	DECLARE_WRITE_LINE_MEMBER(printer_busy_w);
+	void printer_busy_w(int state);
 	void kbout_w(uint8_t data);
 	uint8_t kbin_r();
 	uint8_t casout_r();
@@ -107,19 +111,18 @@ private:
 	uint8_t snd_r();
 	void snd_w(uint8_t data);
 	uint8_t switches_r(offs_t offset);
-	uint8_t cassette_r();
-	DECLARE_WRITE_LINE_MEMBER(color_w);
-	DECLARE_WRITE_LINE_MEMBER(motor_a_w);
-	DECLARE_WRITE_LINE_MEMBER(motor_b_w);
-	DECLARE_WRITE_LINE_MEMBER(rom_ram_w);
-	DECLARE_WRITE_LINE_MEMBER(ctrl_key_w);
+	void color_w(int state);
+	void motor_a_w(int state);
+	void motor_b_w(int state);
+	void rom_ram_w(int state);
+	void ctrl_key_w(int state);
 	uint8_t c080_r(offs_t offset);
 	void c080_w(offs_t offset, uint8_t data);
 	uint8_t c100_r(offs_t offset);
 	void c100_w(offs_t offset, uint8_t data);
 
-	void apple2_map(address_map &map);
-	void inhbank_map(address_map &map);
+	void apple2_map(address_map &map) ATTR_COLD;
+	void inhbank_map(address_map &map) ATTR_COLD;
 
 	uint8_t read_floatingbus();
 };
@@ -149,6 +152,7 @@ void tk2000_state::machine_start()
 	// setup video pointers
 	m_video->set_ram_pointers(m_ram_ptr, m_ram_ptr);
 	m_video->set_char_pointer(nullptr, 0);  // no text modes on this machine
+	m_video->set_hgr2(0xa000);
 }
 
 void tk2000_state::machine_reset()
@@ -172,7 +176,7 @@ TIMER_DEVICE_CALLBACK_MEMBER(tk2000_state::apple2_interrupt)
     I/O
 ***************************************************************************/
 
-WRITE_LINE_MEMBER(tk2000_state::printer_busy_w)
+void tk2000_state::printer_busy_w(int state)
 {
 	m_printer_busy = state;
 }
@@ -202,7 +206,7 @@ uint8_t tk2000_state::kbin_r()
 	if (m_ctrl_key)
 		kbin |= m_kbspecial->read();
 
-	return kbin | (m_printer_busy ? 0x40 : 0);
+	return kbin | (m_printer_busy ? 0x40 : 0) | (m_cassette->input() > 0.0 ? 0x80 : 0);
 }
 
 uint8_t tk2000_state::casout_r()
@@ -233,36 +237,33 @@ void tk2000_state::snd_w(uint8_t data)
 
 uint8_t tk2000_state::switches_r(offs_t offset)
 {
+	const uint8_t uFloatingBus = read_floatingbus(); // video side-effects latch after reading
 	if (!machine().side_effects_disabled())
 		m_softlatch->write_bit((offset & 0x0e) >> 1, offset & 0x01);
-	return read_floatingbus();
+	return uFloatingBus;
 }
 
-uint8_t tk2000_state::cassette_r()
-{
-	return (m_cassette->input() > 0.0 ? 0x80 : 0) | (read_floatingbus() & 0x7f);
-}
-
-WRITE_LINE_MEMBER(tk2000_state::color_w)
+void tk2000_state::color_w(int state)
 {
 	// 0 = color, 1 = black/white
+	m_video->monohgr_w(state);
 }
 
-WRITE_LINE_MEMBER(tk2000_state::motor_a_w)
+void tk2000_state::motor_a_w(int state)
 {
 }
 
-WRITE_LINE_MEMBER(tk2000_state::motor_b_w)
+void tk2000_state::motor_b_w(int state)
 {
 }
 
-WRITE_LINE_MEMBER(tk2000_state::rom_ram_w)
+void tk2000_state::rom_ram_w(int state)
 {
 	// 0 = ROM, 1 = RAM
 	m_upperbank->set_bank(state);
 }
 
-WRITE_LINE_MEMBER(tk2000_state::ctrl_key_w)
+void tk2000_state::ctrl_key_w(int state)
 {
 	m_ctrl_key = state;
 }
@@ -398,8 +399,8 @@ uint8_t tk2000_state::read_floatingbus()
 		// Y: insert hires only address bits
 		//
 		address |= v_C << 12; // a12
-		address |= (1 ^ (Page2 & (1 ^ _80Store))) << 13; // a13
-		address |= (Page2 & (1 ^ _80Store)) << 14; // a14
+		address |= 1 << 13; // a13
+		address |= (Page2 & (1 ^ _80Store)) << 15; // a15, from a000
 	}
 	else
 	{
@@ -446,7 +447,6 @@ void tk2000_state::apple2_map(address_map &map)
 	map(0xc020, 0xc020).rw(FUNC(tk2000_state::casout_r), FUNC(tk2000_state::casout_w));
 	map(0xc030, 0xc030).rw(FUNC(tk2000_state::snd_r), FUNC(tk2000_state::snd_w));
 	map(0xc050, 0xc05f).r(FUNC(tk2000_state::switches_r)).w(m_softlatch, FUNC(addressable_latch_device::write_a0));
-	map(0xc060, 0xc060).mirror(8).r(FUNC(tk2000_state::cassette_r));
 	map(0xc080, 0xc0ff).rw(FUNC(tk2000_state::c080_r), FUNC(tk2000_state::c080_w));
 	map(0xc100, 0xffff).m(m_upperbank, FUNC(address_map_bank_device::amap8));
 }
@@ -619,5 +619,5 @@ ROM_END
 
 
 /*    YEAR  NAME    PARENT  COMPAT  MACHINE  INPUT   CLASS         INIT        COMPANY         FULLNAME */
-COMP( 1984, tk2000, 0,      0,      tk2000,  tk2000, tk2000_state, empty_init, "Microdigital", "TK2000 Color Computer", MACHINE_NOT_WORKING )
-COMP( 1982, mpf2,   tk2000, 0,      tk2000,  tk2000, tk2000_state, empty_init, "Multitech",    "Microprofessor II",     MACHINE_NOT_WORKING )
+COMP( 1984, tk2000, 0,      0,      tk2000,  tk2000, tk2000_state, empty_init, "Microdigital", "TK2000 Color Computer", MACHINE_IMPERFECT_COLORS | MACHINE_SUPPORTS_SAVE )
+COMP( 1982, mpf2,   tk2000, 0,      tk2000,  tk2000, tk2000_state, empty_init, "Multitech",    "Microprofessor II",     MACHINE_IMPERFECT_COLORS | MACHINE_SUPPORTS_SAVE )
