@@ -125,12 +125,12 @@
 #include "machine/rescap.h"
 #include "machine/watchdog.h"
 #include "sound/pokey.h"
-#include "video/resnet.h"
 
 #include "emupal.h"
 #include "screen.h"
 #include "speaker.h"
 #include "tilemap.h"
+#include "video/resnet.h"
 
 
 namespace {
@@ -138,8 +138,8 @@ namespace {
 class cloak_state : public driver_device
 {
 public:
-	cloak_state(const machine_config &mconfig, device_type type, const char *tag)
-		: driver_device(mconfig, type, tag),
+	cloak_state(const machine_config &mconfig, device_type type, const char *tag) :
+		driver_device(mconfig, type, tag),
 		m_maincpu(*this, "maincpu"),
 		m_slave(*this, "slave"),
 		m_gfxdecode(*this, "gfxdecode"),
@@ -153,7 +153,7 @@ public:
 	void cloak(machine_config &config);
 
 protected:
-	virtual void video_start() override;
+	virtual void video_start() override ATTR_COLD;
 
 private:
 	required_device<cpu_device> m_maincpu;
@@ -172,7 +172,7 @@ private:
 	uint8_t m_bitmap_videoram_address_y = 0;
 	tilemap_t *m_bg_tilemap = nullptr;
 
-	template <uint8_t Which> DECLARE_WRITE_LINE_MEMBER(coin_counter_w);
+	template <uint8_t Which> void coin_counter_w(int state);
 	void custom_w(uint8_t data);
 	void irq_reset_0_w(uint8_t data);
 	void irq_reset_1_w(uint8_t data);
@@ -188,13 +188,11 @@ private:
 	void set_pen(int i);
 	void draw_bitmap(bitmap_ind16 &bitmap, const rectangle &cliprect);
 	void draw_sprites(bitmap_ind16 &bitmap, const rectangle &cliprect);
-	void master_map(address_map &map);
-	void slave_map(address_map &map);
+	void master_map(address_map &map) ATTR_COLD;
+	void slave_map(address_map &map) ATTR_COLD;
 };
 
 
-// video
-
 /***************************************************************************
 
   CLOAK & DAGGER uses RAM to dynamically
@@ -381,8 +379,6 @@ uint32_t cloak_state::screen_update(screen_device &screen, bitmap_ind16 &bitmap,
 }
 
 
-// machine
-
 /*************************************
  *
  *  Output ports
@@ -390,7 +386,7 @@ uint32_t cloak_state::screen_update(screen_device &screen, bitmap_ind16 &bitmap,
  *************************************/
 
 template <uint8_t Which>
-WRITE_LINE_MEMBER(cloak_state::coin_counter_w)
+void cloak_state::coin_counter_w(int state)
 {
 	machine().bookkeeping().coin_counter_w(Which, state);
 }
@@ -486,7 +482,7 @@ static INPUT_PORTS_START( cloak )
 	PORT_BIT( 0xff, IP_ACTIVE_LOW, IPT_UNUSED )     // player 2 controls, not used
 
 	PORT_START("SYSTEM")
-	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_CUSTOM ) PORT_VBLANK("screen")
+	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_CUSTOM ) PORT_READ_LINE_DEVICE_MEMBER("screen", FUNC(screen_device::vblank))
 	PORT_SERVICE( 0x02, IP_ACTIVE_LOW )
 	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_COIN2 )
 	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_COIN1 )
