@@ -42,17 +42,266 @@ Year + Game             Main CPU  Sound CPU  Sound                         Video
 ***************************************************************************************/
 
 #include "emu.h"
-#include "fantland.h"
 
 #include "cpu/i86/i86.h"
 #include "cpu/nec/nec.h"
 #include "cpu/z80/z80.h"
+#include "machine/gen_latch.h"
 #include "sound/dac.h"
+#include "sound/msm5205.h"
 #include "sound/sn76496.h"
-#include "sound/ymopm.h"
 #include "sound/ymopl.h"
+#include "sound/ymopm.h"
+
+#include "emupal.h"
+#include "screen.h"
 #include "speaker.h"
 
+namespace {
+
+class fantland_state : public driver_device
+{
+public:
+	fantland_state(const machine_config &mconfig, device_type type, const char *tag) :
+		driver_device(mconfig, type, tag),
+		m_maincpu(*this, "maincpu"),
+		m_audiocpu(*this, "audiocpu"),
+		m_gfxdecode(*this, "gfxdecode"),
+		m_screen(*this, "screen"),
+		m_palette(*this, "palette"),
+		m_soundlatch(*this, "soundlatch"),
+		m_spriteram(*this, "spriteram", 0x2800, ENDIANNESS_LITTLE),
+		m_spriteram2(*this, "spriteram2", 0x10000, ENDIANNESS_LITTLE),
+		m_wheel(*this, "WHEEL%u", 0U)
+	{ }
+
+	void fantland(machine_config &config) ATTR_COLD;
+	void wheelrun(machine_config &config) ATTR_COLD;
+	void galaxygn(machine_config &config) ATTR_COLD;
+
+	template <int Player> ioport_value wheelrun_wheel_r();
+
+protected:
+	/* devices */
+	required_device<cpu_device> m_maincpu;
+	required_device<cpu_device> m_audiocpu;
+	required_device<gfxdecode_device> m_gfxdecode;
+	required_device<screen_device> m_screen;
+	required_device<palette_device> m_palette;
+	required_device<generic_latch_8_device> m_soundlatch;
+
+	/* memory pointers */
+	memory_share_creator<uint8_t> m_spriteram;
+	memory_share_creator<uint8_t> m_spriteram2;
+
+	optional_ioport_array<2> m_wheel;
+
+	/* misc */
+	uint8_t m_nmi_enable = 0;
+
+	virtual void machine_start() override ATTR_COLD;
+	virtual void machine_reset() override ATTR_COLD;
+
+	void nmi_enable_w(uint8_t data);
+	void soundlatch_w(uint8_t data);
+	uint32_t screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect);
+	void vblank_irq(int state);
+	void draw_sprites(bitmap_ind16 &bitmap,const rectangle &cliprect);
+
+private:
+	uint8_t spriteram_r(offs_t offset);
+	uint8_t spriteram2_r(offs_t offset);
+	void spriteram_w(offs_t offset, uint8_t data, uint8_t mem_mask = ~0);
+	void spriteram2_w(offs_t offset, uint8_t data, uint8_t mem_mask = ~0);
+	void galaxygn_sound_irq(int state);
+	INTERRUPT_GEN_MEMBER(fantland_sound_irq);
+	void fantland_map(address_map &map) ATTR_COLD;
+	void fantland_sound_iomap(address_map &map) ATTR_COLD;
+	void fantland_sound_map(address_map &map) ATTR_COLD;
+	void galaxygn_map(address_map &map) ATTR_COLD;
+	void galaxygn_sound_iomap(address_map &map) ATTR_COLD;
+	void wheelrun_map(address_map &map) ATTR_COLD;
+	void wheelrun_sound_map(address_map &map) ATTR_COLD;
+};
+
+class borntofi_state : public fantland_state
+{
+public:
+	borntofi_state(const machine_config &mconfig, device_type type, const char *tag) :
+		fantland_state(mconfig, type, tag),
+		m_msm(*this, "msm%u", 1U),
+		m_adpcm_rom(*this, "adpcm")
+	{
+	}
+
+	void borntofi(machine_config &config);
+
+protected:
+	virtual void machine_start() override ATTR_COLD;
+	virtual void machine_reset() override ATTR_COLD;
+
+private:
+	/* devices */
+	required_device_array<msm5205_device, 4> m_msm;
+	required_region_ptr<uint8_t> m_adpcm_rom;
+
+	/* misc */
+	int        m_old_x[2]{};
+	int        m_old_y[2]{};
+	int        m_old_f[2]{};
+	uint8_t    m_input_ret[2]{};
+	int        m_adpcm_playing[4]{};
+	int        m_adpcm_addr[2][4]{};
+	int        m_adpcm_nibble[4]{};
+
+	uint8_t inputs_r(offs_t offset);
+	void msm5205_w(offs_t offset, uint8_t data);
+	template<int Voice> void adpcm_int(int state);
+	void adpcm_start(int voice);
+	void adpcm_stop(int voice);
+	void main_map(address_map &map) ATTR_COLD;
+	void sound_map(address_map &map) ATTR_COLD;
+};
+
+/***************************************************************************************
+
+    These games has sprites only:
+
+    tiles are 16 x 16 x 6. There are 0x400 sprites, each one is allotted
+    8 bytes of memory (but only 5 are used) in spriteram (0x54000):
+
+    Offset:     Bits:           Value:
+
+        0                       X (low bits)
+
+        1       7--- ----       X (high bit)
+                -6-- ----       Y (high bit)
+                --5- ----       Flip X
+                ---4 ----       Flip Y
+                ---- 32--
+                ---- --10       Color
+
+        2                       Code (high bits)
+
+        3                       Code (low bits)
+
+        4                       Y (low bits)
+
+    Then 2 tables follow, 0x400 bytes each:
+
+    - the first table  (0x56000) contains 1 byte per sprite: an index in the second table
+    - the second table (0x56400) is either an x,y offset or an index in spriteram_2 (0x60000):
+
+        0                       X offset (low bits)
+
+        1                       Y offset (low bits)
+
+        2       7--- ----       If 1, the following bits are an index in spriteram_2 for the real X&Y & Code offsets
+                -654 321-
+                ---- ---0       X offset (high bit)
+
+        3       7654 321-
+                ---- ---0       Y offset (high bit)
+
+
+    Spriteram_2 contains 0x4000 X&Y & Code offsets:
+
+        0                       Y offset (low bits)
+
+        1       7--- ----       Flip X (xor with that in spriteram)
+                -6-- ----       Flip Y ""
+                --54 321-       Code offset
+                ---- ---0       Y offset (high bit)
+
+        2                       X offset (low bits)
+
+        3                       X offset (high bit)
+
+***************************************************************************************/
+
+void fantland_state::draw_sprites(bitmap_ind16 &bitmap,const rectangle &cliprect)
+{
+	uint8_t *indx_ram   =   m_spriteram + 0x2000,    // this ram contains indexes into offs_ram
+			*offs_ram   =   m_spriteram + 0x2400,    // this ram contains x,y offsets or indexes into m_spriteram2
+			*ram        =   m_spriteram,         // current sprite pointer in spriteram
+			*ram2       =   indx_ram;           // current sprite pointer in indx_ram
+
+	// wheelrun is the only game with a smaller visible area
+	const rectangle &visarea = m_screen->visible_area();
+	int special = visarea.height() < 0x100;
+
+	for ( ; ram < indx_ram; ram += 8,ram2++)
+	{
+		int attr,code,color, x,y,xoffs,yoffs,flipx,flipy, idx;
+
+		attr    =   ram[1];
+
+		x       =   ram[0];
+		code    =   ram[3] + (ram[2] << 8);
+		y       =   ram[4];
+
+		color   =   (attr & 0x03);
+		flipy   =   (attr & 0x10) ? 1 : 0;
+		flipx   =   (attr & 0x20) ? 1 : 0;
+
+		y       +=  (attr & 0x40) << 2;
+		x       +=  (attr & 0x80) << 1;
+
+		// Index in the table of offsets
+
+		idx     =   ram2[0] * 4;
+
+		// Fetch the offsets
+
+		if (offs_ram[idx + 2] & 0x80)
+		{
+			// x,y & code offset is in m_spriteram2, this is its index
+
+			idx     =   (((offs_ram[idx + 2] << 8) + offs_ram[idx + 3]) & 0x3fff) * 4;
+
+			yoffs   =   m_spriteram2[idx + 0] + (m_spriteram2[idx + 1] << 8);
+			xoffs   =   m_spriteram2[idx + 2] + (m_spriteram2[idx + 3] << 8);
+
+			code    +=  (yoffs & 0x3e00) >> 9;
+			flipy   ^=  (yoffs & 0x4000) ? 1 : 0;
+			flipx   ^=  (yoffs & 0x8000) ? 1 : 0;
+		}
+		else
+		{
+			// this is an x,y offset
+
+			yoffs   =   ((offs_ram[idx + 3] & 0x01) << 8) + offs_ram[idx + 1];
+			xoffs   =   ((offs_ram[idx + 2] & 0x01) << 8) + offs_ram[idx + 0];
+		}
+
+		yoffs   =   (yoffs & 0xff) - (yoffs & 0x100);
+		xoffs   =   (xoffs & 0x1ff);
+
+		if (xoffs >= 0x180)     xoffs -= 0x200;
+
+		y       +=  yoffs;
+		x       +=  xoffs;
+
+		// wheelrun needs y=0xf0 & yoffs=0x50 to be rendered at screen y 0x40
+		if (special && y > 0)
+			y &= 0xff;
+
+		y       =   (y & 0xff) - (y & 0x100);
+		x       =   (x & 0x1ff);
+
+		if (x >= 0x180)     x -= 0x200;
+
+		m_gfxdecode->gfx(0)->transpen(bitmap,cliprect, code,color, flipx,flipy, x,y,0);
+	}
+}
+
+uint32_t fantland_state::screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect)
+{
+	bitmap.fill(0, cliprect);
+	draw_sprites(bitmap,cliprect);
+
+	return 0;
+}
 
 /***************************************************************************
 
@@ -334,7 +583,7 @@ void borntofi_state::msm5205_w(offs_t offset, uint8_t data)
 }
 
 template<int Voice>
-WRITE_LINE_MEMBER(borntofi_state::adpcm_int)
+void borntofi_state::adpcm_int(int state)
 {
 	if (!m_adpcm_playing[Voice])
 		return;
@@ -686,7 +935,7 @@ INPUT_PORTS_END
 ***************************************************************************/
 
 template <int Player>
-CUSTOM_INPUT_MEMBER(fantland_state::wheelrun_wheel_r)
+ioport_value fantland_state::wheelrun_wheel_r()
 {
 	int delta = m_wheel[Player]->read();
 	delta = (delta & 0x7f) - (delta & 0x80) + 4;
@@ -704,7 +953,7 @@ static INPUT_PORTS_START( wheelrun )
 	PORT_BIT( 0x02, IP_ACTIVE_LOW,  IPT_BUTTON1 ) PORT_PLAYER(1)
 	PORT_BIT( 0x04, IP_ACTIVE_LOW,  IPT_UNKNOWN )
 	PORT_BIT( 0x08, IP_ACTIVE_LOW,  IPT_UNKNOWN )
-	PORT_BIT( 0x70, IP_ACTIVE_HIGH, IPT_CUSTOM ) PORT_CUSTOM_MEMBER(fantland_state, wheelrun_wheel_r<0>)
+	PORT_BIT( 0x70, IP_ACTIVE_HIGH, IPT_CUSTOM ) PORT_CUSTOM_MEMBER(FUNC(fantland_state::wheelrun_wheel_r<0>))
 	PORT_BIT( 0x80, IP_ACTIVE_LOW,  IPT_UNKNOWN )
 
 	PORT_START("53001")     // 53001
@@ -712,7 +961,7 @@ static INPUT_PORTS_START( wheelrun )
 	PORT_BIT( 0x02, IP_ACTIVE_LOW,  IPT_BUTTON1 ) PORT_PLAYER(2)
 	PORT_BIT( 0x04, IP_ACTIVE_LOW,  IPT_UNKNOWN )
 	PORT_BIT( 0x08, IP_ACTIVE_LOW,  IPT_UNKNOWN )
-	PORT_BIT( 0x70, IP_ACTIVE_HIGH, IPT_CUSTOM ) PORT_CUSTOM_MEMBER(fantland_state, wheelrun_wheel_r<1>)
+	PORT_BIT( 0x70, IP_ACTIVE_HIGH, IPT_CUSTOM ) PORT_CUSTOM_MEMBER(FUNC(fantland_state::wheelrun_wheel_r<1>))
 	PORT_BIT( 0x80, IP_ACTIVE_LOW,  IPT_UNKNOWN )
 
 	PORT_START("53002")     // 53002
@@ -793,7 +1042,7 @@ void fantland_state::machine_reset()
 	m_nmi_enable = 0;
 }
 
-WRITE_LINE_MEMBER(fantland_state::vblank_irq)
+void fantland_state::vblank_irq(int state)
 {
 	if (state && BIT(m_nmi_enable, 3))
 		m_maincpu->pulse_input_line(INPUT_LINE_NMI, attotime::zero);
@@ -842,7 +1091,7 @@ void fantland_state::fantland(machine_config &config)
 }
 
 
-WRITE_LINE_MEMBER(fantland_state::galaxygn_sound_irq)
+void fantland_state::galaxygn_sound_irq(int state)
 {
 	m_audiocpu->set_input_line_and_vector(0, state ? ASSERT_LINE : CLEAR_LINE, 0x80/4); // I8088
 }
@@ -1545,6 +1794,7 @@ ROM_START( wheelrun )
 	ROM_LOAD( "pal20l8acns.pal12",     0x000, 0x144, NO_DUMP )
 ROM_END
 
+} // anonymous namespace
 
 GAME( 19??, borntofi,  0,        borntofi, borntofi, borntofi_state, empty_init, ROT0,  "International Games",                  "Born To Fight (set 1)",           MACHINE_SUPPORTS_SAVE )
 GAME( 19??, borntofia, borntofi, borntofi, borntofi, borntofi_state, empty_init, ROT0,  "International Games",                  "Born To Fight (set 2)",           MACHINE_SUPPORTS_SAVE )
