@@ -1,9 +1,16 @@
 // license:BSD-3-Clause
 // copyright-holders:Olivier Galibert
-/***************************************************************************
+/*******************************************************************************
 
     G.I. Joe  (c) 1992 Konami
 
+TODO:
+- sprite gaps (K053247 zoom fraction rounding)
+- shadow masking (eg. the shadow of Baroness' aircraft should not project on the sky)
+- improve K056832 linemap emulation, it shouldn't (ab)use the tilemap pixmap as a
+  render buffer, gijoe is the only game that uses it
+
+********************************************************************************
 
 G.I. Joe
 Konami 1992
@@ -71,16 +78,7 @@ Notes:
       HSync - 15.2036kHz
       VSync - 59.6374Hz
 
-
-****************************************************************************
-
-Known Issues
-------------
-
-- sprite gaps (K053247 zoom fraction rounding)
-- shadow masking (eg. the shadow of Baroness' aircraft should not project on the sky)
-
-***************************************************************************/
+*******************************************************************************/
 
 #include "emu.h"
 
@@ -95,6 +93,7 @@ Known Issues
 #include "machine/eepromser.h"
 #include "machine/k054321.h"
 #include "sound/k054539.h"
+
 #include "emupal.h"
 #include "speaker.h"
 
@@ -124,24 +123,29 @@ public:
 
 	void gijoe(machine_config &config);
 
+protected:
+	virtual void machine_start() override ATTR_COLD;
+	virtual void machine_reset() override ATTR_COLD;
+	virtual void video_start() override ATTR_COLD;
+
 private:
-	/* memory pointers */
+	// memory pointers
 	required_shared_ptr<uint16_t> m_spriteram;
 	required_shared_ptr<uint16_t> m_workram;
 
-	/* video-related */
-	int         m_avac_bits[4]{};
-	int         m_avac_occupancy[4]{};
-	int         m_layer_colorbase[4]{};
-	int         m_layer_pri[4]{};
-	int         m_avac_vrc = 0;
-	int         m_sprite_colorbase = 0;
+	// video-related
+	int32_t     m_avac_bits[4]{};
+	int32_t     m_avac_occupancy[4]{};
+	uint16_t    m_layer_colorbase[4]{};
+	int32_t     m_layer_pri[4]{};
+	int32_t     m_avac_vrc = 0;
+	uint16_t    m_sprite_colorbase = 0;
 
-	/* misc */
-	uint16_t      m_cur_control2 = 0U;
+	// misc
+	uint16_t    m_cur_control2 = 0U;
 	emu_timer   *m_dmadelay_timer = nullptr;
 
-	/* devices */
+	// devices
 	required_device<cpu_device> m_maincpu;
 	required_device<cpu_device> m_audiocpu;
 	required_device<k054539_device> m_k054539;
@@ -154,41 +158,38 @@ private:
 	uint16_t control2_r();
 	void control2_w(offs_t offset, uint16_t data, uint16_t mem_mask = ~0);
 	void sound_irq_w(uint16_t data);
-	virtual void machine_start() override ATTR_COLD;
-	virtual void machine_reset() override ATTR_COLD;
-	virtual void video_start() override ATTR_COLD;
-	uint32_t screen_update_gijoe(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect);
-	INTERRUPT_GEN_MEMBER(gijoe_interrupt);
+	uint32_t screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect);
+	INTERRUPT_GEN_MEMBER(interrupt);
 	TIMER_CALLBACK_MEMBER(dmaend_callback);
-	void gijoe_objdma();
+	void object_dma();
 	K056832_CB_MEMBER(tile_callback);
 	K053246_CB_MEMBER(sprite_callback);
-	void gijoe_map(address_map &map) ATTR_COLD;
+	void main_map(address_map &map) ATTR_COLD;
 	void sound_map(address_map &map) ATTR_COLD;
 };
 
 
 K053246_CB_MEMBER(gijoe_state::sprite_callback)
 {
-	int pri = (*color & 0x03e0) >> 4;
+	int pri = (color & 0x03e0) >> 4;
 
 	if (pri <= m_layer_pri[3])
-		*priority_mask = 0;
+		priority_mask = 0;
 	else if (pri >  m_layer_pri[3] && pri <= m_layer_pri[2])
-		*priority_mask = 0xff00;
+		priority_mask = 0xff00;
 	else if (pri >  m_layer_pri[2] && pri <= m_layer_pri[1])
-		*priority_mask = 0xff00 | 0xf0f0;
+		priority_mask = 0xff00 | 0xf0f0;
 	else if (pri >  m_layer_pri[1] && pri <= m_layer_pri[0])
-		*priority_mask = 0xff00 | 0xf0f0 | 0xcccc;
+		priority_mask = 0xff00 | 0xf0f0 | 0xcccc;
 	else
-		*priority_mask = 0xff00 | 0xf0f0 | 0xcccc | 0xaaaa;
+		priority_mask = 0xff00 | 0xf0f0 | 0xcccc | 0xaaaa;
 
-	*color = m_sprite_colorbase | (*color & 0x001f);
+	color = m_sprite_colorbase | (color & 0x001f);
 }
 
 K056832_CB_MEMBER(gijoe_state::tile_callback)
 {
-	int tile = *code;
+	int tile = code;
 
 	if (tile >= 0xf000 && tile <= 0xf4ff)
 	{
@@ -208,19 +209,15 @@ K056832_CB_MEMBER(gijoe_state::tile_callback)
 			m_avac_occupancy[layer] |= 0x00f0;
 			tile |= m_avac_bits[2];
 		}
-		*code = tile;
+		code = tile;
 	}
 
-	*color = (*color >> 2 & 0x0f) | m_layer_colorbase[layer];
+	color = (color >> 2 & 0x0f) | m_layer_colorbase[layer];
 }
 
 void gijoe_state::video_start()
 {
-	int i;
-
-	m_k056832->linemap_enable(1);
-
-	for (i = 0; i < 4; i++)
+	for (int i = 0; i < 4; i++)
 	{
 		m_avac_occupancy[i] = 0;
 		m_avac_bits[i] = 0;
@@ -233,20 +230,20 @@ void gijoe_state::video_start()
 	save_item(NAME(m_avac_vrc));
 	save_item(NAME(m_sprite_colorbase));
 	save_item(NAME(m_avac_occupancy));
-	save_item(NAME(m_avac_bits));   // these could possibly be re-created at postload k056832 elements
+	save_item(NAME(m_avac_bits)); // these could possibly be re-created at postload k056832 elements
 	save_item(NAME(m_layer_colorbase));
 	save_item(NAME(m_layer_pri));
 }
 
-uint32_t gijoe_state::screen_update_gijoe(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect)
+uint32_t gijoe_state::screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect)
 {
 	static const int K053251_CI[4] = { k053251_device::CI1, k053251_device::CI2, k053251_device::CI3, k053251_device::CI4 };
 	int layer[4];
-	int vrc_mode, vrc_new, colorbase_new, /*primode,*/ dirty, i;
+	int vrc_mode, vrc_new, colorbase_new, /*primode,*/ dirty;
 	int mask = 0;
 
 	// update tile offsets
-	m_k056832->read_avac(&vrc_mode, &vrc_new);
+	m_k056832->read_avac(vrc_mode, vrc_new);
 
 	if (vrc_mode)
 	{
@@ -266,7 +263,7 @@ uint32_t gijoe_state::screen_update_gijoe(screen_device &screen, bitmap_ind16 &b
 	// update color info and refresh tilemaps
 	m_sprite_colorbase = m_k053251->get_palette_index(k053251_device::CI0);
 
-	for (i = 0; i < 4; i++)
+	for (int i = 0; i < 4; i++)
 	{
 		dirty = 0;
 		colorbase_new = m_k053251->get_palette_index(K053251_CI[i]);
@@ -291,7 +288,7 @@ uint32_t gijoe_state::screen_update_gijoe(screen_device &screen, bitmap_ind16 &b
 	    written to the layer's X-scroll register otherwise the chip expects totally
 	    different alignment values.
 	*/
-	if (m_k056832->read_register(0x14) == 2)
+	if (m_k056832->word_r(0x14) == 2)
 	{
 		m_k056832->set_layer_offs(0,  2, 0);
 		m_k056832->set_layer_offs(1,  4, 0);
@@ -328,7 +325,7 @@ uint32_t gijoe_state::screen_update_gijoe(screen_device &screen, bitmap_ind16 &b
 	m_k056832->tilemap_draw(screen, bitmap, cliprect, layer[2], 0, 4);
 	m_k056832->tilemap_draw(screen, bitmap, cliprect, layer[3], 0, 8);
 
-	m_k053246->k053247_sprites_draw( bitmap, cliprect);
+	m_k053246->k053247_sprites_draw(bitmap, cliprect);
 	return 0;
 }
 
@@ -342,28 +339,29 @@ void gijoe_state::control2_w(offs_t offset, uint16_t data, uint16_t mem_mask)
 {
 	if (ACCESSING_BITS_0_7)
 	{
-		/* bit 0  is data */
-		/* bit 1  is cs (active low) */
-		/* bit 2  is clock (active high) */
-		/* bit 3  (unknown: coin) */
-		/* bit 5  is enable irq 6 */
-		/* bit 7  (unknown: enable irq 5?) */
+		// bit 0 = data
+		// bit 1 = cs (active low)
+		// bit 2 = clock (active high)
+		// bit 3 = (unknown: coin)
+		// bit 5 = enable irq 6
+		// bit 7 = (unknown: enable irq 5?)
 		ioport("EEPROMOUT")->write(data, 0xff);
 
 		m_cur_control2 = data;
 
-		/* bit 6 = enable sprite ROM reading */
-		m_k053246->k053246_set_objcha_line( (data & 0x0040) ? ASSERT_LINE : CLEAR_LINE);
+		// bit 6 = enable sprite ROM reading
+		m_k053246->k053246_set_objcha_line((data & 0x0040) ? ASSERT_LINE : CLEAR_LINE);
 	}
 }
 
-void gijoe_state::gijoe_objdma(  )
+void gijoe_state::object_dma()
 {
+	// TODO: implement sprite dma in k053246_k053247_k055673.cpp
 	uint16_t *src_head, *src_tail, *dst_head, *dst_tail;
 
 	src_head = m_spriteram;
 	src_tail = m_spriteram + 255 * 8;
-	m_k053246->k053247_get_ram( &dst_head);
+	m_k053246->k053247_get_ram(&dst_head);
 	dst_tail = dst_head + 255 * 8;
 
 	for (; src_head <= src_tail; src_head += 8)
@@ -387,7 +385,7 @@ TIMER_CALLBACK_MEMBER(gijoe_state::dmaend_callback)
 		m_maincpu->set_input_line(6, HOLD_LINE);
 }
 
-INTERRUPT_GEN_MEMBER(gijoe_state::gijoe_interrupt)
+INTERRUPT_GEN_MEMBER(gijoe_state::interrupt)
 {
 	// global interrupt masking (*this game only)
 	if (!m_k056832->is_irq_enabled(0))
@@ -395,7 +393,7 @@ INTERRUPT_GEN_MEMBER(gijoe_state::gijoe_interrupt)
 
 	if (m_k053246->k053246_is_irq_enabled())
 	{
-		gijoe_objdma();
+		object_dma();
 
 		// 42.7us(clr) + 341.3us(xfer) delay at 6Mhz dotclock
 		m_dmadelay_timer->adjust(JOE_DMADELAY);
@@ -411,7 +409,7 @@ void gijoe_state::sound_irq_w(uint16_t data)
 	m_audiocpu->set_input_line(0, HOLD_LINE);
 }
 
-void gijoe_state::gijoe_map(address_map &map)
+void gijoe_state::main_map(address_map &map)
 {
 	map(0x000000, 0x0fffff).rom();
 	map(0x100000, 0x100fff).ram().share("spriteram");                               // Sprites
@@ -475,22 +473,22 @@ static INPUT_PORTS_START( gijoe )
 	PORT_BIT( 0x0800, IP_ACTIVE_LOW,  IPT_SERVICE4 )
 
 	PORT_START("P1_P2")
-	KONAMI16_LSB_40(1, IPT_BUTTON3 ) PORT_OPTIONAL
+	KONAMI16_LSB_40(1, IPT_BUTTON3 )
 	PORT_DIPNAME( 0x0080, 0x0000, "Sound" )         PORT_DIPLOCATION("SW1:1")
 	PORT_DIPSETTING(      0x0080, DEF_STR( Mono ) )
 	PORT_DIPSETTING(      0x0000, DEF_STR( Stereo ) )
-	KONAMI16_MSB_40(2, IPT_BUTTON3 ) PORT_OPTIONAL
+	KONAMI16_MSB_40(2, IPT_BUTTON3 )
 	PORT_DIPNAME( 0x8000, 0x8000, "Coin mechanism" )    PORT_DIPLOCATION("SW1:2")
 	PORT_DIPSETTING(      0x8000, "Common" )
 	PORT_DIPSETTING(      0x0000, "Independent" )
 
 	PORT_START("P3_P4")
-	KONAMI16_LSB_40(3, IPT_BUTTON3 ) PORT_OPTIONAL
+	KONAMI16_LSB_40(3, IPT_BUTTON3 )
 	PORT_DIPNAME( 0x0080, 0x0080, DEF_STR( Players ) )  PORT_DIPLOCATION("SW1:3")
 	PORT_DIPSETTING(      0x0080, "2" )
 	PORT_DIPSETTING(      0x0000, "4" )
-	KONAMI16_MSB_40(4, IPT_BUTTON3 ) PORT_OPTIONAL
-	PORT_DIPUNUSED_DIPLOC( 0x8000, 0x8000, "SW1:4" )    /* Listed as "Unused" */
+	KONAMI16_MSB_40(4, IPT_BUTTON3 )
+	PORT_DIPUNUSED_DIPLOC( 0x8000, 0x8000, "SW1:4" )    // Listed as "Unused"
 INPUT_PORTS_END
 
 void gijoe_state::machine_start()
@@ -507,24 +505,20 @@ void gijoe_state::machine_reset()
 
 void gijoe_state::gijoe(machine_config &config)
 {
-	/* basic machine hardware */
-	M68000(config, m_maincpu, XTAL(32'000'000)/2);   /* 16MHz Confirmed */
-	m_maincpu->set_addrmap(AS_PROGRAM, &gijoe_state::gijoe_map);
-	m_maincpu->set_vblank_int("screen", FUNC(gijoe_state::gijoe_interrupt));
+	// basic machine hardware
+	M68000(config, m_maincpu, 32_MHz_XTAL / 2); // 16MHz Confirmed
+	m_maincpu->set_addrmap(AS_PROGRAM, &gijoe_state::main_map);
+	m_maincpu->set_vblank_int("screen", FUNC(gijoe_state::interrupt));
 
-	Z80(config, m_audiocpu, XTAL(32'000'000)/4);     /* Amuse & confirmed. Z80E at 8MHz */
+	Z80(config, m_audiocpu, 32_MHz_XTAL / 4); // Amuse & confirmed. Z80E at 8MHz
 	m_audiocpu->set_addrmap(AS_PROGRAM, &gijoe_state::sound_map);
 
 	EEPROM_ER5911_8BIT(config, "eeprom");
 
-	/* video hardware */
+	// video hardware
 	screen_device &screen(SCREEN(config, "screen", SCREEN_TYPE_RASTER));
-	screen.set_video_attributes(VIDEO_UPDATE_BEFORE_VBLANK);
-	screen.set_refresh_hz(60);
-	screen.set_vblank_time(ATTOSECONDS_IN_USEC(0));
-	screen.set_size(64*8, 32*8);
-	screen.set_visarea(24, 24+288-1, 16, 16+224-1);
-	screen.set_screen_update(FUNC(gijoe_state::screen_update_gijoe));
+	screen.set_raw(24_MHz_XTAL / 4, 384, 24, 312, 262, 16, 240); // measured 59.637Hz
+	screen.set_screen_update(FUNC(gijoe_state::screen_update));
 	screen.set_palette("palette");
 
 	PALETTE(config, "palette").set_format(palette_device::xBGR_555, 2048).enable_shadows();
@@ -532,6 +526,7 @@ void gijoe_state::gijoe(machine_config &config)
 	K056832(config, m_k056832, 0);
 	m_k056832->set_tile_callback(FUNC(gijoe_state::tile_callback));
 	m_k056832->set_config(K056832_BPP_4, 1, 0);
+	m_k056832->set_linemap_enable(true);
 	m_k056832->set_palette(m_palette);
 
 	K053246(config, m_k053246, 0);
@@ -541,12 +536,12 @@ void gijoe_state::gijoe(machine_config &config)
 
 	K053251(config, m_k053251, 0);
 
-	/* sound hardware */
+	// sound hardware
 	SPEAKER(config, "speaker", 2).front();
 
 	K054321(config, m_k054321, "speaker");
 
-	k054539_device &k054539(K054539(config, "k054539", XTAL(18'432'000)));
+	k054539_device &k054539(K054539(config, "k054539", 18.432_MHz_XTAL));
 	k054539.timer_handler().set_inputline("audiocpu", INPUT_LINE_NMI);
 	k054539.add_route(0, "speaker", 1.0, 1);
 	k054539.add_route(1, "speaker", 1.0, 0);
@@ -576,16 +571,16 @@ ROM_START( gijoe )
 	ROM_REGION( 0x200000, "k054539", 0 )
 	ROM_LOAD( "069a04.1e", 0x000000, 0x200000, CRC(11d6dcd6) SHA1(04cbff9f61cd8641db538db809ddf20da29fd5ac) )
 
-	ROM_REGION( 0x80, "eeprom", 0 ) // default eeprom to prevent game booting upside down with error
+	ROM_REGION( 0x80, "eeprom", 0 ) // default EEPROM to prevent game booting upside down with error
 	ROM_LOAD( "er5911.7d", 0x0000, 0x080, CRC(a0d50a79) SHA1(972533ea45a0e84d9dd14c55f58cd7247926792e) )
 ROM_END
 
-// this set is strange, instead of showing program OK it shows the location and checksums of the ROMs
-// this doesn't indicate failure, as if you hack the parent set it will show the checksum and the word 'BAD' and refuse to boot
-// It will boot as whatever version string is in the EEPROM.  If no version string is in the EEPROM it just shows a blank string
-// If you factory default it you get the string 'EB8'
-// the roms had no proper labels
-// maybe it's some interim / test revision
+/* This set is strange, instead of showing program OK it shows the location and checksums of the ROMs.
+   This doesn't indicate failure, as if you hack the parent set it will show the checksum and the word 'BAD' and refuse to boot.
+   It will boot as whatever version string is in the EEPROM.  If no version string is in the EEPROM it just shows a blank string
+   If you factory default it you get the string 'EB8'
+   The ROMs had no proper labels.
+   Maybe it's some interim / test revision. */
 ROM_START( gijoeea )
 	ROM_REGION( 0x100000, "maincpu", 0 )
 	ROM_LOAD16_BYTE( "rom3.14e",   0x000000,  0x40000, CRC(0a11f63a) SHA1(06174682907e718017146665b8636be20843b119) )
@@ -609,7 +604,7 @@ ROM_START( gijoeea )
 	ROM_REGION( 0x200000, "k054539", 0 )
 	ROM_LOAD( "069a04.1e", 0x000000, 0x200000, CRC(11d6dcd6) SHA1(04cbff9f61cd8641db538db809ddf20da29fd5ac) )
 
-	ROM_REGION( 0x80, "eeprom", 0 ) // default eeprom
+	ROM_REGION( 0x80, "eeprom", 0 ) // default EEPROM
 	ROM_LOAD( "er5911.7d", 0x0000, 0x080, CRC(64f5c87b) SHA1(af81abc54eb59ef7d2250b5ab6cc9642fbd9bfb2) ) // sldh
 ROM_END
 
@@ -636,7 +631,7 @@ ROM_START( gijoeu )
 	ROM_REGION( 0x200000, "k054539", 0 )
 	ROM_LOAD( "069a04.1e", 0x000000, 0x200000, CRC(11d6dcd6) SHA1(04cbff9f61cd8641db538db809ddf20da29fd5ac) )
 
-	ROM_REGION( 0x80, "eeprom", 0 ) // default eeprom to prevent game booting upside down with error
+	ROM_REGION( 0x80, "eeprom", 0 ) // default EEPROM to prevent game booting upside down with error
 	ROM_LOAD( "er5911.7d", 0x0000, 0x080, CRC(ca966023) SHA1(6f07ece0f95213bc12387192986f468d23dfdfc8) ) // sldh
 ROM_END
 
@@ -700,7 +695,7 @@ ROM_START( gijoeua ) // this uses a standard GX069 main PCB, but the GFX ROMs ar
 	// overlay standard ROM for now
 	ROM_LOAD( "069a04a.1e", 0x000000, 0x200000, BAD_DUMP CRC(11d6dcd6) SHA1(04cbff9f61cd8641db538db809ddf20da29fd5ac) )
 
-	ROM_REGION( 0x80, "eeprom", 0 ) // default eeprom to prevent game booting upside down with error
+	ROM_REGION( 0x80, "eeprom", 0 ) // default EEPROM to prevent game booting upside down with error
 	ROM_LOAD( "er5911.7d", 0x0000, 0x080, CRC(33b07813) SHA1(aa4df1b4265e24cb79d1405dfdf5998689f6561e) ) // sldh
 ROM_END
 
@@ -727,11 +722,38 @@ ROM_START( gijoej )
 	ROM_REGION( 0x200000, "k054539", 0 )
 	ROM_LOAD( "069a04.1e", 0x000000, 0x200000, CRC(11d6dcd6) SHA1(04cbff9f61cd8641db538db809ddf20da29fd5ac) )
 
-	ROM_REGION( 0x80, "eeprom", 0 ) // default eeprom to prevent game booting upside down with error
+	ROM_REGION( 0x80, "eeprom", 0 ) // default EEPROM to prevent game booting upside down with error
 	ROM_LOAD( "er5911.7d", 0x0000, 0x080, CRC(c914fcf2) SHA1(b4f0a0b5d6d4075b004b061336d162336ce1a754) ) // sldh
 ROM_END
 
 ROM_START( gijoea )
+	ROM_REGION( 0x100000, "maincpu", 0 )
+	ROM_LOAD16_BYTE( "069aab03.14e", 0x000000, 0x40000, CRC(60434367) SHA1(240d6c6f6f44811f367b13b960c731a74c6d9b2a) )
+	ROM_LOAD16_BYTE( "069aab02.18e", 0x000001, 0x40000, CRC(1ab8727f) SHA1(5574d2bea0e0daa98258e1e240c97dd0b96da010) )
+	ROM_LOAD16_BYTE( "069a12.13e",   0x080000, 0x40000, CRC(75a7585c) SHA1(443d6dee99edbe81ab1b7289e6cad403fe01cc0d) )
+	ROM_LOAD16_BYTE( "069a11.16e",   0x080001, 0x40000, CRC(3153e788) SHA1(fde4543eac707ef24b431e64011cf0f923d4d3ac) )
+
+	ROM_REGION( 0x010000, "audiocpu", 0 )
+	ROM_LOAD( "069a01.7c", 0x000000, 0x010000, CRC(74172b99) SHA1(f5e0e0d43317454fdacd3df7cd3035fcae4aef68) )
+
+	ROM_REGION( 0x200000, "k056832", 0 )
+	ROM_LOAD32_WORD( "069a10.18j", 0x000000, 0x100000, CRC(4c6743ee) SHA1(fa94fbfb55955fdb40705e79b49103676961d919) )
+	ROM_LOAD32_WORD( "069a09.16j", 0x000002, 0x100000, CRC(e6e36b05) SHA1(fecad503f2c285b2b0312e888c06dd6e87f95a07) )
+
+	ROM_REGION( 0x400000, "k053246", 0 )
+	ROM_LOAD64_WORD( "069a08.6h", 0x000000, 0x100000, CRC(325477d4) SHA1(140c57b0ac9e5cf702d788f416408a5eeb5d6d3c) )
+	ROM_LOAD64_WORD( "069a05.1h", 0x000002, 0x100000, CRC(c4ab07ed) SHA1(dc806eff00937d9465b1726fae8fdc3022464a28) )
+	ROM_LOAD64_WORD( "069a07.4h", 0x000004, 0x100000, CRC(ccaa3971) SHA1(16989cbbd65fe1b41c4a85fea02ba1e9880818a9) )
+	ROM_LOAD64_WORD( "069a06.2h", 0x000006, 0x100000, CRC(63eba8e1) SHA1(aa318d356c2580765452106ea0d2228273a90523) )
+
+	ROM_REGION( 0x200000, "k054539", 0 )
+	ROM_LOAD( "069a04.1e", 0x000000, 0x200000, CRC(11d6dcd6) SHA1(04cbff9f61cd8641db538db809ddf20da29fd5ac) )
+
+	ROM_REGION( 0x80, "eeprom", 0 ) // default EEPROM
+	ROM_LOAD( "er5911.7d", 0x0000, 0x080, CRC(e5676727) SHA1(48965da3dc10382152f172adc0992484203671e1) ) // sldh
+ROM_END
+
+ROM_START( gijoeaa )
 	ROM_REGION( 0x100000, "maincpu", 0 )
 	ROM_LOAD16_BYTE( "069aa03.14e", 0x000000,  0x40000, CRC(74355c6e) SHA1(01d7b5994c5b9b6e87fb9a35ffed9cc540cfcd05) )
 	ROM_LOAD16_BYTE( "069aa02.18e", 0x000001,  0x40000, CRC(d3dd0397) SHA1(6caac73d259ff6707ded2457b4968d3d0a3c4eb3) )
@@ -754,16 +776,18 @@ ROM_START( gijoea )
 	ROM_REGION( 0x200000, "k054539", 0 )
 	ROM_LOAD( "069a04.1e", 0x000000, 0x200000, CRC(11d6dcd6) SHA1(04cbff9f61cd8641db538db809ddf20da29fd5ac) )
 
-	ROM_REGION( 0x80, "eeprom", 0 ) // default eeprom
+	ROM_REGION( 0x80, "eeprom", 0 ) // default EEPROM
 	ROM_LOAD( "er5911.7d", 0x0000, 0x080, CRC(6363513c) SHA1(181cbf2bd4960740d437c714dc70bb7e64c95348) ) // sldh
 ROM_END
 
+
 } // anonymous namespace
 
 
-GAME( 1992, gijoe,   0,     gijoe, gijoe, gijoe_state, empty_init, ROT0, "Konami", "G.I. Joe (World, EAB, set 1)", MACHINE_SUPPORTS_SAVE )
+GAME( 1992, gijoe,   0,     gijoe, gijoe, gijoe_state, empty_init, ROT0, "Konami", "G.I. Joe (World, EAB)",             MACHINE_SUPPORTS_SAVE )
 GAME( 1992, gijoeea, gijoe, gijoe, gijoe, gijoe_state, empty_init, ROT0, "Konami", "G.I. Joe (World, EB8, prototype?)", MACHINE_SUPPORTS_SAVE )
-GAME( 1992, gijoeu,  gijoe, gijoe, gijoe, gijoe_state, empty_init, ROT0, "Konami", "G.I. Joe (US, UAB)", MACHINE_SUPPORTS_SAVE )
-GAME( 1992, gijoeua, gijoe, gijoe, gijoe, gijoe_state, empty_init, ROT0, "Konami", "G.I. Joe (US, UAA)", MACHINE_SUPPORTS_SAVE )
-GAME( 1992, gijoej,  gijoe, gijoe, gijoe, gijoe_state, empty_init, ROT0, "Konami", "G.I. Joe (Japan, JAA)", MACHINE_SUPPORTS_SAVE )
-GAME( 1992, gijoea,  gijoe, gijoe, gijoe, gijoe_state, empty_init, ROT0, "Konami", "G.I. Joe (Asia, AA)", MACHINE_SUPPORTS_SAVE )
+GAME( 1992, gijoeu,  gijoe, gijoe, gijoe, gijoe_state, empty_init, ROT0, "Konami", "G.I. Joe (US, UAB)",                MACHINE_SUPPORTS_SAVE )
+GAME( 1992, gijoeua, gijoe, gijoe, gijoe, gijoe_state, empty_init, ROT0, "Konami", "G.I. Joe (US, UAA)",                MACHINE_SUPPORTS_SAVE )
+GAME( 1992, gijoej,  gijoe, gijoe, gijoe, gijoe_state, empty_init, ROT0, "Konami", "G.I. Joe (Japan, JAA)",             MACHINE_SUPPORTS_SAVE )
+GAME( 1992, gijoea,  gijoe, gijoe, gijoe, gijoe_state, empty_init, ROT0, "Konami", "G.I. Joe (Asia, AAB)",              MACHINE_SUPPORTS_SAVE )
+GAME( 1992, gijoeaa, gijoe, gijoe, gijoe, gijoe_state, empty_init, ROT0, "Konami", "G.I. Joe (Asia, AA)",               MACHINE_SUPPORTS_SAVE )
