@@ -64,6 +64,7 @@ public:
 		m_keyboard(*this, "kbd"),
 		m_rs232b(*this, "rs232b"),
 		m_drive_led(*this, "drive%u_led", 0U),
+		m_rom_view(*this, "rom_view"),
 		m_sasi_dma(false),
 		m_dma_map(0),
 		m_status0(0), m_status1(0), m_status2(0),
@@ -73,17 +74,14 @@ public:
 	void psi98(machine_config &config);
 
 protected:
-	virtual void machine_start() override;
-	virtual void machine_reset() override;
+	virtual void machine_start() override ATTR_COLD;
+	virtual void machine_reset() override ATTR_COLD;
 
 private:
-	DECLARE_WRITE_LINE_MEMBER(busreq_w);
 	uint8_t memory_r(offs_t offset);
 	void memory_w(offs_t offset, uint8_t data);
 	uint8_t io_r(offs_t offset);
 	void io_w(offs_t offset, uint8_t data);
-	uint8_t page0_r(offs_t offset);
-	uint8_t page1_r(offs_t offset);
 	uint8_t mapper_r(offs_t offset);
 	void mapper_w(offs_t offset, uint8_t data);
 	uint8_t sasi_ctrl_r();
@@ -104,7 +102,7 @@ private:
 	TIMER_DEVICE_CALLBACK_MEMBER(beeper_off);
 
 	void fdc_tc_w(uint8_t data);
-	DECLARE_WRITE_LINE_MEMBER(fdc_drq_w);
+	void fdc_drq_w(int state);
 	void drive0_led_cb(floppy_image_device *floppy, int state);
 	void drive1_led_cb(floppy_image_device *floppy, int state);
 
@@ -112,12 +110,12 @@ private:
 	uint32_t screen_update(screen_device &screen, bitmap_rgb32 &bitmap, const rectangle &cliprect);
 
 	void pio_porta_w(uint8_t data);
-	DECLARE_WRITE_LINE_MEMBER(keyboard_rx_w);
-	DECLARE_WRITE_LINE_MEMBER(rs232b_rx_w);
-	DECLARE_WRITE_LINE_MEMBER(siob_tx_w);
+	void keyboard_rx_w(int state);
+	void rs232b_rx_w(int state);
+	void siob_tx_w(int state);
 
-	void psi98_io(address_map &map);
-	void psi98_mem(address_map &map);
+	void psi98_io(address_map &map) ATTR_COLD;
+	void psi98_mem(address_map &map) ATTR_COLD;
 
 	required_device<z80_device> m_cpu;
 	required_device<z80dma_device> m_dma;
@@ -138,6 +136,8 @@ private:
 	required_device<rs232_port_device> m_rs232b;
 	output_finder<2> m_drive_led;
 
+	memory_view m_rom_view;
+
 	std::unique_ptr<uint8_t[]> m_ram;
 	std::unique_ptr<uint16_t[]> m_vram; // 10-bit
 	std::unique_ptr<uint8_t[]> m_dummy_r, m_dummy_w;
@@ -158,22 +158,13 @@ private:
 
 void kdt6_state::psi98_mem(address_map &map)
 {
-	map(0x0000, 0x0fff).bankr("page0_r").bankw("page0_w");
-	map(0x1000, 0x1fff).bankr("page1_r").bankw("page1_w");
-	map(0x2000, 0x2fff).bankr("page2_r").bankw("page2_w");
-	map(0x3000, 0x3fff).bankr("page3_r").bankw("page3_w");
-	map(0x4000, 0x4fff).bankr("page4_r").bankw("page4_w");
-	map(0x5000, 0x5fff).bankr("page5_r").bankw("page5_w");
-	map(0x6000, 0x6fff).bankr("page6_r").bankw("page6_w");
-	map(0x7000, 0x7fff).bankr("page7_r").bankw("page7_w");
-	map(0x8000, 0x8fff).bankr("page8_r").bankw("page8_w");
-	map(0x9000, 0x9fff).bankr("page9_r").bankw("page9_w");
-	map(0xa000, 0xafff).bankr("pagea_r").bankw("pagea_w");
-	map(0xb000, 0xbfff).bankr("pageb_r").bankw("pageb_w");
-	map(0xc000, 0xcfff).bankr("pagec_r").bankw("pagec_w");
-	map(0xd000, 0xdfff).bankr("paged_r").bankw("paged_w");
-	map(0xe000, 0xefff).bankr("pagee_r").bankw("pagee_w");
-	map(0xf000, 0xffff).bankr("pagef_r").bankw("pagef_w");
+	for (int i = 0; i < 16; i++)
+	{
+		map((i << 12) + 0x0000, (i << 12) + 0x0fff).bankr(m_page_r[i]).bankw(m_page_w[i]);
+	}
+	// override the region 0x0000 to 0x1fff here to enable prom reading
+	map(0x0000, 0x1fff).view(m_rom_view);
+	m_rom_view[0](0x0000, 0x1fff).rom().region("boot", 0);
 }
 
 void kdt6_state::psi98_io(address_map &map)
@@ -239,7 +230,7 @@ void kdt6_state::fdc_tc_w(uint8_t data)
 	m_fdc->tc_w(0);
 }
 
-WRITE_LINE_MEMBER( kdt6_state::fdc_drq_w )
+void kdt6_state::fdc_drq_w(int state)
 {
 	if (!m_sasi_dma && BIT(m_status0, 4) == 0)
 		m_dma->rdy_w(state);
@@ -307,23 +298,23 @@ MC6845_UPDATE_ROW( kdt6_state::crtc_update_row )
 		if (BIT(m_status1, 6))
 		{
 			// text mode
-			uint16_t code = m_vram[((m_status1 & 0x03) << 14) | (ma + i)];
-			uint8_t data = m_gfx[((code & 0xff) << 4) | ra];
+			uint16_t const code = m_vram[((m_status1 & 0x03) << 14) | (ma + i)];
+			uint8_t const data = m_gfx[((code & 0xff) << 4) | ra];
 
-			int inverse = BIT(code, 8) | BIT(m_status1, 5) | ((i == cursor_x) ? 1 : 0);
-			int blink = BIT(code, 9) & m_crtc->cursor_r();
+			int const inverse = BIT(code, 8) | BIT(m_status1, 5) | ((i == cursor_x) ? 1 : 0);
+			int const blink = BIT(code, 9) & m_crtc->cursor_r();
 
 			// draw 8 pixels of the character
 			for (int x = 0; x < 8; x++)
 			{
-				int color = BIT(data, 7 - x);
+				int const color = BIT(data, 7 - x);
 				bitmap.pix(y, x + i*8) = pen[blink ? 0 : (color ^ inverse)];
 			}
 		}
 		else
 		{
 			// gfx mode
-			uint8_t data = m_vram[(ma << 4) | (ra << 6) | i];
+			uint8_t const data = m_vram[(ma << 4) | (ra << 6) | i];
 
 			// draw 8 pixels of the cell
 			for (int x = 0; x < 8; x++)
@@ -353,7 +344,7 @@ void kdt6_state::pio_porta_w(uint8_t data)
 	m_centronics->write_init(BIT(data, 1));
 }
 
-WRITE_LINE_MEMBER( kdt6_state::keyboard_rx_w )
+void kdt6_state::keyboard_rx_w(int state)
 {
 	if (machine().phase() >= machine_phase::RESET)
 	{
@@ -362,7 +353,7 @@ WRITE_LINE_MEMBER( kdt6_state::keyboard_rx_w )
 	}
 }
 
-WRITE_LINE_MEMBER( kdt6_state::rs232b_rx_w )
+void kdt6_state::rs232b_rx_w(int state)
 {
 	if (machine().phase() >= machine_phase::RESET)
 	{
@@ -371,7 +362,7 @@ WRITE_LINE_MEMBER( kdt6_state::rs232b_rx_w )
 	}
 }
 
-WRITE_LINE_MEMBER( kdt6_state::siob_tx_w )
+void kdt6_state::siob_tx_w(int state)
 {
 	if (machine().phase() >= machine_phase::RESET)
 	{
@@ -387,12 +378,6 @@ WRITE_LINE_MEMBER( kdt6_state::siob_tx_w )
 //  MACHINE
 //**************************************************************************
 
-WRITE_LINE_MEMBER( kdt6_state::busreq_w )
-{
-	m_cpu->set_input_line(Z80_INPUT_LINE_BUSRQ, state);
-	m_dma->bai_w(state);
-}
-
 uint8_t kdt6_state::memory_r(offs_t offset)
 {
 	return m_ram[m_dma_map << 16 | offset];
@@ -413,22 +398,6 @@ void kdt6_state::io_w(offs_t offset, uint8_t data)
 	m_cpu->space(AS_IO).write_byte(offset, data);
 }
 
-uint8_t kdt6_state::page0_r(offs_t offset)
-{
-	if (BIT(m_status0, 5) == 0)
-		return m_boot->as_u8(offset);
-
-	return reinterpret_cast<uint8_t *>(m_page_r[0]->base())[offset];
-}
-
-uint8_t kdt6_state::page1_r(offs_t offset)
-{
-	if (BIT(m_status0, 5) == 0)
-		return m_boot->as_u8(0x1000 + offset);
-
-	return reinterpret_cast<uint8_t *>(m_page_r[1]->base())[offset];
-}
-
 uint8_t kdt6_state::sasi_ctrl_r()
 {
 	uint8_t data = 0;
@@ -470,7 +439,7 @@ void kdt6_state::mapper_w(offs_t offset, uint8_t data)
 
 	if (BIT(m_status1, 7) == 0)
 	{
-		offs_t addr = bitswap(m_mapper[offset], 8, 9, 10, 11, 7, 6, 5, 4, 0, 1, 2, 3) << 12;
+		offs_t const addr = bitswap(m_mapper[offset], 8, 9, 10, 11, 7, 6, 5, 4, 0, 1, 2, 3) << 12;
 
 		m_page_r[offset]->set_base(addr < 0x40000 ? &m_ram[addr] : &m_dummy_r[0]);
 		m_page_w[offset]->set_base(addr < 0x40000 ? &m_ram[addr] : &m_dummy_w[0]);
@@ -501,6 +470,11 @@ void kdt6_state::status0_w(uint8_t data)
 		m_beep_timer->adjust(attotime::from_msec(250)); // timing unknown
 	}
 
+	if (BIT(data, 5))
+		m_rom_view.disable();
+	else
+		m_rom_view.select(0);
+
 	for (auto &floppy : m_floppy)
 		if (floppy->get_device())
 			floppy->get_device()->mon_w(BIT(data, 7) ? 0 : 1);
@@ -577,10 +551,6 @@ void kdt6_state::machine_start()
 	m_dummy_r = std::make_unique<uint8_t[]>(0x1000);
 	m_dummy_w = std::make_unique<uint8_t[]>(0x1000);
 
-	// override the region 0x0000 to 0x1fff here to enable prom reading
-	m_cpu->space(AS_PROGRAM).install_read_handler(0x0000, 0x0fff, read8sm_delegate(*this, FUNC(kdt6_state::page0_r)));
-	m_cpu->space(AS_PROGRAM).install_read_handler(0x1000, 0x1fff, read8sm_delegate(*this, FUNC(kdt6_state::page1_r)));
-
 	m_fdc->set_rate(250000);
 
 	if (m_floppy[0]->get_device())
@@ -596,7 +566,7 @@ void kdt6_state::machine_start()
 	save_item(NAME(m_status0));
 	save_item(NAME(m_status1));
 	save_item(NAME(m_status2));
-	save_pointer(NAME(m_mapper), 16);
+	save_item(NAME(m_mapper));
 	save_item(NAME(m_video_address));
 }
 
@@ -627,6 +597,7 @@ void kdt6_state::psi98(machine_config &config)
 	m_cpu->set_addrmap(AS_PROGRAM, &kdt6_state::psi98_mem);
 	m_cpu->set_addrmap(AS_IO, &kdt6_state::psi98_io);
 	m_cpu->set_daisy_config(daisy_chain_intf);
+	m_cpu->busack_cb().set(m_dma, FUNC(z80dma_device::bai_w));
 
 	// video hardware
 	screen_device &screen(SCREEN(config, "screen", SCREEN_TYPE_RASTER));
@@ -653,7 +624,7 @@ void kdt6_state::psi98(machine_config &config)
 	TIMER(config, m_beep_timer).configure_generic(FUNC(kdt6_state::beeper_off));
 
 	Z80DMA(config, m_dma, 16_MHz_XTAL / 4);
-	m_dma->out_busreq_callback().set(FUNC(kdt6_state::busreq_w));
+	m_dma->out_busreq_callback().set_inputline(m_cpu, Z80_INPUT_LINE_BUSREQ);
 	m_dma->out_int_callback().set_inputline(m_cpu, INPUT_LINE_IRQ0);
 	m_dma->in_mreq_callback().set(FUNC(kdt6_state::memory_r));
 	m_dma->out_mreq_callback().set(FUNC(kdt6_state::memory_w));
