@@ -32,9 +32,9 @@ The Grid         v1.2   10/18/2000
 
 #include "dcs.h"
 
-#include "cpu/tms32031/tms32031.h"
 #include "cpu/adsp2100/adsp2100.h"
 #include "cpu/pic16c5x/pic16c5x.h"
+#include "cpu/tms320c3x/tms320c3x.h"
 #include "machine/ibm21s850.h"
 #include "machine/nvram.h"
 #include "machine/tsb12lv01a.h"
@@ -44,14 +44,15 @@ The Grid         v1.2   10/18/2000
 
 #include "crusnexo.lh"
 
+// table in the code indicates an offset of 20 with a beam height of 7
+static constexpr int BEAM_XOFFS = 40;
 static constexpr int BEAM_DY = 3;
 static constexpr int BEAM_DX = 3;
-static constexpr int BEAM_XOFFS = 40; // table in the code indicates an offset of 20 with a beam height of 7
 
 #define LOG_FIREWIRE    (1U << 1)
 #define LOG_DISK        (1U << 2)
 #define LOG_DISK_JR     (1U << 3)
-#define LOG_TMS32032    (1U << 4)
+#define LOG_TMS320C32   (1U << 4)
 #define LOG_INPUT       (1U << 5)
 #define LOG_CMOS        (1U << 6)
 #define LOG_UNKNOWN     (1U << 7)
@@ -255,18 +256,18 @@ void midzeus_state::machine_reset()
 
 TIMER_CALLBACK_MEMBER(midzeus_state::display_irq_off)
 {
-	m_maincpu->set_input_line(TMS3203X_IRQ0, CLEAR_LINE);
+	m_maincpu->set_input_line(TMS320C3X_IRQ0, CLEAR_LINE);
 }
 
 INTERRUPT_GEN_MEMBER(midzeus_state::display_irq)
 {
-	m_maincpu->set_input_line(TMS3203X_IRQ0, ASSERT_LINE);
+	m_maincpu->set_input_line(TMS320C3X_IRQ0, ASSERT_LINE);
 	m_display_irq_off_timer->adjust(attotime::from_hz(30000000));
 }
 
 void midzeus2_state::zeus_irq(int state)
 {
-	m_maincpu->set_input_line(TMS3203X_IRQ2, ASSERT_LINE);
+	m_maincpu->set_input_line(TMS320C3X_IRQ2, ASSERT_LINE);
 }
 
 
@@ -610,16 +611,16 @@ void midzeus2_state::firewire_irq(int state)
 void midzeus2_state::update_firewire_irq()
 {
 	LOGMASKED(LOG_FIREWIRE, "%ssserting FW IRQ\n", (m_fw_int_enable && m_fw_int) ? "A" : "Dea");
-	m_maincpu->set_input_line(TMS3203X_IRQ3, (m_fw_int_enable && m_fw_int) ? ASSERT_LINE : CLEAR_LINE);
+	m_maincpu->set_input_line(TMS320C3X_IRQ3, (m_fw_int_enable && m_fw_int) ? ASSERT_LINE : CLEAR_LINE);
 }
 
 /*************************************
  *
- *  TMS32031 I/O accesses
+ *  TMS320C31 I/O accesses
  *
  *************************************/
 
-uint32_t midzeus_state::tms32032_control_r(offs_t offset)
+uint32_t midzeus_state::tms320c32_control_r(offs_t offset)
 {
 	// watch for accesses to the timers
 	if (offset == 0x24 || offset == 0x34)
@@ -634,16 +635,16 @@ uint32_t midzeus_state::tms32032_control_r(offs_t offset)
 	if (!machine().side_effects_disabled())
 	{
 		if (offset != 0x64)
-			LOGMASKED(LOG_TMS32032, "%06X:tms32032_control_r(%02X)\n", m_maincpu->pc(), offset);
+			LOGMASKED(LOG_TMS320C32, "%06X:tms320c32_control_r(%02X)\n", m_maincpu->pc(), offset);
 	}
 
-	return m_tms32032_control[offset];
+	return m_tms320c32_control[offset];
 }
 
 
-void midzeus_state::tms32032_control_w(offs_t offset, uint32_t data, uint32_t mem_mask)
+void midzeus_state::tms320c32_control_w(offs_t offset, uint32_t data, uint32_t mem_mask)
 {
-	COMBINE_DATA(&m_tms32032_control[offset]);
+	COMBINE_DATA(&m_tms320c32_control[offset]);
 
 	// ignore changes to the memory control register
 	if (offset == 0x64)
@@ -657,7 +658,7 @@ void midzeus_state::tms32032_control_w(offs_t offset, uint32_t data, uint32_t me
 			m_timer[which]->adjust(attotime::never);
 	}
 	else
-		LOGMASKED(LOG_TMS32032, "%06X:tms32032_control_w(%02X) = %08X\n", m_maincpu->pc(), offset, data);
+		LOGMASKED(LOG_TMS320C32, "%06X:tms320c32_control_w(%02X) = %08X\n", m_maincpu->pc(), offset, data);
 }
 
 
@@ -744,9 +745,9 @@ void invasnab_state::update_gun_irq()
 {
 	// low 2 bits of gun_control seem to enable IRQs
 	if (m_gun_irq_state & m_gun_control & 0x03)
-		m_maincpu->set_input_line(TMS3203X_IRQ3, ASSERT_LINE);
+		m_maincpu->set_input_line(TMS320C3X_IRQ3, ASSERT_LINE);
 	else
-		m_maincpu->set_input_line(TMS3203X_IRQ3, CLEAR_LINE);
+		m_maincpu->set_input_line(TMS320C3X_IRQ3, CLEAR_LINE);
 }
 
 
@@ -783,8 +784,11 @@ void invasnab_state::invasn_gun_w(offs_t offset, uint32_t data, uint32_t mem_mas
 		if (((old_control ^ m_gun_control) & pmask) != 0 && (m_gun_control & pmask) == 0)
 		{
 			const rectangle &visarea = m_screen->visible_area();
-			m_gun_x[player] = m_io_gun_x[player]->read() * visarea.width() / 255 + visarea.min_x + BEAM_XOFFS;
-			m_gun_y[player] = m_io_gun_y[player]->read() * visarea.height() / 255 + visarea.min_y;
+
+			m_gun_x[player] = (m_io_gun_x[player]->read() * visarea.width() / 0xff + visarea.min_x + BEAM_XOFFS + BEAM_DX * 2) % m_screen->width();
+			m_gun_x[player] = std::clamp(m_gun_x[player], BEAM_DX, m_screen->width() - BEAM_DX);
+			m_gun_y[player] = m_io_gun_y[player]->read() * visarea.height() / 0xff + visarea.min_y + BEAM_DY * 2;
+
 			m_gun_timer[player]->adjust(m_screen->time_until_pos(std::max(0, m_gun_y[player] - BEAM_DY), std::max(0, m_gun_x[player] - BEAM_DX)), player);
 		}
 	}
@@ -793,17 +797,27 @@ void invasnab_state::invasn_gun_w(offs_t offset, uint32_t data, uint32_t mem_mas
 
 uint32_t invasnab_state::invasn_gun_r()
 {
+	// bits 8,9: gun triggers
+	uint32_t result = 0xffff;
+	result ^= m_io_gun_trigger->read() << 8;
+
 	int const beamx = m_screen->hpos();
 	int const beamy = m_screen->vpos();
-	uint32_t result = 0xffff;
 
 	for (int player = 0; player < 2; player++)
 	{
-		int const diffx = beamx - m_gun_x[player];
-		int const diffy = beamy - m_gun_y[player];
-		if (diffx >= -BEAM_DX && diffx <= BEAM_DX && diffy >= -BEAM_DY && diffy <= BEAM_DY)
-			result ^= 0x1000 << player;
+		// interpret left/right screen edge as off-screen
+		if (int io_gun_x = m_io_gun_x[player]->read(); io_gun_x > 0 && io_gun_x < 0xff)
+		{
+			int const diffx = beamx - m_gun_x[player];
+			int const diffy = beamy - m_gun_y[player];
+
+			// bits 12,13: gun position in range
+			if (diffx >= -BEAM_DX && diffx <= BEAM_DX && diffy >= -BEAM_DY && diffy <= BEAM_DY)
+				result ^= 0x1000 << player;
+		}
 	}
+
 	return result;
 }
 
@@ -820,11 +834,11 @@ void midzeus_state::zeus_map(address_map &map)
 	map.unmap_value_high();
 	map(0x000000, 0x03ffff).ram().share(m_ram_base);
 	map(0x400000, 0x41ffff).ram();
-	map(0x808000, 0x80807f).rw(FUNC(midzeus_state::tms32032_control_r), FUNC(midzeus_state::tms32032_control_w)).share(m_tms32032_control);
+	map(0x808000, 0x80807f).rw(FUNC(midzeus_state::tms320c32_control_r), FUNC(midzeus_state::tms320c32_control_w)).share(m_tms320c32_control);
 	map(0x880000, 0x8803ff).rw(FUNC(midzeus_state::zeus_r), FUNC(midzeus_state::zeus_w)).share(m_zeusbase);
 	map(0x8d0000, 0x8d0009).rw(FUNC(midzeus_state::disk_asic_jr_r), FUNC(midzeus_state::disk_asic_jr_w));
 	map(0x990000, 0x99000f).rw(m_ioasic, FUNC(midway_ioasic_device::read), FUNC(midway_ioasic_device::write));
-	map(0x9e0000, 0x9e0000).nopw();        // watchdog?
+	map(0x9e0000, 0x9e0000).nopw(); // watchdog?
 	map(0x9f0000, 0x9f7fff).rw(FUNC(midzeus_state::cmos_r), FUNC(midzeus_state::cmos_w)).share(m_nvram);
 	map(0x9f8000, 0x9f8000).w(FUNC(midzeus_state::cmos_protect_w));
 	map(0xa00000, 0xffffff).rom().region("maindata", 0);
@@ -842,7 +856,7 @@ void midzeus2_state::zeus2_map(address_map &map)
 	map.unmap_value_high();
 	map(0x000000, 0x03ffff).ram().share(m_ram_base);
 	map(0x400000, 0x43ffff).ram();
-	map(0x808000, 0x80807f).rw(FUNC(midzeus2_state::tms32032_control_r), FUNC(midzeus2_state::tms32032_control_w)).share(m_tms32032_control);
+	map(0x808000, 0x80807f).rw(FUNC(midzeus2_state::tms320c32_control_r), FUNC(midzeus2_state::tms320c32_control_w)).share(m_tms320c32_control);
 	map(0x880000, 0x88007f).rw(m_zeus, FUNC(zeus2_device::zeus2_r), FUNC(zeus2_device::zeus2_w));
 	map(0x8a0000, 0x8a00cf).rw(m_fw_link, FUNC(tsb12lv01a_device::read), FUNC(tsb12lv01a_device::write));
 	//map(0x8a0000, 0x8a00cf).rw(FUNC(midzeus2_state::firewire_r), FUNC(midzeus2_state::firewire_w)).share("firewire");
@@ -850,7 +864,7 @@ void midzeus2_state::zeus2_map(address_map &map)
 	map(0x900000, 0x91ffff).rw(FUNC(midzeus2_state::zpram_r), FUNC(midzeus2_state::zpram_w)).share(m_nvram).mirror(0x020000);
 	map(0x990000, 0x99000f).rw(m_ioasic, FUNC(midway_ioasic_device::read), FUNC(midway_ioasic_device::write));
 	map(0x9d0000, 0x9d000f).rw(FUNC(midzeus2_state::disk_asic_r), FUNC(midzeus2_state::disk_asic_w));
-	map(0x9e0000, 0x9e0000).nopw();        // watchdog?
+	map(0x9e0000, 0x9e0000).nopw(); // watchdog?
 	map(0x9f0000, 0x9f7fff).rw(FUNC(midzeus2_state::timekeeper_r), FUNC(midzeus2_state::timekeeper_w));
 	map(0x9f8000, 0x9f8000).w(FUNC(midzeus2_state::cmos_protect_w));
 	map(0xa00000, 0xbfffff).rom().region("maindata", 0);
@@ -918,7 +932,7 @@ void thegrid_state::thegrid_map(address_map &map)
  *************************************/
 
 static INPUT_PORTS_START( mk4 )
-	PORT_START("DIPS")      // DS1
+	PORT_START("DIPS") // DS1
 	PORT_DIPNAME( 0x0001, 0x0001, "Coinage Source" )
 	PORT_DIPSETTING(      0x0001, "Dipswitch" )
 	PORT_DIPSETTING(      0x0000, "CMOS" )
@@ -941,7 +955,7 @@ static INPUT_PORTS_START( mk4 )
 	PORT_DIPSETTING(      0x0018, "French-4" )
 	PORT_DIPSETTING(      0x0016, "French-ECA" )
 	PORT_DIPSETTING(      0x0030, DEF_STR( Free_Play ) )
-	PORT_DIPNAME( 0x0040, 0x0040, DEF_STR( Unknown ) )  // Manual lists this dip as Unused
+	PORT_DIPNAME( 0x0040, 0x0040, DEF_STR( Unknown ) ) // Manual lists this dip as Unused
 	PORT_DIPSETTING(      0x0040, DEF_STR( Off ) )
 	PORT_DIPSETTING(      0x0000, DEF_STR( On ) )
 	PORT_DIPNAME( 0x0080, 0x0080, "Test Switch" )
@@ -986,8 +1000,9 @@ static INPUT_PORTS_START( mk4 )
 	PORT_BIT( 0x0400, IP_ACTIVE_LOW, IPT_START4 )
 	PORT_BIT( 0x0800, IP_ACTIVE_LOW, IPT_VOLUME_DOWN )
 	PORT_BIT( 0x1000, IP_ACTIVE_LOW, IPT_VOLUME_UP )
-	PORT_BIT( 0x6000, IP_ACTIVE_LOW, IPT_UNUSED )
-	PORT_BIT( 0x8000, IP_ACTIVE_LOW, IPT_BILL1 )    // Bill
+	PORT_BIT( 0x2000, IP_ACTIVE_LOW, IPT_DOOR ) // Coin Door
+	PORT_BIT( 0x4000, IP_ACTIVE_LOW, IPT_UNUSED )
+	PORT_BIT( 0x8000, IP_ACTIVE_LOW, IPT_BILL1 )
 
 	PORT_START("IN1")
 	PORT_BIT( 0x0001, IP_ACTIVE_LOW, IPT_JOYSTICK_UP    ) PORT_PLAYER(1) PORT_8WAY
@@ -1020,7 +1035,7 @@ INPUT_PORTS_END
 
 
 static INPUT_PORTS_START( invasn )
-	PORT_START("DIPS")      // DS1
+	PORT_START("DIPS") // DS1
 	PORT_DIPNAME( 0x0001, 0x0001, "Coinage Source" )
 	PORT_DIPSETTING(      0x0001, "Dipswitch" )
 	PORT_DIPSETTING(      0x0000, "CMOS" )
@@ -1091,36 +1106,36 @@ static INPUT_PORTS_START( invasn )
 	PORT_BIT( 0x0400, IP_ACTIVE_LOW, IPT_START4 )
 	PORT_BIT( 0x0800, IP_ACTIVE_LOW, IPT_VOLUME_DOWN )
 	PORT_BIT( 0x1000, IP_ACTIVE_LOW, IPT_VOLUME_UP )
-	PORT_BIT( 0x6000, IP_ACTIVE_LOW, IPT_UNUSED )
-	PORT_BIT( 0x8000, IP_ACTIVE_LOW, IPT_BILL1 )    // Bill
+	PORT_BIT( 0x2000, IP_ACTIVE_LOW, IPT_DOOR ) // Coin Door
+	PORT_BIT( 0x4000, IP_ACTIVE_LOW, IPT_UNUSED )
+	PORT_BIT( 0x8000, IP_ACTIVE_LOW, IPT_BILL1 )
 
 	PORT_START("IN1")
-	PORT_BIT( 0x000f, IP_ACTIVE_LOW, IPT_UNUSED )
-	PORT_BIT( 0x0010, IP_ACTIVE_LOW, IPT_BUTTON1 ) PORT_PLAYER(1)
-	PORT_BIT( 0x00e0, IP_ACTIVE_LOW, IPT_UNUSED )
-	PORT_BIT( 0x0f00, IP_ACTIVE_LOW, IPT_UNUSED )
-	PORT_BIT( 0x1000, IP_ACTIVE_LOW, IPT_BUTTON1 ) PORT_PLAYER(2)
-	PORT_BIT( 0xe000, IP_ACTIVE_LOW, IPT_UNUSED )
+	PORT_BIT( 0xffff, IP_ACTIVE_LOW, IPT_UNUSED )
 
 	PORT_START("IN2")
 	PORT_BIT( 0xffff, IP_ACTIVE_LOW, IPT_UNUSED )
 
-	PORT_START("GUNX1")     // fake analog X
+	PORT_START("TRIGGER")
+	PORT_BIT( 0x01, IP_ACTIVE_HIGH, IPT_BUTTON1 ) PORT_PLAYER(1)
+	PORT_BIT( 0x02, IP_ACTIVE_HIGH, IPT_BUTTON1 ) PORT_PLAYER(2)
+
+	PORT_START("GUNX1") // fake analog X
 	PORT_BIT( 0xff, 0x80, IPT_LIGHTGUN_X ) PORT_CROSSHAIR(X, 1.0, 0.0, 0) PORT_SENSITIVITY(50) PORT_KEYDELTA(10)
 
-	PORT_START("GUNY1")     // fake analog Y
+	PORT_START("GUNY1") // fake analog Y
 	PORT_BIT( 0xff, 0x80, IPT_LIGHTGUN_Y ) PORT_CROSSHAIR(Y, 1.0, 0.0, 0) PORT_SENSITIVITY(70) PORT_KEYDELTA(10)
 
-	PORT_START("GUNX2")     // fake analog X
+	PORT_START("GUNX2") // fake analog X
 	PORT_BIT( 0xff, 0x80, IPT_LIGHTGUN_X ) PORT_CROSSHAIR(X, 1.0, 0.0, 0) PORT_SENSITIVITY(50) PORT_KEYDELTA(10) PORT_PLAYER(2)
 
-	PORT_START("GUNY2")     // fake analog Y
+	PORT_START("GUNY2") // fake analog Y
 	PORT_BIT( 0xff, 0x80, IPT_LIGHTGUN_Y ) PORT_CROSSHAIR(Y, 1.0, 0.0, 0) PORT_SENSITIVITY(70) PORT_KEYDELTA(10) PORT_PLAYER(2)
 INPUT_PORTS_END
 
 
 static INPUT_PORTS_START( crusnexo )
-	PORT_START("DIPS")      // DS1
+	PORT_START("DIPS") // DS1
 	PORT_DIPNAME( 0x001f, 0x001f, "Country Code" )
 	PORT_DIPSETTING(      0x001f, DEF_STR( USA ) )
 	PORT_DIPSETTING(      0x001e, "Germany" )
@@ -1150,10 +1165,10 @@ static INPUT_PORTS_START( crusnexo )
 	PORT_DIPNAME( 0x0080, 0x0080, "Test Switch" )
 	PORT_DIPSETTING(      0x0080, DEF_STR( Off ) )
 	PORT_DIPSETTING(      0x0000, DEF_STR( On ) )
-	PORT_DIPNAME( 0x0100, 0x0100, "Game Type" ) // Manual states "*DIP 1, Switch 1 MUST be set
-	PORT_DIPSETTING(      0x0100, "Dedicated" ) //   to OFF position for proper operation"
+	PORT_DIPNAME( 0x0100, 0x0100, "Game Type" ) // Manual states "*DIP 1, Switch 1 MUST be set to OFF position for proper operation"
+	PORT_DIPSETTING(      0x0100, "Dedicated" )
 	PORT_DIPSETTING(      0x0000, "Kit" )
-	PORT_DIPNAME( 0x0200, 0x0200, "Seat Motion" )   // For dedicated Sit Down models with Motion Seat
+	PORT_DIPNAME( 0x0200, 0x0200, "Seat Motion" ) // For dedicated Sit Down models with Motion Seat
 	PORT_DIPSETTING(      0x0200, DEF_STR( Off ) )
 	PORT_DIPSETTING(      0x0000, DEF_STR( On ) )
 	PORT_DIPNAME( 0x0400, 0x0400, DEF_STR( Cabinet ) )
@@ -1188,25 +1203,26 @@ static INPUT_PORTS_START( crusnexo )
 	PORT_BIT( 0x0400, IP_ACTIVE_LOW, IPT_START4 )
 	PORT_BIT( 0x0800, IP_ACTIVE_LOW, IPT_VOLUME_DOWN )
 	PORT_BIT( 0x1000, IP_ACTIVE_LOW, IPT_VOLUME_UP )
-	PORT_BIT( 0x6000, IP_ACTIVE_LOW, IPT_UNUSED )
+	PORT_BIT( 0x2000, IP_ACTIVE_LOW, IPT_DOOR ) // Coin Door
+	PORT_BIT( 0x4000, IP_ACTIVE_LOW, IPT_UNUSED )
 	PORT_BIT( 0x8000, IP_ACTIVE_LOW, IPT_BILL1 )
 
-	PORT_START("IN1")   // Listed "names" are via the manual's "JAMMA" pinout sheet"
-	PORT_BIT( 0x0001, IP_ACTIVE_LOW, IPT_UNKNOWN )                          // Not Used
-	PORT_BIT( 0x0002, IP_ACTIVE_LOW, IPT_BUTTON6 ) PORT_NAME("Radio")       // Radio Switch
-	PORT_BIT( 0x0004, IP_ACTIVE_LOW, IPT_UNKNOWN )                          // Not Used
-	PORT_BIT( 0x0008, IP_ACTIVE_LOW, IPT_UNKNOWN )                          // Not Used
-	PORT_BIT( 0x0010, IP_ACTIVE_LOW, IPT_BUTTON7 ) PORT_NAME("View 1")      // View 1
-	PORT_BIT( 0x0020, IP_ACTIVE_LOW, IPT_BUTTON8 ) PORT_NAME("View 2")      // View 2
-	PORT_BIT( 0x0040, IP_ACTIVE_LOW, IPT_BUTTON9 ) PORT_NAME("View 3")      // View 3
-	PORT_BIT( 0x0080, IP_ACTIVE_LOW, IPT_BUTTON10 ) PORT_NAME("View 4")     // View 4
-	PORT_BIT( 0x0100, IP_ACTIVE_LOW, IPT_BUTTON2 ) PORT_NAME("1st Gear")    // Gear 1
-	PORT_BIT( 0x0200, IP_ACTIVE_LOW, IPT_BUTTON3 ) PORT_NAME("2nd Gear")    // Gear 2
-	PORT_BIT( 0x0400, IP_ACTIVE_LOW, IPT_BUTTON4 ) PORT_NAME("3rd Gear")    // Gear 3
-	PORT_BIT( 0x0800, IP_ACTIVE_LOW, IPT_BUTTON5 ) PORT_NAME("4th Gear")    // Gear 4
-	PORT_BIT( 0x1000, IP_ACTIVE_LOW, IPT_UNKNOWN )                          // Not Used
-	PORT_BIT( 0x2000, IP_ACTIVE_LOW, IPT_UNKNOWN )                          // Not Used
-	PORT_BIT( 0x4000, IP_ACTIVE_LOW, IPT_UNKNOWN )                          // Not Used
+	PORT_START("IN1") // Listed "names" are via the manual's "JAMMA" pinout sheet"
+	PORT_BIT( 0x0001, IP_ACTIVE_LOW, IPT_UNKNOWN )                       // Not Used
+	PORT_BIT( 0x0002, IP_ACTIVE_LOW, IPT_BUTTON6 ) PORT_NAME("Radio")    // Radio Switch
+	PORT_BIT( 0x0004, IP_ACTIVE_LOW, IPT_UNKNOWN )                       // Not Used
+	PORT_BIT( 0x0008, IP_ACTIVE_LOW, IPT_UNKNOWN )                       // Not Used
+	PORT_BIT( 0x0010, IP_ACTIVE_LOW, IPT_BUTTON7 ) PORT_NAME("View 1")   // View 1
+	PORT_BIT( 0x0020, IP_ACTIVE_LOW, IPT_BUTTON8 ) PORT_NAME("View 2")   // View 2
+	PORT_BIT( 0x0040, IP_ACTIVE_LOW, IPT_BUTTON9 ) PORT_NAME("View 3")   // View 3
+	PORT_BIT( 0x0080, IP_ACTIVE_LOW, IPT_BUTTON10 ) PORT_NAME("View 4")  // View 4
+	PORT_BIT( 0x0100, IP_ACTIVE_LOW, IPT_BUTTON2 ) PORT_NAME("1st Gear") // Gear 1
+	PORT_BIT( 0x0200, IP_ACTIVE_LOW, IPT_BUTTON3 ) PORT_NAME("2nd Gear") // Gear 2
+	PORT_BIT( 0x0400, IP_ACTIVE_LOW, IPT_BUTTON4 ) PORT_NAME("3rd Gear") // Gear 3
+	PORT_BIT( 0x0800, IP_ACTIVE_LOW, IPT_BUTTON5 ) PORT_NAME("4th Gear") // Gear 4
+	PORT_BIT( 0x1000, IP_ACTIVE_LOW, IPT_UNKNOWN )                       // Not Used
+	PORT_BIT( 0x2000, IP_ACTIVE_LOW, IPT_UNKNOWN )                       // Not Used
+	PORT_BIT( 0x4000, IP_ACTIVE_LOW, IPT_UNKNOWN )                       // Not Used
 	PORT_BIT( 0x8000, IP_ACTIVE_LOW, IPT_UNKNOWN )
 
 	PORT_START("IN2")
@@ -1214,18 +1230,18 @@ static INPUT_PORTS_START( crusnexo )
 	PORT_BIT( 0xfff8, IP_ACTIVE_LOW, IPT_UNUSED )
 
 	PORT_START("KEYPAD")
-	PORT_BIT( 0x0001, IP_ACTIVE_LOW, IPT_OTHER ) PORT_NAME("Keypad 3") PORT_CODE(KEYCODE_3_PAD)   // keypad 3
-	PORT_BIT( 0x0002, IP_ACTIVE_LOW, IPT_OTHER ) PORT_NAME("Keypad 1") PORT_CODE(KEYCODE_1_PAD)   // keypad 1
-	PORT_BIT( 0x0004, IP_ACTIVE_LOW, IPT_OTHER ) PORT_NAME("Keypad 2") PORT_CODE(KEYCODE_2_PAD)   // keypad 2
-	PORT_BIT( 0x0010, IP_ACTIVE_LOW, IPT_OTHER ) PORT_NAME("Keypad 6") PORT_CODE(KEYCODE_6_PAD)   // keypad 6
-	PORT_BIT( 0x0020, IP_ACTIVE_LOW, IPT_OTHER ) PORT_NAME("Keypad 4") PORT_CODE(KEYCODE_4_PAD)   // keypad 4
-	PORT_BIT( 0x0040, IP_ACTIVE_LOW, IPT_OTHER ) PORT_NAME("Keypad 5") PORT_CODE(KEYCODE_5_PAD)   // keypad 5
-	PORT_BIT( 0x0100, IP_ACTIVE_LOW, IPT_OTHER ) PORT_NAME("Keypad 9") PORT_CODE(KEYCODE_9_PAD)   // keypad 9
-	PORT_BIT( 0x0200, IP_ACTIVE_LOW, IPT_OTHER ) PORT_NAME("Keypad 7") PORT_CODE(KEYCODE_7_PAD)   // keypad 7
-	PORT_BIT( 0x0400, IP_ACTIVE_LOW, IPT_OTHER ) PORT_NAME("Keypad 8") PORT_CODE(KEYCODE_8_PAD)   // keypad 8
-	PORT_BIT( 0x1000, IP_ACTIVE_LOW, IPT_OTHER ) PORT_NAME("Keypad #") PORT_CODE(KEYCODE_PLUS_PAD)    // keypad #
-	PORT_BIT( 0x2000, IP_ACTIVE_LOW, IPT_OTHER ) PORT_NAME("Keypad *") PORT_CODE(KEYCODE_MINUS_PAD)   // keypad *
-	PORT_BIT( 0x4000, IP_ACTIVE_LOW, IPT_OTHER ) PORT_NAME("Keypad 0") PORT_CODE(KEYCODE_0_PAD)   // keypad 0
+	PORT_BIT( 0x0001, IP_ACTIVE_LOW, IPT_OTHER ) PORT_NAME("Keypad 3") PORT_CODE(KEYCODE_3_PAD)     // keypad 3
+	PORT_BIT( 0x0002, IP_ACTIVE_LOW, IPT_OTHER ) PORT_NAME("Keypad 1") PORT_CODE(KEYCODE_1_PAD)     // keypad 1
+	PORT_BIT( 0x0004, IP_ACTIVE_LOW, IPT_OTHER ) PORT_NAME("Keypad 2") PORT_CODE(KEYCODE_2_PAD)     // keypad 2
+	PORT_BIT( 0x0010, IP_ACTIVE_LOW, IPT_OTHER ) PORT_NAME("Keypad 6") PORT_CODE(KEYCODE_6_PAD)     // keypad 6
+	PORT_BIT( 0x0020, IP_ACTIVE_LOW, IPT_OTHER ) PORT_NAME("Keypad 4") PORT_CODE(KEYCODE_4_PAD)     // keypad 4
+	PORT_BIT( 0x0040, IP_ACTIVE_LOW, IPT_OTHER ) PORT_NAME("Keypad 5") PORT_CODE(KEYCODE_5_PAD)     // keypad 5
+	PORT_BIT( 0x0100, IP_ACTIVE_LOW, IPT_OTHER ) PORT_NAME("Keypad 9") PORT_CODE(KEYCODE_9_PAD)     // keypad 9
+	PORT_BIT( 0x0200, IP_ACTIVE_LOW, IPT_OTHER ) PORT_NAME("Keypad 7") PORT_CODE(KEYCODE_7_PAD)     // keypad 7
+	PORT_BIT( 0x0400, IP_ACTIVE_LOW, IPT_OTHER ) PORT_NAME("Keypad 8") PORT_CODE(KEYCODE_8_PAD)     // keypad 8
+	PORT_BIT( 0x1000, IP_ACTIVE_LOW, IPT_OTHER ) PORT_NAME("Keypad #") PORT_CODE(KEYCODE_PLUS_PAD)  // keypad #
+	PORT_BIT( 0x2000, IP_ACTIVE_LOW, IPT_OTHER ) PORT_NAME("Keypad *") PORT_CODE(KEYCODE_MINUS_PAD) // keypad *
+	PORT_BIT( 0x4000, IP_ACTIVE_LOW, IPT_OTHER ) PORT_NAME("Keypad 0") PORT_CODE(KEYCODE_0_PAD)     // keypad 0
 
 	PORT_START("ANALOG3")
 	PORT_BIT( 0xff, 0x80, IPT_PADDLE ) PORT_MINMAX(0x10,0xf0) PORT_SENSITIVITY(25) PORT_KEYDELTA(20)
@@ -1242,7 +1258,7 @@ INPUT_PORTS_END
 
 
 static INPUT_PORTS_START( thegrid )
-	PORT_START("DIPS")      // DS1
+	PORT_START("DIPS") // DS1
 	PORT_DIPNAME( 0x0100, 0x0100, "Show Blood" )
 	PORT_DIPSETTING(      0x0100, "Show Blood" )
 	PORT_DIPSETTING(      0x0000, "Do not show blood" )
@@ -1283,7 +1299,7 @@ static INPUT_PORTS_START( thegrid )
 	PORT_DIPSETTING(      0x0004, "UK-6 ECA" )
 	PORT_DIPSETTING(      0x0002, "UK-7 ECA" )
 	PORT_DIPSETTING(      0x0000, DEF_STR( Free_Play ) )
-	PORT_DIPNAME( 0x0040, 0x0040, DEF_STR( Unknown ) )  // Manual states switches 7 & 8 are Unused
+	PORT_DIPNAME( 0x0040, 0x0040, DEF_STR( Unknown ) ) // Manual states switches 7 & 8 are Unused
 	PORT_DIPSETTING(      0x0040, DEF_STR( Off ) )
 	PORT_DIPSETTING(      0x0000, DEF_STR( On ) )
 	PORT_DIPNAME( 0x0080, 0x0080, "Game Mode" )
@@ -1304,10 +1320,11 @@ static INPUT_PORTS_START( thegrid )
 	PORT_BIT( 0x0400, IP_ACTIVE_LOW, IPT_START4 )
 	PORT_BIT( 0x0800, IP_ACTIVE_LOW, IPT_VOLUME_DOWN )
 	PORT_BIT( 0x1000, IP_ACTIVE_LOW, IPT_VOLUME_UP )
-	PORT_BIT( 0x6000, IP_ACTIVE_LOW, IPT_UNUSED )
-	PORT_BIT( 0x8000, IP_ACTIVE_LOW, IPT_BILL1 )    // Bill
+	PORT_BIT( 0x2000, IP_ACTIVE_LOW, IPT_DOOR ) // Coin Door
+	PORT_BIT( 0x4000, IP_ACTIVE_LOW, IPT_UNUSED )
+	PORT_BIT( 0x8000, IP_ACTIVE_LOW, IPT_BILL1 )
 
-	PORT_START("IN1")   // Listed "names" are via the manual's "JAMMA" pinout sheet"
+	PORT_START("IN1") // Listed "names" are via the manual's "JAMMA" pinout sheet"
 	PORT_BIT( 0x0001, IP_ACTIVE_LOW, IPT_JOYSTICK_UP    ) PORT_PLAYER(1) PORT_8WAY // Not Used
 	PORT_BIT( 0x0002, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN  ) PORT_PLAYER(1) PORT_8WAY // Not Used
 	PORT_BIT( 0x0004, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT  ) PORT_PLAYER(1) PORT_8WAY // Not Used
@@ -1336,25 +1353,24 @@ static INPUT_PORTS_START( thegrid )
 	PORT_BIT( 0xff, 0x38, IPT_AD_STICK_Y ) PORT_MINMAX(0x00,0x6f) PORT_SENSITIVITY(100) PORT_KEYDELTA(10) PORT_REVERSE
 
 	PORT_START("KEYPAD")
-	PORT_BIT(0x001, IP_ACTIVE_HIGH, IPT_OTHER) PORT_NAME("Keypad 1") PORT_CODE(KEYCODE_1_PAD)     // keypad 1
-	PORT_BIT(0x002, IP_ACTIVE_HIGH, IPT_OTHER) PORT_NAME("Keypad 4") PORT_CODE(KEYCODE_4_PAD)     // keypad 4
-	PORT_BIT(0x004, IP_ACTIVE_HIGH, IPT_OTHER) PORT_NAME("Keypad 7") PORT_CODE(KEYCODE_7_PAD)     // keypad 7
-	PORT_BIT(0x008, IP_ACTIVE_HIGH, IPT_OTHER) PORT_NAME("Keypad *") PORT_CODE(KEYCODE_ASTERISK)  // keypad *
-	PORT_BIT(0x010, IP_ACTIVE_HIGH, IPT_OTHER) PORT_NAME("Keypad 2") PORT_CODE(KEYCODE_2_PAD)     // keypad 2
-	PORT_BIT(0x020, IP_ACTIVE_HIGH, IPT_OTHER) PORT_NAME("Keypad 5") PORT_CODE(KEYCODE_5_PAD)     // keypad 5
-	PORT_BIT(0x040, IP_ACTIVE_HIGH, IPT_OTHER) PORT_NAME("Keypad 8") PORT_CODE(KEYCODE_8_PAD)     // keypad 8
-	PORT_BIT(0x080, IP_ACTIVE_HIGH, IPT_OTHER) PORT_NAME("Keypad 0") PORT_CODE(KEYCODE_0_PAD)     // keypad 0
-	PORT_BIT(0x100, IP_ACTIVE_HIGH, IPT_OTHER) PORT_NAME("Keypad 3") PORT_CODE(KEYCODE_3_PAD)     // keypad 3
-	PORT_BIT(0x200, IP_ACTIVE_HIGH, IPT_OTHER) PORT_NAME("Keypad 6") PORT_CODE(KEYCODE_6_PAD)     // keypad 6
-	PORT_BIT(0x400, IP_ACTIVE_HIGH, IPT_OTHER) PORT_NAME("Keypad 9") PORT_CODE(KEYCODE_9_PAD)     // keypad 9
-	PORT_BIT(0x800, IP_ACTIVE_HIGH, IPT_OTHER) PORT_NAME("Keypad #") PORT_CODE(KEYCODE_PLUS_PAD)  // keypad #
+	PORT_BIT(0x001, IP_ACTIVE_HIGH, IPT_OTHER) PORT_NAME("Keypad 1") PORT_CODE(KEYCODE_1_PAD)    // keypad 1
+	PORT_BIT(0x002, IP_ACTIVE_HIGH, IPT_OTHER) PORT_NAME("Keypad 4") PORT_CODE(KEYCODE_4_PAD)    // keypad 4
+	PORT_BIT(0x004, IP_ACTIVE_HIGH, IPT_OTHER) PORT_NAME("Keypad 7") PORT_CODE(KEYCODE_7_PAD)    // keypad 7
+	PORT_BIT(0x008, IP_ACTIVE_HIGH, IPT_OTHER) PORT_NAME("Keypad *") PORT_CODE(KEYCODE_ASTERISK) // keypad *
+	PORT_BIT(0x010, IP_ACTIVE_HIGH, IPT_OTHER) PORT_NAME("Keypad 2") PORT_CODE(KEYCODE_2_PAD)    // keypad 2
+	PORT_BIT(0x020, IP_ACTIVE_HIGH, IPT_OTHER) PORT_NAME("Keypad 5") PORT_CODE(KEYCODE_5_PAD)    // keypad 5
+	PORT_BIT(0x040, IP_ACTIVE_HIGH, IPT_OTHER) PORT_NAME("Keypad 8") PORT_CODE(KEYCODE_8_PAD)    // keypad 8
+	PORT_BIT(0x080, IP_ACTIVE_HIGH, IPT_OTHER) PORT_NAME("Keypad 0") PORT_CODE(KEYCODE_0_PAD)    // keypad 0
+	PORT_BIT(0x100, IP_ACTIVE_HIGH, IPT_OTHER) PORT_NAME("Keypad 3") PORT_CODE(KEYCODE_3_PAD)    // keypad 3
+	PORT_BIT(0x200, IP_ACTIVE_HIGH, IPT_OTHER) PORT_NAME("Keypad 6") PORT_CODE(KEYCODE_6_PAD)    // keypad 6
+	PORT_BIT(0x400, IP_ACTIVE_HIGH, IPT_OTHER) PORT_NAME("Keypad 9") PORT_CODE(KEYCODE_9_PAD)    // keypad 9
+	PORT_BIT(0x800, IP_ACTIVE_HIGH, IPT_OTHER) PORT_NAME("Keypad #") PORT_CODE(KEYCODE_PLUS_PAD) // keypad #
 
 	PORT_START("TRACKX")
 	PORT_BIT(0xff, 0x00, IPT_TRACKBALL_X) PORT_SENSITIVITY(1) PORT_KEYDELTA(1) PORT_PLAYER(1)
 
 	PORT_START("TRACKY")
 	PORT_BIT(0xff, 0x00, IPT_TRACKBALL_Y) PORT_SENSITIVITY(1) PORT_KEYDELTA(1) PORT_REVERSE PORT_PLAYER(1)
-
 INPUT_PORTS_END
 
 
@@ -1365,12 +1381,10 @@ INPUT_PORTS_END
  *
  *************************************/
 
-static constexpr XTAL CPU_CLOCK = XTAL(60'000'000);
-
 void midzeus_state::midzeus(machine_config &config)
 {
 	// basic machine hardware
-	TMS32032(config, m_maincpu, CPU_CLOCK);
+	TMS320C32(config, m_maincpu, 60_MHz_XTAL);
 	m_maincpu->set_addrmap(AS_PROGRAM, &midzeus_state::zeus_map);
 	m_maincpu->set_vblank_int("screen", FUNC(midzeus_state::display_irq));
 
@@ -1405,7 +1419,7 @@ void midzeus_state::midzeus(machine_config &config)
 void midzeus_state::mk4(machine_config &config)
 {
 	midzeus(config);
-	m_ioasic->set_upper(461/* or 474 */);
+	m_ioasic->set_upper(461 /* or 474 */ );
 	m_ioasic->set_shuffle_default(1);
 }
 
@@ -1414,14 +1428,14 @@ void invasnab_state::invasn(machine_config &config)
 	midzeus(config);
 	m_maincpu->set_addrmap(AS_PROGRAM, &invasnab_state::invasnab_map);
 
-	PIC16C57(config, "pic", 8000000);  // ?
-	m_ioasic->set_upper(468/* or 488 */);
+	PIC16C57(config, "pic", 8000000); // ?
+	m_ioasic->set_upper(468 /* or 488 */ );
 }
 
 void midzeus2_state::midzeus2(machine_config &config)
 {
 	// basic machine hardware
-	TMS32032(config, m_maincpu, CPU_CLOCK);
+	TMS320C32(config, m_maincpu, 60_MHz_XTAL);
 	m_maincpu->set_addrmap(AS_PROGRAM, &midzeus2_state::zeus2_map);
 	m_maincpu->set_vblank_int("screen", FUNC(midzeus2_state::display_irq));
 
@@ -1470,7 +1484,7 @@ void crusnexo_state::crusnexo(machine_config &config)
 	midzeus2(config);
 	m_maincpu->set_addrmap(AS_PROGRAM, &crusnexo_state::crusnexo_map);
 
-	m_ioasic->set_upper(472/* or 476,477,478,110 */);
+	m_ioasic->set_upper(472 /* or 476,477,478,110 */ );
 }
 
 void thegrid_state::thegrid(machine_config &config)
@@ -1478,8 +1492,8 @@ void thegrid_state::thegrid(machine_config &config)
 	midzeus2(config);
 	m_maincpu->set_addrmap(AS_PROGRAM, &thegrid_state::thegrid_map);
 
-	PIC16C57(config, "pic", 8000000).set_disable();  // unverified clock, not hooked up
-	m_ioasic->set_upper(474/* or 491 */);
+	PIC16C57(config, "pic", 8000000).set_disable(); // unverified clock, not hooked up
+	m_ioasic->set_upper(474 /* or 491 */ );
 }
 
 
@@ -1562,7 +1576,7 @@ ROM_START( invasnab ) // Version 5.0 Program ROMs, v4.0 Graphics ROMs, v2.0 Soun
 	ROM_LOAD16_BYTE( "invasion2.u5", 0xc00000, 0x200000, CRC(e42805c9) SHA1(e5b71eb1852809a649ac43a82168b3bdaf4b1526) )
 
 	ROM_REGION( 0x2000, "pic", 0 ) // PIC16c57 Code
-	ROM_LOAD( "pic16c57.u76", 0x00000, 0x2000, BAD_DUMP CRC(f62729c9) SHA1(9642c53dd7eceeb7eb178497d367691c44abc5c5) ) // is this even a valid dump?
+	ROM_LOAD( "468_invasion_25_u76.u76", 0x00000, 0x2000, CRC(eca69715) SHA1(b49787faf9d034ade65828ddbcd3197170a48123) ) // decapped but not hooked up
 
 	ROM_REGION32_LE( 0x1800000, "maindata", 0 )
 	ROM_LOAD32_WORD( "invasion5.u10", 0x0000000, 0x200000, CRC(8c7785d9) SHA1(701602314cd4eba4215c47ea0ae75fd4eddad43b) ) // ROMs U10 & U11 were labeled as v5.0
@@ -1585,7 +1599,7 @@ ROM_START( invasnab4 ) // Version 4.0 Program ROMs & Graphics ROMs, v2.0 Sound R
 	ROM_LOAD16_BYTE( "invasion2.u5", 0xc00000, 0x200000, CRC(e42805c9) SHA1(e5b71eb1852809a649ac43a82168b3bdaf4b1526) )
 
 	ROM_REGION( 0x2000, "pic", 0 ) // PIC16c57 Code
-	ROM_LOAD( "pic16c57.u76", 0x00000, 0x2000, BAD_DUMP CRC(f62729c9) SHA1(9642c53dd7eceeb7eb178497d367691c44abc5c5) ) // is this even a valid dump?
+	ROM_LOAD( "468_invasion_25_u76.u76", 0x00000, 0x2000, CRC(eca69715) SHA1(b49787faf9d034ade65828ddbcd3197170a48123) ) // decapped but not hooked up
 
 	ROM_REGION32_LE( 0x1800000, "maindata", 0 )
 	ROM_LOAD32_WORD( "invasion4.u10", 0x0000000, 0x200000, CRC(b3ce958b) SHA1(ed51c167d85bc5f6155b8046ec056a4f4ad5cf9d) ) // These ROM were all labeled as v4.0
@@ -1608,7 +1622,7 @@ ROM_START( invasnab3 ) // Version 3.0 Program ROMs & v2.0 Graphics ROMs, v2.0 So
 	ROM_LOAD16_BYTE( "invasion2.u5", 0xc00000, 0x200000, CRC(e42805c9) SHA1(e5b71eb1852809a649ac43a82168b3bdaf4b1526) )
 
 	ROM_REGION( 0x2000, "pic", 0 ) // PIC16c57 Code
-	ROM_LOAD( "pic16c57.u76", 0x00000, 0x2000, BAD_DUMP CRC(f62729c9) SHA1(9642c53dd7eceeb7eb178497d367691c44abc5c5) ) // is this even a valid dump?
+	ROM_LOAD( "468_invasion_25_u76.u76", 0x00000, 0x2000, CRC(eca69715) SHA1(b49787faf9d034ade65828ddbcd3197170a48123) ) // decapped but not hooked up
 
 	ROM_REGION32_LE( 0x1800000, "maindata", 0 )
 	ROM_LOAD32_WORD( "invasion3.u10", 0x0000000, 0x200000, CRC(8404830e) SHA1(808fea45fb09fb7bf60f9f1e195a51d39e9966f5) ) // ROMs U10 through U13 were labeled as v3.0 Dated 8/30
@@ -1968,9 +1982,11 @@ ROM_END
 GAME(  1997, mk4,        0,        mk4,      mk4,      midzeus_state,  empty_init, ROT0, "Midway", "Mortal Kombat 4 (version 3.0)",                       MACHINE_IMPERFECT_GRAPHICS | MACHINE_SUPPORTS_SAVE )
 GAME(  1997, mk4a,       mk4,      mk4,      mk4,      midzeus_state,  empty_init, ROT0, "Midway", "Mortal Kombat 4 (version 2.1)",                       MACHINE_IMPERFECT_GRAPHICS | MACHINE_SUPPORTS_SAVE )
 GAME(  1997, mk4b,       mk4,      mk4,      mk4,      midzeus_state,  empty_init, ROT0, "Midway", "Mortal Kombat 4 (version 1.0)",                       MACHINE_IMPERFECT_GRAPHICS | MACHINE_SUPPORTS_SAVE )
-GAME(  1999, invasnab,   0,        invasn,   invasn,   invasnab_state, empty_init, ROT0, "Midway", "Invasion - The Abductors (version 5.0)",              MACHINE_IMPERFECT_GRAPHICS | MACHINE_SUPPORTS_SAVE )
-GAME(  1999, invasnab4,  invasnab, invasn,   invasn,   invasnab_state, empty_init, ROT0, "Midway", "Invasion - The Abductors (version 4.0)",              MACHINE_IMPERFECT_GRAPHICS | MACHINE_SUPPORTS_SAVE )
-GAME(  1999, invasnab3,  invasnab, invasn,   invasn,   invasnab_state, empty_init, ROT0, "Midway", "Invasion - The Abductors (version 3.0)",              MACHINE_IMPERFECT_GRAPHICS | MACHINE_SUPPORTS_SAVE )
+
+GAME(  1999, invasnab,   0,        invasn,   invasn,   invasnab_state, empty_init, ROT0, "Midway", "Invasion: The Abductors (version 5.0)",               MACHINE_IMPERFECT_GRAPHICS | MACHINE_SUPPORTS_SAVE )
+GAME(  1999, invasnab4,  invasnab, invasn,   invasn,   invasnab_state, empty_init, ROT0, "Midway", "Invasion: The Abductors (version 4.0)",               MACHINE_IMPERFECT_GRAPHICS | MACHINE_SUPPORTS_SAVE )
+GAME(  1999, invasnab3,  invasnab, invasn,   invasn,   invasnab_state, empty_init, ROT0, "Midway", "Invasion: The Abductors (version 3.0)",               MACHINE_IMPERFECT_GRAPHICS | MACHINE_SUPPORTS_SAVE )
+
 GAMEL( 1999, crusnexo,   0,        crusnexo, crusnexo, crusnexo_state, empty_init, ROT0, "Midway", "Cruis'n Exotica (version 2.4)",                       MACHINE_NOT_WORKING | MACHINE_IMPERFECT_GRAPHICS | MACHINE_SUPPORTS_SAVE, layout_crusnexo )
 GAMEL( 1999, crusnexoa,  crusnexo, crusnexo, crusnexo, crusnexo_state, empty_init, ROT0, "Midway", "Cruis'n Exotica (version 2.0)",                       MACHINE_NOT_WORKING | MACHINE_IMPERFECT_GRAPHICS | MACHINE_SUPPORTS_SAVE, layout_crusnexo )
 GAMEL( 1999, crusnexoaa, crusnexo, crusnexo, crusnexo, crusnexo_state, empty_init, ROT0, "Midway", "Cruis'n Exotica (version 2.0, alternate ROM format)", MACHINE_NOT_WORKING | MACHINE_IMPERFECT_GRAPHICS | MACHINE_SUPPORTS_SAVE, layout_crusnexo )
@@ -1978,6 +1994,7 @@ GAMEL( 1999, crusnexob,  crusnexo, crusnexo, crusnexo, crusnexo_state, empty_ini
 GAMEL( 1999, crusnexoc,  crusnexo, crusnexo, crusnexo, crusnexo_state, empty_init, ROT0, "Midway", "Cruis'n Exotica (version 1.3)",                       MACHINE_NOT_WORKING | MACHINE_IMPERFECT_GRAPHICS | MACHINE_SUPPORTS_SAVE, layout_crusnexo )
 GAMEL( 1999, crusnexod,  crusnexo, crusnexo, crusnexo, crusnexo_state, empty_init, ROT0, "Midway", "Cruis'n Exotica (version 1.0, build 8764)",           MACHINE_NOT_WORKING | MACHINE_IMPERFECT_GRAPHICS | MACHINE_SUPPORTS_SAVE, layout_crusnexo ) //  8-Feb-2000
 GAMEL( 1999, crusnexoe,  crusnexo, crusnexo, crusnexo, crusnexo_state, empty_init, ROT0, "Midway", "Cruis'n Exotica (version 1.0, build 8643)",           MACHINE_NOT_WORKING | MACHINE_IMPERFECT_GRAPHICS | MACHINE_SUPPORTS_SAVE, layout_crusnexo ) // 31-Jan-2000
+
 GAME(  2000, thegrid,    0,        thegrid,  thegrid,  thegrid_state,  empty_init, ROT0, "Midway", "The Grid (version 1.2)",                              MACHINE_NOT_WORKING | MACHINE_IMPERFECT_GRAPHICS | MACHINE_SUPPORTS_SAVE ) // 10/16/00
 GAME(  2000, thegrida,   thegrid,  thegrid,  thegrid,  thegrid_state,  empty_init, ROT0, "Midway", "The Grid (version 1.1)",                              MACHINE_NOT_WORKING | MACHINE_IMPERFECT_GRAPHICS | MACHINE_SUPPORTS_SAVE ) // 07/26/00
 GAME(  2000, thegridb,   thegrid,  thegrid,  thegrid,  thegrid_state,  empty_init, ROT0, "Midway", "The Grid (version 1.01)",                             MACHINE_NOT_WORKING | MACHINE_IMPERFECT_GRAPHICS | MACHINE_SUPPORTS_SAVE ) // 07/17/00
