@@ -7,7 +7,8 @@
 #define LOG_IRQ   (1U << 1) // enable bits (verbose)
 #define LOG_TIMER (1U << 2)
 #define LOG_XBUS  (1U << 3)
-#define LOG_DSPP  (1U << 4)
+#define LOG_XBUSV (1U << 4) // verbose XBus stuff
+#define LOG_DSPP  (1U << 5)
 
 #define VERBOSE (LOG_GENERAL | LOG_XBUS | LOG_DSPP)
 //#define LOG_OUTPUT_FUNC osd_printf_warning
@@ -17,6 +18,7 @@
 #define LOGIRQ(...)   LOGMASKED(LOG_IRQ,     __VA_ARGS__)
 #define LOGTIMER(...) LOGMASKED(LOG_TIMER,   __VA_ARGS__)
 #define LOGXBUS(...)  LOGMASKED(LOG_XBUS,    __VA_ARGS__)
+#define LOGXBUSV(...) LOGMASKED(LOG_XBUSV,   __VA_ARGS__)
 #define LOGDSPP(...)  LOGMASKED(LOG_DSPP,    __VA_ARGS__)
 
 
@@ -29,8 +31,10 @@ clio_device::clio_device(const machine_config &mconfig, const char *tag, device_
 	, m_firq_cb(*this)
 	, m_vsync_cb(*this)
 	, m_hsync_cb(*this)
+	, m_xbus_sel_cb(*this)
 	, m_xbus_read_cb(*this, 0xff)
 	, m_xbus_write_cb(*this)
+	, m_exp_dma_enable_cb(*this)
 	, m_dac_l(*this)
 	, m_dac_r(*this)
 //  , m_adb_in_cb(*this)
@@ -41,7 +45,10 @@ clio_device::clio_device(const machine_config &mconfig, const char *tag, device_
 void clio_device::device_add_mconfig(machine_config &config)
 {
 	DSPP(config, m_dspp, DERIVED_CLOCK(1, 1));
-//  m_dspp->int_handler().set([this] (int state) { printf("%d\n", state); });
+	m_dspp->int_handler().set([this] (int state) {
+		if (state)
+			request_fiq<0>(1 << IRQ_DSPPINT);
+	});
 //  m_dspp->dma_read_handler().set(FUNC(m2_bda_device::read_bus8));
 //  m_dspp->dma_write_handler().set(FUNC(m2_bda_device::write_bus8));
 }
@@ -72,12 +79,10 @@ void clio_device::device_start()
 //  save_item(NAME(m_vcnt));
 	save_item(NAME(m_seed));
 	save_item(NAME(m_random));
-	save_item(NAME(m_irq0));
-	save_item(NAME(m_irq0_enable));
+	save_item(NAME(m_irq));
+	save_item(NAME(m_irq_enable));
 	save_item(NAME(m_mode));
 	save_item(NAME(m_badbits));
-	save_item(NAME(m_irq1));
-	save_item(NAME(m_irq1_enable));
 	save_item(NAME(m_hdelay));
 	save_item(NAME(m_adbio));
 	save_item(NAME(m_adbctl));
@@ -96,18 +101,22 @@ void clio_device::device_start()
 
 	save_item(NAME(m_sel));
 	save_item(NAME(m_poll));
+	save_item(NAME(m_xfrcnt));
+	save_item(NAME(m_xbus_dev));
 }
 
 void clio_device::device_reset()
 {
 	m_cstatbits = 0x01; /* bit 0 = reset of clio caused by power on */
 
-	m_irq0_enable = m_irq1_enable = 0;
-	m_irq0 = m_irq1 = 0;
+	m_irq_enable[0] = m_irq_enable[1] = 0;
+	m_irq[0] = m_irq[1] = 0;
 	m_timer_ctrl = 0;
 	m_vint0 = m_vint1 = 0xffff'ffff;
 	m_slack = 336;
 	m_adbio = 0x00;
+	std::fill(std::begin(m_xbus_dev), std::end(m_xbus_dev), 0x00);
+	m_xfrcnt = 0;
 	m_system_timer->adjust(attotime::from_ticks(m_slack, this->clock()));
 	m_scan_timer->adjust(m_screen->time_until_pos(0), 0);
 }
@@ -115,23 +124,59 @@ void clio_device::device_reset()
 void clio_device::dply_w(int state)
 {
 	if (state)
-		request_fiq(1 << 0, 1);
+		request_fiq<1>(1 << IRQ_PLYINT);
 }
 
 void clio_device::xbus_int_w(int state)
 {
-	if (state && m_poll & 7)
+	if (state)
 	{
-		request_fiq(1 << 2, 0);
+		m_xbus_dev[0] |= 0x10;
+		if (m_xbus_dev[0] & 1)
+			request_fiq<0>(1 << IRQ_EXINT);
 	}
+	else
+		m_xbus_dev[0] &= ~0x10;
+}
 
-	//if ((m_sel & 0x0f) == 0)
-	//{
-	//  if (state)
-	//      m_poll |= 0x10;
-	//  else
-	//      m_poll &= ~0x10;
-	//}
+void clio_device::xbus_wr_w(int state)
+{
+//  printf("%d WR %02x\n", state, m_xbus_dev[0]);
+	if (state)
+	{
+		m_xbus_dev[0] |= 0x20;
+		if (m_xbus_dev[0] & 2)
+			request_fiq<0>(1 << IRQ_EXINT);
+	}
+	else
+		m_xbus_dev[0] &= ~0x20;
+}
+
+void clio_device::dexp_w(int state)
+{
+	//printf("dexp_w %d\n", state);
+	if (state)
+	{
+		m_expctl &= ~(1 << 10);
+		request_fiq<0>(1 << IRQ_DEXINT);
+	}
+}
+
+void clio_device::arm_ctl_w(int state)
+{
+	if (state)
+	{
+		m_expctl |= 0x80;
+		// TODO: is this really a thing?
+		// avoids starting the DMA again (with count already underflowed),
+		// but BIOS clears this status on receiving dexp_w anyway ...
+		m_dma_enable &= ~(1 << 20);
+	}
+	else
+	{
+		m_expctl &= ~0x80;
+		m_expctl |= (1 << 10);
+	}
 }
 
 // $0340'0000 base
@@ -217,32 +262,36 @@ void clio_device::map(address_map &map)
 	);
 	// TODO: should likely follow seed number, and be truly RNG
 	map(0x003c, 0x003f).lr32(NAME([this] () {
-		LOG("random read (!)\n");
+		if (!machine().side_effects_disabled())
+			LOG("random read (!)\n");
 		return m_random;
 	}));
 
 	// interrupt control
 	map(0x0040, 0x0047).lrw32(
-		NAME([this] () { return m_irq0; }),
+		NAME([this] () { return m_irq[0]; }),
 		NAME([this] (offs_t offset, u32 data, u32 mem_mask) {
 			if (offset)
-				m_irq0 &= ~data;
+				m_irq[0] &= ~data;
 			else
-				m_irq0 |= data;
+				m_irq[0] |= data;
 
-			request_fiq(0, 0);
+			request_fiq<0>(0);
 		})
 	);
 	map(0x0048, 0x004f).lrw32(
-		NAME([this] () { return m_irq0_enable; }),
+		// TODO: second priority bit 31 should always read 1
+		// this however causes continous irq loop cycles during POST as soon as IRQ_PLYINT is requested
+		// (never actually enabled in irq1, just IRQ_BADBITS is)
+		NAME([this] () { return m_irq_enable[0]; }),
 		NAME([this] (offs_t offset, u32 data, u32 mem_mask) {
 			if (offset)
-				m_irq0_enable &= ~data;
+				m_irq_enable[0] &= ~data;
 			else
-				m_irq0_enable |= data;
+				m_irq_enable[0] |= data;
 
-			LOGIRQ("irq0 enable %s: %08x & %08x -> %08x\n", offset ? "clear" : "set", data, mem_mask, m_irq0_enable);
-			request_fiq(0, 0);
+			LOGIRQ("irq0 enable %s: %08x & %08x -> %08x\n", offset ? "clear" : "set", data, mem_mask, m_irq_enable[0]);
+			request_fiq<0>(0);
 		})
 	);
 	map(0x0050, 0x0057).lw32(
@@ -264,26 +313,26 @@ void clio_device::map(address_map &map)
 
 //  map(0x005c, 0x005f) unknown if used at all
 	map(0x0060, 0x0067).lrw32(
-		NAME([this] () { return m_irq1; }),
+		NAME([this] () { return m_irq[1]; }),
 		NAME([this] (offs_t offset, u32 data, u32 mem_mask) {
 			if (offset)
-				m_irq1 &= ~data;
+				m_irq[1] &= ~data;
 			else
-				m_irq1 |= data;
+				m_irq[1] |= data;
 
-			request_fiq(0, 1);
+			request_fiq<1>(0);
 		})
 	);
 	map(0x0068, 0x006f).lrw32(
-		NAME([this] () { return m_irq1_enable; }),
+		NAME([this] () { return m_irq_enable[1]; }),
 		NAME([this] (offs_t offset, u32 data, u32 mem_mask) {
 			if (offset)
-				m_irq1_enable &= ~data;
+				m_irq_enable[1] &= ~data;
 			else
-				m_irq1_enable |= data;
+				m_irq_enable[1] |= data;
 
-			LOGIRQ("irq1 enable %s: %08x & %08x -> %08x\n", offset ? "clear" : "set", data, mem_mask, m_irq1_enable);
-			request_fiq(0, 1);
+			LOGIRQ("irq1 enable %s: %08x & %08x -> %08x\n", offset ? "clear" : "set", data, mem_mask, m_irq_enable[1]);
+			request_fiq<1>(0);
 		})
 	);
 
@@ -393,21 +442,25 @@ void clio_device::map(address_map &map)
 			else
 				m_dma_enable |= data;
 			LOG("DMA request %s: %08x & %08x\n", offset ? "clear" : "set", data, mem_mask);
+			m_exp_dma_enable_cb(BIT(m_dma_enable, 20) && BIT(m_expctl, 11));
 		})
 	);
 //  map(0x0380, 0x0383) FIFO status
 
 	// XBus
 	map(0x0400, 0x0407).lrw32(
-		NAME([this] () { return m_expctl; }),
+		NAME([this] () {
+			return m_expctl;
+		}),
 		NAME([this] (offs_t offset, u32 data, u32 mem_mask) {
-			if (ACCESSING_BITS_8_15)
+			if (ACCESSING_BITS_0_15)
 			{
 				if (offset)
-					m_expctl &= ~(data & 0xca00);
+					m_expctl &= ~(data & 0xca80);
 				else
-					m_expctl |= (data & 0xca00);
+					m_expctl |= (data & 0xca80);
 				LOGXBUS("xbus expctl %s: %08x & %08x\n", offset ? "clear" : "set", data, mem_mask);
+				m_exp_dma_enable_cb(BIT(m_dma_enable, 20) && BIT(m_expctl, 11));
 			}
 		})
 	);
@@ -417,6 +470,13 @@ void clio_device::map(address_map &map)
 			COMBINE_DATA(&m_type0_4);
 		})
 	);
+	map(0x040c, 0x040f).lrw32(
+		NAME([this] () { return m_xfrcnt; }),
+		NAME([this] (offs_t offset, u32 data, u32 mem_mask) {
+			LOGXBUS("DMA xfercnt %08x & %08x\n", data, mem_mask);
+			COMBINE_DATA(&m_xfrcnt);
+		})
+	);
 	// TODO: these are identification bits
 	// x--- ---- ---- ---- active
 	// -x-- ---- ---- ---- "happened before reset"
@@ -441,7 +501,8 @@ void clio_device::map(address_map &map)
 			m_sel &= 0xff;
 			/* Start WRSEL cycle */
 
-			LOGXBUS("xbus sel: %02x & %08x\n", data, mem_mask);
+			m_xbus_sel_cb(data);
+			LOGXBUSV("xbus sel: %02x & %08x\n", data, mem_mask);
 			/* Detection of too many devices on the bus */
 			switch ( data & 0xff )
 			{
@@ -454,18 +515,37 @@ void clio_device::map(address_map &map)
 			}
 		})
 	);
+	/*
+	 * x--- ---- media access (read clear)
+	 * -x-- ---- write valid (r/o)
+	 * --x- ---- read valid (r/o)
+	 * ---x ---- status valid (r/o)
+	 * ---- x--- reset
+	 * ---- -x-- write irq enable
+	 * ---- --x- read irq enable
+	 * ---- ---x status irq enable
+	 */
 	map(0x0540, 0x057f).lrw32(
-		NAME([this] () {
-			// HACK: until I understand semantics
+		NAME([this] () -> u8 {
+			u8 res = m_poll;
+			//m_poll &= ~0x80;
 			if (m_sel == 0)
-				return (m_poll & 0xef) | (machine().rand() & 0x10);
-			return m_poll;
+				return m_xbus_dev[0]; //(res & 0xcf) | (m_xbus_dev[0] & 0x30);
+			return (res);
 		}),
 		NAME([this] (offs_t offset, u32 data, u32 mem_mask) {
-			LOGXBUS("xbus poll: %02x & %08x\n", data, mem_mask);
-			COMBINE_DATA(&m_poll);
-			m_poll &= 0xf8;
-			m_poll |= data & 7;
+			LOGXBUS("xbus poll: %02x & %08x (sel=%d)\n", data, mem_mask, m_sel);
+			//COMBINE_DATA(&m_poll);
+			//m_poll &= 0xf8;
+			if (ACCESSING_BITS_0_7)
+			{
+				if (m_sel == 0)
+				{
+					m_xbus_dev[0] = (data & 0xf) | (m_xbus_dev[0] & 0xf0);
+				}
+				else
+					m_poll = data & 0xff;
+			}
 		})
 	);
 	// 1--- 1111 external device
@@ -478,20 +558,24 @@ void clio_device::map(address_map &map)
 
 	// TODO: should really map these directly in DSPP core
 //  map(0x17d0, 0x17d3) Semaphore
-	// HACK: for 3do_gdo101
+	// HACK: temporary to allow 3do_gdo101 boot
 	map(0x17d0, 0x17d3).lr32(NAME([] () { return 0x0004'0000; }));
 //  map(0x17d4, 0x17d7) Semaphore ACK
-//  map(0x17e0, 0x17ff) DSPP DMA and state
+//  map(0x17e0, 0x17e3) DSPP DMA
+//  map(0x17e4, 0x17e7) DSPPRST0 (use current reload)
+	// DSPPRST1 (set 568 tick cycle)
 	map(0x17e8, 0x17eb).lw32(
 		NAME([this] (offs_t offset, u32 data, u32 mem_mask) {
-			// reset?
-			m_dspp->write(0x6074 >> 2, 1);
-			m_dspp->write(0x6074 >> 2, 0);
+			//m_dspp->write(0x6074 >> 2, 1);
+			//m_dspp->write(0x6074 >> 2, 0);
 			//m_dspp->write((0x1300 + 8) >> 2, 568);
 			//m_dspp->write((0x1340 + 8) >> 2, 568);
 			LOGDSPP("DSPP $17e8 %08x & %08x\n", data, mem_mask);
 		})
 	);
+//  map(0x17f0, 0x17f3) Read noise value (Red only?)
+//  map(0x17f4, 0x17f7) Read DSPP PC (bits 15:0 only)
+//  map(0x17f8, 0x17fb) Read DSPP NR (bits 15:0 only)
 	/*
 	---- x--- DSPPError
 	---- -x-- DSPPReset
@@ -507,111 +591,69 @@ void clio_device::map(address_map &map)
 	map(0x17fc, 0x17ff).lw32(
 		NAME([this] (offs_t offset, u32 data, u32 mem_mask) {
 			LOGDSPP("DSPP $17fc %08x & %08x\n", data, mem_mask);
-			m_dspp->write(0x6070 >> 2, data);
-			//if (data & 1)
-			//  machine().debug_break();
+			m_dspp->host_gw_control_write(0, data);
 		})
 	);
-	// DSPP N paths (code)
+	// DSPP iNstruction paths (code)
 	map(0x1800, 0x1fff).lw32(
 		NAME([this] (offs_t offset, u32 data, u32 mem_mask) {
 			if (ACCESSING_BITS_16_31)
-				m_dspp->write((offset << 1) + 0, data >> 16);
+				m_dspp->host_n_write((offset << 1) + 0, data >> 16);
 			if (ACCESSING_BITS_0_15)
-				m_dspp->write((offset << 1) + 1, data & 0xffff);
+				m_dspp->host_n_write((offset << 1) + 1, data & 0xffff);
 		})
 	);
 	map(0x2000, 0x2fff).lw32(
 		NAME([this] (offs_t offset, u32 data, u32 mem_mask) {
 			// code path
 			if (ACCESSING_BITS_0_15)
-				m_dspp->write(offset, data);
+				m_dspp->host_n_write(offset, data);
 		})
 	);
-	// EI paths (data writes)
+	// EI paths (data writes from host)
 	map(0x3000, 0x31ff).lw32(
 		NAME([this] (offs_t offset, u32 data, u32 mem_mask) {
 			if (ACCESSING_BITS_16_31)
-				m_dspp->write(((offset<<1) + 0) | 0x400, data);
+				m_dspp->host_ei_write(((offset<<1) + 0), data);
 			if (ACCESSING_BITS_0_15)
-				m_dspp->write(((offset<<1) + 1) | 0x400, data);
+				m_dspp->host_ei_write(((offset<<1) + 1), data);
 		})
 	);
 	map(0x3400, 0x37ff).lw32(
 		NAME([this] (offs_t offset, u32 data, u32 mem_mask) {
 			if (ACCESSING_BITS_0_15)
-				m_dspp->write(offset | 0x400, data);
+				m_dspp->host_ei_write(offset, data);
 		})
 	);
-	// EO paths (data reads)
+	// EO paths (data reads from host)
 	map(0x3800, 0x39ff).lr32(
 		NAME([this] (offs_t offset) {
 			uint32_t res = 0;
-			res =  m_dspp->read(((offset << 1) + 0) | 0x400) << 16;
-			res |= m_dspp->read(((offset << 1) + 1) | 0x400);
+			res =  m_dspp->host_eo_read(((offset << 1) + 0)) << 16;
+			res |= m_dspp->host_eo_read(((offset << 1) + 1));
 			return res;
 		})
 	);
 	map(0x3c00, 0x3fff).lr32(
 		NAME([this] (offs_t offset) {
-			return m_dspp->read(offset | 0x400);
+			return m_dspp->host_eo_read(offset);
 		})
 	);
 }
 
-/*
-IRQ0
-0x80000000 Second Priority
-0x40000000 SW irq
-0x20000000 DMA<->EXP
-0x1fff0000 DMA RAM->DSPP *
-0x0000f000 DMA DSPP->RAM *
-0x00000800 DSPP
-0x00000400 Timer  1
-0x00000200 Timer  3 <- needed to surpass current hang point
-0x00000100 Timer  5
-0x00000080 Timer  7
-0x00000040 Timer  9
-0x00000020 Timer 11
-0x00000010 Timer 13
-0x00000008 Timer 15
-0x00000004 Expansion Bus
-0x00000002 Vertical 1
-0x00000001 Vertical 0
-
----
-IRQ1
-0x00000400 DSPPOVER (Red rev. only)
-0x00000200 DSPPUNDER (Red rev. only)
-0x00000100 BadBits
-0x00000080 DMA<-External
-0x00000040 DMA->External
-0x00000020 DMA<-Uncle
-0x00000010 DMA->Uncle
-0x00000008 DMA RAM->DSPP N
-0x00000004 SlowBus
-0x00000002 Disk Inserted
-0x00000001 DMA Player bus
-
-*/
-void clio_device::request_fiq(uint32_t irq_req, uint8_t type)
+template <unsigned N> void clio_device::request_fiq(uint32_t irq_req)
 {
-	if(type)
-		m_irq1 |= irq_req;
+	if (N)
+		m_irq[1] |= irq_req;
 	else
-		m_irq0 |= irq_req;
+		m_irq[0] |= irq_req;
 
-	if(m_irq1)
-		m_irq0 |= 1 << 31; // Second Priority
+	if(m_irq[1])
+		m_irq[0] |= 1 << IRQ_SCNDPINT; // Second Priority
 	else
-		m_irq0 &= ~(1 << 31);
+		m_irq[0] &= ~(1 << IRQ_SCNDPINT);
 
-	if((m_irq0 & m_irq0_enable) || (m_irq1 & m_irq1_enable))
-	{
-		//printf("Go irq %08x & %08x %08x & %08x\n",m_irq0, m_irq0_enable, m_irq1, m_irq1_enable);
-		m_firq_cb(1);
-		//m_maincpu->pulse_input_line(arm7_cpu_device::ARM7_FIRQ_LINE, m_maincpu->minimum_quantum_time());
-	}
+	m_firq_cb((m_irq[0] & m_irq_enable[0]) || (m_irq[1] & m_irq_enable[1]));
 }
 
 // TODO: this actually generates from Amy not from Clio
@@ -622,7 +664,7 @@ TIMER_CALLBACK_MEMBER(clio_device::scan_timer_cb)
 	// TODO: does it triggers on odd fields only?
 	if (scanline == m_vint1 && m_screen->frame_number() & 1)
 	{
-		request_fiq(1 << 1, 0);
+		request_fiq<0>(1 << IRQ_VINT1);
 	}
 
 	// 22, 262
@@ -655,6 +697,10 @@ TIMER_CALLBACK_MEMBER(clio_device::scan_timer_cb)
  */
 TIMER_CALLBACK_MEMBER( clio_device::system_timer_cb )
 {
+	static const u32 timer_irqs[8] = {
+		1 << IRQ_TIMINT1, 1 << IRQ_TIMINT3,  1 << IRQ_TIMINT5,  1 << IRQ_TIMINT7,
+		1 << IRQ_TIMINT9, 1 << IRQ_TIMINT11, 1 << IRQ_TIMINT13, 1 << IRQ_TIMINT15
+	};
 	u8 timer_flag;
 	u8 carry_val;
 
@@ -678,7 +724,7 @@ TIMER_CALLBACK_MEMBER( clio_device::system_timer_cb )
 			{
 				// only odd numbered timers causes irqs
 				if(i & 1)
-					request_fiq(8 << (7 - (i >> 1)), 0);
+					request_fiq<0>(timer_irqs[i >> 1]);
 
 				carry_val = 1;
 
@@ -697,6 +743,10 @@ TIMER_CALLBACK_MEMBER( clio_device::system_timer_cb )
 
 	// Opera specification goes this lengthy explaination about "64" being the unit of time
 	// but 3do_fc2 and 3do_gdo101 won't boot with a timer tick this small ...
+	// TODO: causes "PAL system detected" in most BIOSes
+	// - slack may just be the start time
+	// - in an ideal world we would split this implementation in separate timers,
+	//   however that will give more scheduler roundtrips, in an already crowded scenario ...
 	m_system_timer->adjust(attotime::from_ticks(m_slack, this->clock()));
 }
 
