@@ -1,5 +1,5 @@
 // license:BSD-3-Clause
-// copyright-holders:R. Belmont, Olivier Galibert, ElSemi, Angelo Salese
+// copyright-holders:R. Belmont, Olivier Galibert, ElSemi, Angelo Salese, Matthew Daniels
 #ifndef MAME_SEGA_MODEL2_H
 #define MAME_SEGA_MODEL2_H
 
@@ -39,11 +39,13 @@ class model2_state : public driver_device
 public:
 	struct plane;
 	struct texture_parameter;
-	struct triangle;
+	struct polygon;
 	struct quad_m2;
 	struct raster_state;
 	struct geo_state;
 
+	static constexpr feature_type imperfect_features() { return feature::GRAPHICS; }
+
 	model2_state(const machine_config &mconfig, device_type type, const char *tag) :
 		driver_device(mconfig, type, tag),
 		m_textureram0(*this, "textureram0"),
@@ -72,7 +74,8 @@ public:
 		m_copro_data(*this, "copro_data"),
 		m_in0(*this, "IN0"),
 		m_gears(*this, "GEARS"),
-		m_lightgun_ports(*this, {"P1_Y", "P1_X", "P2_Y", "P2_X"})
+		m_lightgun_ports(*this, {"P1_Y", "P1_X", "P2_Y", "P2_X"}),
+		m_lamps(*this, "lamp%u", 0U)
 	{ }
 
 	/* Public for access by the rendering functions */
@@ -80,17 +83,13 @@ public:
 	required_shared_ptr<u32> m_textureram1;
 	std::unique_ptr<u16[]> m_palram;
 	std::unique_ptr<u16[]> m_colorxlat;
-	std::unique_ptr<u16[]> m_lumaram;
+	std::unique_ptr<u8[]> m_lumaram;
 	u8 m_gamma_table[256]{};
-	std::unique_ptr<model2_renderer> m_poly;
+	std::unique_ptr<model2_renderer> m_renderer;
 
 	/* Public for access by the ioports */
 	ioport_value daytona_gearbox_r();
 
-	/* Public for access by MCFG */
-	TIMER_DEVICE_CALLBACK_MEMBER(model2_interrupt);
-	u16 crypt_read_callback(u32 addr);
-
 
 	/* Public for access by GAME() */
 	void init_overrev();
@@ -102,9 +101,11 @@ public:
 	void init_sgt24h();
 	void init_srallyc();
 	void init_powsledm();
+	void lamp_output_w(u8 data);
 
 protected:
 	virtual void machine_start() override ATTR_COLD;
+	virtual void video_start() override ATTR_COLD;
 	virtual void machine_reset() override ATTR_COLD;
 
 	required_shared_ptr<u32> m_workram;
@@ -136,6 +137,7 @@ protected:
 	required_ioport m_in0;
 	optional_ioport m_gears;
 	optional_ioport_array<4> m_lightgun_ports;
+	output_finder<6> m_lamps;
 
 	u32 m_timervals[4]{};
 	u32 m_timerorig[4]{};
@@ -143,20 +145,23 @@ protected:
 	int m_ctrlmode = 0;
 	u16 m_cmd_data = 0;
 	u8 m_driveio_comm_data = 0;
-	int m_iop_write_num = 0;
-	u32 m_iop_data = 0;
+	emu_timer *m_irq_delay_timer;
 
 	u32 m_geo_read_start_address = 0;
 	u32 m_geo_write_start_address = 0;
 	std::unique_ptr<raster_state> m_raster;
 	std::unique_ptr<geo_state> m_geo;
 	bitmap_rgb32 m_sys24_bitmap;
-//  u32 m_soundack;
-	void model2_check_irq_state();
-	void model2_check_irqack_state(u32 data);
 	u8 m_gearsel = 0;
 	u8 m_lightgun_mux = 0;
 
+	int m_prot_a = 0;
+
+	u32 m_intreq = 0;
+	u32 m_intena = 0;
+	u32 m_coproctl = 0;
+	u32 m_coprocnt = 0;
+
 	// Coprocessor communications
 	u32 copro_prg_r();
 	u32 copro_ctl1_r();
@@ -179,7 +184,7 @@ protected:
 	void colorxlat_w(offs_t offset, u16 data, u16 mem_mask = ~0);
 	void eeprom_w(u8 data);
 	u8 in0_r();
-	u32 fifo_control_2a_r();
+	u32 fifo_control_r();
 	u32 videoctl_r();
 	void videoctl_w(offs_t offset, u32 data, u32 mem_mask = ~0);
 	u8 rchase2_drive_board_r();
@@ -193,21 +198,20 @@ protected:
 	void irq_ack_w(u32 data);
 	u32 irq_enable_r();
 	void irq_enable_w(offs_t offset, u32 data, u32 mem_mask = ~0);
+	void irq_update();
 	u8 model2_serial_r(offs_t offset);
 	void model2_serial_w(offs_t offset, u8 data);
 	void horizontal_sync_w(u16 data);
 	void vertical_sync_w(u16 data);
 	u32 doa_prot_r(offs_t offset, u32 mem_mask = ~0);
 	u32 doa_unk_r();
-	void sega_0229_map(address_map &map) ATTR_COLD;
-	int m_prot_a = 0;
 
 	void raster_init(memory_region *texture_rom);
 	void geo_init(memory_region *polygon_rom);
 	u32 render_mode_r();
 	void render_mode_w(u32 data);
-	u16 lumaram_r(offs_t offset);
-	void lumaram_w(offs_t offset, u16 data, u16 mem_mask = ~0);
+	u8 lumaram_r(offs_t offset);
+	void lumaram_w(offs_t offset, u8 data);
 	u16 fbvram_bankA_r(offs_t offset);
 	void fbvram_bankA_w(offs_t offset, u16 data, u16 mem_mask = ~0);
 	u16 fbvram_bankB_r(offs_t offset);
@@ -222,23 +226,27 @@ protected:
 	void driveio_port_w(u8 data);
 	void push_geo_data(u32 data);
 	void reset_model2_scsp();
-	u32 screen_update_model2(screen_device &screen, bitmap_rgb32 &bitmap, const rectangle &cliprect);
-//  void screen_vblank_model2(int state);
-//  void sound_ready_w(int state);
+	u32 screen_update(screen_device &screen, bitmap_rgb32 &bitmap, const rectangle &cliprect);
+	void screen_vblank(int state);
+	void sound_ready_w(int state);
 	template <int TNum> TIMER_DEVICE_CALLBACK_MEMBER(model2_timer_cb);
 	void scsp_irq(offs_t offset, u8 data);
+	TIMER_CALLBACK_MEMBER(irq_mask_delayed_update);
 
-	void model2_3d_frame_start();
+	u16 crypt_read_callback(u32 addr);
+
+	void render_frame_start();
 	void geo_parse();
-	void model2_3d_frame_end( bitmap_rgb32 &bitmap, const rectangle &cliprect );
-	void draw_framebuffer(bitmap_rgb32 &bitmap, const rectangle &cliprect );
+	void render_polygons(bitmap_rgb32 &bitmap, const rectangle &cliprect);
+	void draw_framebuffer(bitmap_rgb32 &bitmap, const rectangle &cliprect);
 
-	void model2_timers(machine_config &config);
-	void model2_screen(machine_config &config);
-	void model2_scsp(machine_config &config);
+	void model2_timers(machine_config &config) ATTR_COLD;
+	void model2_screen(machine_config &config) ATTR_COLD;
+	void model2_scsp(machine_config &config) ATTR_COLD;
 
-	void sj25_0207_01(machine_config &config);
+	void sj25_0207_01(machine_config &config) ATTR_COLD;
 
+	void sega_0229_map(address_map &map) ATTR_COLD;
 	void drive_io_map(address_map &map) ATTR_COLD;
 	void drive_map(address_map &map) ATTR_COLD;
 	void geo_sharc_map(address_map &map) ATTR_COLD;
@@ -251,30 +259,24 @@ protected:
 	void debug_init();
 	void debug_commands(const std::vector<std::string_view> &params);
 	void debug_geo_dasm_command(const std::vector<std::string_view> &params);
-	void debug_tri_dump_command(const std::vector<std::string_view> &params);
+	void debug_poly_dump_command(const std::vector<std::string_view> &params);
 	void debug_help_command(const std::vector<std::string_view> &params);
 
-	virtual void video_start() override ATTR_COLD;
-
-	u32 m_intreq = 0;
-	u32 m_intena = 0;
-	u32 m_coproctl = 0;
-	u32 m_coprocnt = 0;
-
 	virtual void copro_halt() = 0;
 	virtual void copro_boot() = 0;
 
 private:
-	void tri_list_dump(FILE *dst);
-
 	u32 m_geoctl = 0;
 	u32 m_geocnt = 0;
 	u32 m_videocontrol = 0;
+	u32 m_framenum = 0;
 
 	bool m_render_unk = false;
 	bool m_render_mode = false;
 	bool m_render_test_mode = false;
+	bool m_render_done = false;
 	int16_t m_crtc_xoffset = 0, m_crtc_yoffset = 0;
+	bool m_palette_dirty = false;
 
 	u32 *geo_process_command( geo_state *geo, u32 opcode, u32 *input, bool *end_code );
 	// geo commands
@@ -307,15 +309,15 @@ private:
 
 	// raster functions
 	// main data input port
-	void model2_3d_push( raster_state *raster, u32 input );
-	// quad & triangle push paths
-	void model2_3d_process_quad( raster_state *raster, u32 attr );
-	void model2_3d_process_triangle( raster_state *raster, u32 attr );
+	void model2_3d_push(raster_state *raster, u32 input);
+	// polygon push path
+	template <unsigned NumVerts>
+	void model2_3d_process_polygon(raster_state *raster, u32 attr);
 
 	// inliners
-	inline void model2_3d_project( triangle *tri );
-	inline u16 float_to_zval( float floatval );
-	inline bool check_culling( raster_state *raster, u32 attr, float min_z, float max_z );
+	inline void model2_3d_project(polygon *poly);
+	inline u16 float_to_zval(float floatval, s32 z_adjust);
+	inline bool check_culling(raster_state *raster, u32 attr, float min_z, float max_z);
 };
 
 /*****************************
@@ -327,12 +329,12 @@ private:
 class model2_tgp_state : public model2_state
 {
 public:
-	model2_tgp_state(const machine_config &mconfig, device_type type, const char *tag)
-		: model2_state(mconfig, type, tag),
-		  m_copro_tgp(*this, "copro_tgp"),
-		  m_copro_tgp_program(*this, "copro_tgp_program"),
-		  m_copro_tgp_tables(*this, "copro_tgp_tables"),
-		  m_copro_tgp_bank(*this, "copro_tgp_bank")
+	model2_tgp_state(const machine_config &mconfig, device_type type, const char *tag) :
+		model2_state(mconfig, type, tag),
+		m_copro_tgp(*this, "copro_tgp"),
+		m_copro_tgp_program(*this, "copro_tgp_program"),
+		m_copro_tgp_tables(*this, "copro_tgp_tables"),
+		m_copro_tgp_bank(*this, "copro_tgp_bank")
 	{}
 
 protected:
@@ -389,17 +391,16 @@ protected:
 class model2o_state : public model2_tgp_state
 {
 public:
-	model2o_state(const machine_config &mconfig, device_type type, const char *tag)
-		: model2_tgp_state(mconfig, type, tag)
+	model2o_state(const machine_config &mconfig, device_type type, const char *tag) :
+		model2_tgp_state(mconfig, type, tag)
 	{}
 
-	void model2o(machine_config &config);
-	void daytona(machine_config &config);
-	void desert(machine_config &config);
-	void vcop(machine_config &config);
+	void model2o(machine_config &config) ATTR_COLD;
+	void daytona(machine_config &config) ATTR_COLD;
+	void desert(machine_config &config) ATTR_COLD;
+	void vcop(machine_config &config) ATTR_COLD;
 
 protected:
-	u32 fifo_control_2o_r();
 	void daytona_output_w(u8 data);
 	void desert_output_w(u8 data);
 	void vcop_output_w(u8 data);
@@ -416,16 +417,21 @@ protected:
 class model2o_maxx_state : public model2o_state
 {
 public:
-	model2o_maxx_state(const machine_config &mconfig, device_type type, const char *tag)
-		: model2o_state(mconfig, type, tag)
+	model2o_maxx_state(const machine_config &mconfig, device_type type, const char *tag) :
+		model2o_state(mconfig, type, tag)
 	{}
 
-	u32 maxx_r(offs_t offset, u32 mem_mask = ~0);
-	void daytona_maxx(machine_config &config);
 	void model2o_maxx_mem(address_map &map) ATTR_COLD;
 
+	void daytona_maxx(machine_config &config) ATTR_COLD;
+
+protected:
+	virtual void machine_start() override ATTR_COLD;
+
 private:
-	int m_maxxstate = 0;
+	u8 m_maxxstate = 0;
+
+	u32 maxx_r(offs_t offset, u32 mem_mask = ~0);
 };
 
 /*****************************
@@ -437,19 +443,23 @@ private:
 class model2o_gtx_state : public model2o_state
 {
 public:
-	model2o_gtx_state(const machine_config &mconfig, device_type type, const char *tag)
-		: model2o_state(mconfig, type, tag)
+	model2o_gtx_state(const machine_config &mconfig, device_type type, const char *tag) :
+		model2o_state(mconfig, type, tag),
+		m_prot_data(*this, "prot_data")
 	{}
 
-	void daytona_gtx(machine_config &config);
+	void daytona_gtx(machine_config &config) ATTR_COLD;
 
 protected:
+	virtual void machine_start() override ATTR_COLD;
 	virtual void machine_reset() override ATTR_COLD;
 
 private:
-	int m_gtx_state = 0;
+	required_region_ptr<u32> m_prot_data;
+	u8 m_gtx_state = 0;
 
 	u8 gtx_r(offs_t offset);
+
 	void model2o_gtx_mem(address_map &map) ATTR_COLD;
 };
 
@@ -462,20 +472,22 @@ private:
 class model2a_state : public model2_tgp_state
 {
 public:
-	model2a_state(const machine_config &mconfig, device_type type, const char *tag)
-		: model2_tgp_state(mconfig, type, tag),
-		  m_billboard(*this, "billboard")
+	static constexpr feature_type imperfect_features() { return feature::GRAPHICS | feature::SOUND; }
+
+	model2a_state(const machine_config &mconfig, device_type type, const char *tag) :
+		model2_tgp_state(mconfig, type, tag),
+		m_billboard(*this, "billboard")
 	{}
 
-	void manxtt(machine_config &config);
-	void manxttdx(machine_config &config);
-	void model2a(machine_config &config);
-	void model2a_0229(machine_config &config);
-	void model2a_5881(machine_config &config);
-	void srallyc(machine_config &config);
-	void vcop2(machine_config &config);
-	void skytargt(machine_config &config);
-	void zeroguna(machine_config &config);
+	void manxtt(machine_config &config) ATTR_COLD;
+	void manxttdx(machine_config &config) ATTR_COLD;
+	void model2a(machine_config &config) ATTR_COLD;
+	void model2a_0229(machine_config &config) ATTR_COLD;
+	void model2a_5881(machine_config &config) ATTR_COLD;
+	void srallyc(machine_config &config) ATTR_COLD;
+	void vcop2(machine_config &config) ATTR_COLD;
+	void skytargt(machine_config &config) ATTR_COLD;
+	void zeroguna(machine_config &config) ATTR_COLD;
 
 protected:
 	virtual void machine_reset() override ATTR_COLD;
@@ -491,16 +503,19 @@ private:
 class model2a_airwlkrs_state : public model2a_state
 {
 public:
-	model2a_airwlkrs_state(const machine_config &mconfig, device_type type, const char *tag)
-		: model2a_state(mconfig, type, tag),
-		  m_player_in(*this, "IN_P%u", 1U),
-		  m_start_in(*this, "IN_START")
+	model2a_airwlkrs_state(const machine_config &mconfig, device_type type, const char *tag) :
+		model2a_state(mconfig, type, tag),
+		m_player_in(*this, "IN_P%u", 1U),
+		m_start_in(*this, "IN_START")
 	{}
 
-	void airwlkrs(machine_config &config);
+	void airwlkrs(machine_config &config) ATTR_COLD;
 
 	template <unsigned N> ioport_value start_in_r();
 
+protected:
+	virtual void machine_start() override ATTR_COLD;
+
 private:
 	required_ioport_array<4> m_player_in;
 	required_ioport m_start_in;
@@ -516,22 +531,23 @@ private:
 class model2b_state : public model2_state
 {
 public:
-	model2b_state(const machine_config &mconfig, device_type type, const char *tag)
-		: model2_state(mconfig, type, tag),
-		  m_copro_adsp(*this, "copro_adsp"),
-		  m_billboard(*this, "billboard")
+	static constexpr feature_type imperfect_features() { return feature::GRAPHICS | feature::SOUND; }
+
+	model2b_state(const machine_config &mconfig, device_type type, const char *tag) :
+		model2_state(mconfig, type, tag),
+		m_copro_adsp(*this, "copro_adsp"),
+		m_billboard(*this, "billboard")
 	{}
 
-	void model2b(machine_config &config);
-	void model2b_0229(machine_config &config);
-	void model2b_5881(machine_config &config);
-	void indy500(machine_config &config);
-	void overrev2b(machine_config &config);
-	void powsled(machine_config &config);
-	void rchase2(machine_config &config);
-	void gunblade(machine_config &config);
-	void dynabb(machine_config &config);
-	void zerogun(machine_config &config);
+	void model2b(machine_config &config) ATTR_COLD;
+	void model2b_0229(machine_config &config) ATTR_COLD;
+	void model2b_5881(machine_config &config) ATTR_COLD;
+	void indy500(machine_config &config) ATTR_COLD;
+	void powsled(machine_config &config) ATTR_COLD;
+	void rchase2(machine_config &config) ATTR_COLD;
+	void gunblade(machine_config &config) ATTR_COLD;
+	void dynabb(machine_config &config) ATTR_COLD;
+	void zerogun(machine_config &config) ATTR_COLD;
 
 protected:
 	virtual void machine_start() override ATTR_COLD;
@@ -542,7 +558,6 @@ protected:
 	void copro_function_port_w(offs_t offset, u32 data);
 	u32 copro_fifo_r();
 	void copro_fifo_w(u32 data);
-	void copro_sharc_iop_w(offs_t offset, u32 data);
 	u32 copro_sharc_buffer_r(offs_t offset);
 	void copro_sharc_buffer_w(offs_t offset, u32 data);
 
@@ -571,22 +586,25 @@ private:
 class model2c_state : public model2_state
 {
 public:
-	model2c_state(const machine_config &mconfig, device_type type, const char *tag)
-		: model2_state(mconfig, type, tag),
-		  m_copro_tgpx4(*this, "copro_tgpx4"),
-		  m_copro_tgpx4_program(*this, "copro_tgpx4_program")
-	{}
+	static constexpr feature_type imperfect_features() { return feature::GRAPHICS | feature::SOUND; }
 
-	void model2c(machine_config &config);
-	void model2c_5881(machine_config &config);
-	void skisuprg(machine_config &config);
-	void stcc(machine_config &config);
-	void waverunr(machine_config &config);
-	void bel(machine_config &config);
-	void hotd(machine_config &config);
-	void overrev2c(machine_config &config);
-	void segawski(machine_config &config);
-	void topskatr(machine_config &config);
+	model2c_state(const machine_config &mconfig, device_type type, const char *tag) :
+		model2_state(mconfig, type, tag),
+		m_copro_tgpx4(*this, "copro_tgpx4"),
+		m_copro_tgpx4_program(*this, "copro_tgpx4_program")
+	{
+	}
+
+	void model2c(machine_config &config) ATTR_COLD;
+	void model2c_5881(machine_config &config) ATTR_COLD;
+	void skisuprg(machine_config &config) ATTR_COLD;
+	void stcc(machine_config &config) ATTR_COLD;
+	void waverunr(machine_config &config) ATTR_COLD;
+	void bel(machine_config &config) ATTR_COLD;
+	void hotd(machine_config &config) ATTR_COLD;
+	void overrev2c(machine_config &config) ATTR_COLD;
+	void segawski(machine_config &config) ATTR_COLD;
+	void topskatr(machine_config &config) ATTR_COLD;
 
 protected:
 	virtual void machine_start() override ATTR_COLD;
@@ -599,8 +617,6 @@ protected:
 	u32 copro_fifo_r();
 	void copro_fifo_w(u32 data);
 
-	TIMER_DEVICE_CALLBACK_MEMBER(model2c_interrupt);
-
 	void model2c_crx_mem(address_map &map) ATTR_COLD;
 	void model2c_5881_mem(address_map &map) ATTR_COLD;
 	void copro_tgpx4_map(address_map &map) ATTR_COLD;
@@ -621,135 +637,70 @@ struct m2_poly_extra_data
 	model2_state *  state;
 	u32      lumabase;
 	u32      colorbase;
-	u32 *    texsheet;
+	u8       checker;
+	u32 *    texsheet[2];
 	u32      texwidth;
 	u32      texheight;
-	u32      texx, texy;
+	u32      texx;
+	u32      texy;
+	u8       texwrapx;
+	u8       texwrapy;
 	u8       texmirrorx;
 	u8       texmirrory;
+	u8       utex;
+	u8       utexminlod;
+	u32      utexx;
+	u32      utexy;
+	s32      texlod;
 	u8       luma;
 };
 
-
-static inline u16 get_texel( u32 base_x, u32 base_y, int x, int y, u32 *sheet )
-{
-	int x2 = base_x + x;
-	int y2 = base_y + y;
-	if (x2 >= 1024)
-	{
-		// texture sheets are mapped as 2048x1024 but stored in RAM as 1024x2048
-		x2 -= 1024;
-		y2 ^= 1024;
-	}
-	u32  offset = ((y2 / 2) * 512) + (x2 / 2);
-	u32  texel = sheet[offset>>1];
-
-	if ( offset & 1 )
-		texel >>= 16;
-
-	if ( (y & 1) == 0 )
-		texel >>= 8;
-
-	if ( (x & 1) == 0 )
-		texel >>= 4;
-
-	return (texel & 0x0f);
-}
-
-// 0x10000 = size of the tri_sorted_list array
+// 0x10000 = size of the poly_sorted_list array
 class model2_renderer : public poly_manager<float, m2_poly_extra_data, 4>
 {
 public:
-	typedef void (model2_renderer::*scanline_render_func)(int32_t scanline, const extent_t& extent, const m2_poly_extra_data& object, int threadid);
+	using polygon = model2_state::polygon;
 
-public:
-	using triangle = model2_state::triangle;
-
-	model2_renderer(model2_state& state)
-		: poly_manager<float, m2_poly_extra_data, 4>(state.machine())
-		, m_state(state)
-		, m_destmap(512, 512)
+	model2_renderer(model2_state& state) :
+		poly_manager<float, m2_poly_extra_data, 4>(state.machine()),
+		m_render_callbacks{
+				{ &model2_renderer::draw_scanline_solid<false>, this },
+				{ &model2_renderer::draw_scanline_solid<true>, this },
+				{ &model2_renderer::draw_scanline_tex<false>, this },
+				{ &model2_renderer::draw_scanline_tex<true>, this } },
+		m_state(state),
+		m_destmap(512, 512),
+		m_fillmap(512, 512),
+		m_xoffs(90),
+		m_yoffs(-8)
 	{
-		m_renderfuncs[0] = &model2_renderer::model2_3d_render_0;
-		m_renderfuncs[1] = &model2_renderer::model2_3d_render_1;
-		m_renderfuncs[2] = &model2_renderer::model2_3d_render_2;
-		m_renderfuncs[3] = &model2_renderer::model2_3d_render_3;
-		m_renderfuncs[4] = &model2_renderer::model2_3d_render_4;
-		m_renderfuncs[5] = &model2_renderer::model2_3d_render_5;
-		m_renderfuncs[6] = &model2_renderer::model2_3d_render_6;
-		m_renderfuncs[7] = &model2_renderer::model2_3d_render_7;
-		m_xoffs = 90;
-		m_yoffs = -8;
 	}
 
-	bitmap_rgb32& destmap() { return m_destmap; }
+	bitmap_rgb32 &destmap() { return m_destmap; }
+	bitmap_ind8 &fillmap() { return m_fillmap; }
 
-	void model2_3d_render(triangle *tri, const rectangle &cliprect);
+	void model2_3d_render(polygon *poly, const rectangle &cliprect);
 	void set_xoffset(int16_t xoffs) { m_xoffs = xoffs; }
 	void set_yoffset(int16_t yoffs) { m_yoffs = yoffs; }
 
-	/* checker = 0, textured = 0, transparent = 0 */
-	#define MODEL2_FUNC 0
-	#define MODEL2_FUNC_NAME    model2_3d_render_0
-	#include "model2rd.ipp"
-	#undef MODEL2_FUNC
-	#undef MODEL2_FUNC_NAME
+	template <bool Translucent>
+	void draw_scanline_solid(int32_t scanline, const extent_t &extent, const m2_poly_extra_data &object, int threadid);
 
-	/* checker = 0, textured = 0, translucent = 1 */
-	#define MODEL2_FUNC 1
-	#define MODEL2_FUNC_NAME    model2_3d_render_1
-	#include "model2rd.ipp"
-	#undef MODEL2_FUNC
-	#undef MODEL2_FUNC_NAME
-
-	/* checker = 0, textured = 1, translucent = 0 */
-	#define MODEL2_FUNC 2
-	#define MODEL2_FUNC_NAME    model2_3d_render_2
-	#include "model2rd.ipp"
-	#undef MODEL2_FUNC
-	#undef MODEL2_FUNC_NAME
-
-	/* checker = 0, textured = 1, translucent = 1 */
-	#define MODEL2_FUNC 3
-	#define MODEL2_FUNC_NAME    model2_3d_render_3
-	#include "model2rd.ipp"
-	#undef MODEL2_FUNC
-	#undef MODEL2_FUNC_NAME
-
-	/* checker = 1, textured = 0, translucent = 0 */
-	#define MODEL2_FUNC 4
-	#define MODEL2_FUNC_NAME    model2_3d_render_4
-	#include "model2rd.ipp"
-	#undef MODEL2_FUNC
-	#undef MODEL2_FUNC_NAME
-
-	/* checker = 1, textured = 0, translucent = 1 */
-	#define MODEL2_FUNC 5
-	#define MODEL2_FUNC_NAME    model2_3d_render_5
-	#include "model2rd.ipp"
-	#undef MODEL2_FUNC
-	#undef MODEL2_FUNC_NAME
-
-	/* checker = 1, textured = 1, translucent = 0 */
-	#define MODEL2_FUNC 6
-	#define MODEL2_FUNC_NAME    model2_3d_render_6
-	#include "model2rd.ipp"
-	#undef MODEL2_FUNC
-	#undef MODEL2_FUNC_NAME
-
-	/* checker = 1, textured = 1, translucent = 1 */
-	#define MODEL2_FUNC 7
-	#define MODEL2_FUNC_NAME    model2_3d_render_7
-	#include "model2rd.ipp"
-	#undef MODEL2_FUNC
-	#undef MODEL2_FUNC_NAME
-
-	scanline_render_func m_renderfuncs[8];
+	template <bool Translucent>
+	void draw_scanline_tex(int32_t scanline, const extent_t &extent, const m2_poly_extra_data &object, int threadid);
 
 private:
-	model2_state& m_state;
+	render_delegate m_render_callbacks[4];
+
+	model2_state &m_state;
 	bitmap_rgb32 m_destmap;
-	int16_t m_xoffs = 0, m_yoffs = 0;
+	bitmap_ind8 m_fillmap;
+	int16_t m_xoffs, m_yoffs;
+
+	template <bool Translucent>
+	u32 fetch_bilinear_texel(const m2_poly_extra_data& object, s32 miplevel, s32 fu, s32 fv);
+
+	static u16 get_texel(u32 base_x, u32 base_y, int x, int y, const u32 *sheet);
 };
 
 typedef model2_renderer::vertex_t poly_vertex;
@@ -780,19 +731,22 @@ struct model2_state::texture_parameter
 	float           specular_scale = 0;
 };
 
-struct model2_state::triangle
+struct model2_state::polygon
 {
-	triangle() : v{ { 0, 0 }, { 0, 0 }, { 0, 0 } }
+	polygon() : v{ { 0, 0 }, { 0, 0 }, { 0, 0 }, { 0, 0 },
+				   { 0, 0 }, { 0, 0 }, { 0, 0 }, { 0, 0 } }
 	{
 		for (poly_vertex &vertex : v)
 			std::fill(std::begin(vertex.p), std::end(vertex.p), 0);
 	}
 
 	void *          next = nullptr;
-	poly_vertex     v[3];
+	poly_vertex     v[8];
+	u8              num_vertices = 3;
 	u16             z = 0;
 	u16             texheader[4] = { 0, 0, 0, 0 };
 	u8              luma = 0;
+	s32             texlod = 0;
 	int16_t         viewport[4] = { 0, 0, 0, 0 };
 	int16_t         center[2] = { 0, 0 };
 	u8              window = 0;
@@ -810,6 +764,7 @@ struct model2_state::quad_m2
 	u16             z = 0;
 	u16             texheader[4] = { 0, 0, 0, 0 };
 	u8              luma = 0;
+	s32             texlod = 0;
 };
 
 /*******************************************
@@ -818,14 +773,14 @@ struct model2_state::quad_m2
  *
  *******************************************/
 
-#define MAX_TRIANGLES       32768
+#define MAX_POLYGONS       32768
 
 struct model2_state::raster_state
 {
 	raster_state()
 	{
 		std::fill(std::begin(command_buffer), std::end(command_buffer), 0);
-		std::fill(std::begin(tri_sorted_list), std::end(tri_sorted_list), nullptr);
+		std::fill(std::begin(poly_sorted_list), std::end(poly_sorted_list), nullptr);
 		std::fill(std::begin(texture_ram), std::end(texture_ram), 0);
 		std::fill(std::begin(log_ram), std::end(log_ram), 0);
 	}
@@ -837,19 +792,19 @@ struct model2_state::raster_state
 	int16_t         center[4][2] = { { 0, 0 }, { 0, 0 }, { 0, 0 }, { 0, 0 } }; // Centers (eye 0[x,y],1[x,y],2[x,y],3[x,y])
 	u16             center_sel = 0;                 // Selected center
 	u32             reverse = 0;                    // Left/Right Reverse
-	float           z_adjust = 0;                   // ZSort Mode
-	float           triangle_z = 0;                 // Current Triangle z value
+	s32             z_adjust = 0;                   // ZSort Mode
+	float           polygon_z = 0;                  // Current polygon z value
 	u8              master_z_clip = 0;              // Master Z-Clip value
 	u32             cur_command = 0;                // Current command
 	u32             command_buffer[32];             // Command buffer
 	u32             command_index = 0;              // Command buffer index
-	triangle        tri_list[MAX_TRIANGLES];        // Triangle list
-	u32             tri_list_index = 0;             // Triangle list index
-	triangle *      tri_sorted_list[0x10000];       // Sorted Triangle list
+	polygon         poly_list[MAX_POLYGONS];        // Polygon list
+	u32             poly_list_index = 0;            // Polygon list index
+	polygon *       poly_sorted_list[0x10000];      // Sorted polygon list
 	u16             min_z = 0;                      // Minimum sortable Z value
 	u16             max_z = 0;                      // Maximum sortable Z value
 	u16             texture_ram[0x10000];           // Texture RAM pointer
-	u8              log_ram[0x40000];               // Log RAM pointer
+	u8              log_ram[0x8000];                // Log RAM pointer
 	u8              cur_window = 0;                 // Current window
 	plane           clip_plane[4][4];               // Polygon clipping planes
 };
