@@ -52,31 +52,31 @@ is "Treasure Fall" (despite the cart label is "Treasure Hall").
 
 Dumped games:
 
+1999 Dokidoki Kingyo Sukui        https://youtu.be/Z0tOjG_tteU
+1999 Shatekids                    https://youtu.be/aWzYlvm6uIs
 2000 Animal Catch                 https://youtu.be/U4L5EwWbxqw
 2000 Itazura Monkey               https://youtu.be/GHxiqUQRpV8
 2000 Pye-nage Taikai              https://youtu.be/oL2OIbrv-KI
 2000 Taihou de Doboon             https://youtu.be/loPP3jt0Ob0
 2001 Hae Hae Ka Ka Ka             https://youtu.be/37IxYCg0tic
+2001 Kotekitai Slot               https://youtu.be/IohrnGIma4A
+2002 Gun Kids
 2003 Go Go Cowboy (EN, prize)     https://youtu.be/rymtzmSXjuA
+2003 Wantouchable                 https://youtu.be/aRcTCdZZLRo
 
 Games with the same cabinet, or in the Treasure Fall series, which might be on the same hardware:
 
-1999 Shatekids                    https://youtu.be/aWzYlvm6uIs
 1999 Otakara Locomo               https://youtu.be/J0NwMWO3SdY
-1999 Dokidoki Kingyosukui         https://youtu.be/Z0tOjG_tteU
 2000 Otoshicha Ottotto            https://youtu.be/AybhPHTFvMo
 2001 Mushitori Meijin
 2001 Morino Dodgeball Senshuken   https://youtu.be/k98KIRjTYbY
 2001 Waiwai Wanage                https://youtu.be/4GmwPTk_Er4
 2001 Zarigani Tsuri               https://youtu.be/NppRdebkUaQ
-2001 Kotekitai Slot               https://youtu.be/IohrnGIma4A
 2002 Shateki Yokochou             https://youtu.be/LPZLWP1x5o8
 2002 Ipponzuri Slot
 2002 Karateman                    https://youtu.be/EIrVHEAv3Sc
-2002 One-touchable
 2002 Perfect Goal (screenless)    https://youtu.be/ilneyp-8dBI
 2003 Go Go Cowboy (JP, medal)     https://youtu.be/qYDw2sxNRqE
-2003 Gun Kids
 2003 Kurukuru Train               https://youtu.be/Ef7TQX4C9fA
 2003 Safari Kingdom (screenless)
 2003 Zakuzaku Kaizokudan
@@ -114,6 +114,7 @@ Notes:
 *************************************************************************************************************/
 
 #include "emu.h"
+
 #include "cpu/z80/kl5c80a12.h"
 #include "cpu/z80/ky80.h"
 #include "machine/74165.h"
@@ -126,6 +127,7 @@ Notes:
 #include "sound/okim9810.h"
 #include "sound/ymz280b.h"
 #include "video/bufsprite.h"
+
 #include "emupal.h"
 #include "screen.h"
 #include "speaker.h"
@@ -150,7 +152,7 @@ public:
 	{ }
 
 protected:
-	virtual void video_start() override;
+	virtual void video_start() override ATTR_COLD;
 
 	// TODO: unify these handlers
 	void vregs_w(offs_t offset, uint8_t data);
@@ -164,7 +166,7 @@ protected:
 	void draw_sprites(bitmap_ind16 &bitmap, const rectangle &cliprect, int pri_mask);
 	uint32_t screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect);
 
-	[[maybe_unused]] DECLARE_WRITE_LINE_MEMBER(screen_vblank_sammymdl);
+	[[maybe_unused]] void screen_vblank_sammymdl(int state);
 
 	// Required devices
 	required_device<screen_device> m_screen;
@@ -218,9 +220,9 @@ protected:
 
 	INTERRUPT_GEN_MEMBER(sigmab98_vblank_interrupt);
 
-	void dodghero_mem_map(address_map &map);
-	void gegege_io_map(address_map &map);
-	void gegege_mem_map(address_map &map);
+	void dodghero_mem_map(address_map &map) ATTR_COLD;
+	void gegege_io_map(address_map &map) ATTR_COLD;
+	void gegege_mem_map(address_map &map) ATTR_COLD;
 
 	virtual void machine_start() override { m_leds.resolve(); }
 
@@ -253,14 +255,16 @@ public:
 	}
 
 	void init_lufykzku();
+	void init_rockman();
 	void lufykzku(machine_config &config);
+	void rockman(machine_config& config);
 
 private:
 	required_device<mb3773_device> m_watchdog;
 	required_device_array<ttl165_device, 2> m_dsw_shifter;
 
 	int m_dsw_bit;
-	DECLARE_WRITE_LINE_MEMBER(dsw_w);
+	void dsw_w(int state);
 
 	void lufykzku_c4_w(uint8_t data);
 	void lufykzku_c6_w(uint8_t data);
@@ -269,9 +273,10 @@ private:
 	void lufykzku_watchdog_w(uint8_t data);
 
 	TIMER_DEVICE_CALLBACK_MEMBER(lufykzku_irq);
+	TIMER_DEVICE_CALLBACK_MEMBER(rockman_timer_irq);
 
-	void lufykzku_io_map(address_map &map);
-	void lufykzku_mem_map(address_map &map);
+	void lufykzku_io_map(address_map &map) ATTR_COLD;
+	void lufykzku_mem_map(address_map &map) ATTR_COLD;
 
 	uint8_t m_vblank_vector = 0;
 	uint8_t m_timer0_vector = 0;
@@ -325,13 +330,13 @@ private:
 	uint8_t eeprom_r();
 	void eeprom_w(uint8_t data);
 
-	void animalc_io(address_map &map);
-	void animalc_map(address_map &map);
-	void gocowboy_io(address_map &map);
-	void gocowboy_map(address_map &map);
-	void haekaka_map(address_map &map);
-	void itazuram_map(address_map &map);
-	void tdoboon_map(address_map &map);
+	void animalc_io(address_map &map) ATTR_COLD;
+	void animalc_map(address_map &map) ATTR_COLD;
+	void gocowboy_io(address_map &map) ATTR_COLD;
+	void gocowboy_map(address_map &map) ATTR_COLD;
+	void haekaka_map(address_map &map) ATTR_COLD;
+	void itazuram_map(address_map &map) ATTR_COLD;
+	void tdoboon_map(address_map &map) ATTR_COLD;
 
 	// Required devices
 	required_device<kl5c80a12_device> m_maincpu;
@@ -731,7 +736,7 @@ void sigmab98_state::show_outputs()
 void sigmab98_state::eeprom_w(uint8_t data)
 {
 	// latch the bit
-	m_eeprom->di_write((data & 0x40) >> 6);
+	m_eeprom->di_write(BIT(data, 6));
 
 	// reset line asserted: reset.
 //  if ((m_c0 ^ data) & 0x20)
@@ -762,9 +767,9 @@ void sigmab98_state::c4_w(uint8_t data)
 // 20 led? (starts blinking after coin in)
 void sigmab98_state::c6_w(uint8_t data)
 {
-	machine().bookkeeping().coin_lockout_w(0, (~data) & 0x02);
+	machine().bookkeeping().coin_lockout_w(0, BIT(~data, 1));
 
-	machine().bookkeeping().coin_counter_w(0,   data  & 0x04);
+	machine().bookkeeping().coin_counter_w(0, BIT(data, 2));
 
 	if ((data & 0x08) && !(m_c6 & 0x08))
 		m_buffered_spriteram->copy();
@@ -781,7 +786,7 @@ void sigmab98_state::c6_w(uint8_t data)
 // 02 hopper motor on (active low)?
 void sigmab98_state::c8_w(uint8_t data)
 {
-	m_hopper->motor_w((!(data & 0x02) && (data & 0x01)) ? 0 : 1);
+	m_hopper->motor_w((!(data & 0x02) && (data & 0x01)) ? 1 : 0);
 
 	m_c8 = data;
 	show_outputs();
@@ -816,7 +821,7 @@ void sigmab98_state::gegege_io_map(address_map &map)
 	map(0xc6, 0xc6).w(FUNC(sigmab98_state::c6_w));
 	map(0xc8, 0xc8).w(FUNC(sigmab98_state::c8_w));
 
-	map(0xe5, 0xe5).nopr();   // during irq
+	map(0xe5, 0xe5).nopr(); // during irq
 }
 
 
@@ -824,7 +829,7 @@ void sigmab98_state::gegege_io_map(address_map &map)
                        Otakara Itadaki Luffy Kaizoku-Dan!
 ***************************************************************************/
 
-WRITE_LINE_MEMBER(lufykzku_state::dsw_w)
+void lufykzku_state::dsw_w(int state)
 {
 	m_dsw_bit = state;
 }
@@ -872,7 +877,7 @@ void lufykzku_state::lufykzku_c8_w(uint8_t data)
 {
 	// bit 0? on payout button
 	// bit 1? when ending payment
-	m_hopper->motor_w(( (data & 0x01) && !(data & 0x02)) ? 0 : 1);
+	m_hopper->motor_w(((data & 0x01) && !(data & 0x02)) ? 1 : 0);
 
 	m_dsw_shifter[0]->shift_load_w(BIT(data, 4));
 	m_dsw_shifter[1]->shift_load_w(BIT(data, 4));
@@ -947,7 +952,7 @@ void sigmab98_base_state::vblank_w(uint8_t data)
 	m_vblank = (m_vblank & ~0x03) | (data & 0x03);
 }
 
-WRITE_LINE_MEMBER(sigmab98_base_state::screen_vblank_sammymdl)
+void sigmab98_base_state::screen_vblank_sammymdl(int state)
 {
 	// rising edge
 	if (state)
@@ -993,7 +998,7 @@ void sammymdl_state::leds_w(uint8_t data)
 // 01 hopper motor on (active low)?
 void sammymdl_state::hopper_w(uint8_t data)
 {
-	m_hopper->motor_w((!(data & 0x01) && (data & 0x02)) ? 0 : 1);
+	m_hopper->motor_w((!(data & 0x01) && (data & 0x02)) ? 1 : 0);
 
 	m_out[2] = data;
 	show_3_outputs();
@@ -1057,8 +1062,8 @@ void sammymdl_state::gocowboy_leds_w(uint8_t data)
 
 	// 10 hopper enable?
 	// 20 hopper motor on (active low)?
-	m_hopper_small->motor_w((!(data & 0x20) && (data & 0x10)) ? 0 : 1);
-	m_hopper_large->motor_w((!(data & 0x80) && (data & 0x40)) ? 0 : 1);
+	m_hopper_small->motor_w((!(data & 0x20) && (data & 0x10)) ? 1 : 0);
+	m_hopper_large->motor_w((!(data & 0x80) && (data & 0x40)) ? 1 : 0);
 
 	m_out[1] = data;
 	show_3_outputs();
@@ -1198,19 +1203,19 @@ GFXDECODE_END
 // 1 button (plus bet and payout)
 static INPUT_PORTS_START( sigma_1b )
 	PORT_START("EEPROM")
-	PORT_BIT( 0x01, IP_ACTIVE_HIGH, IPT_CUSTOM )                       // Related to d013. Must be 0
-	PORT_BIT( 0x02, IP_ACTIVE_LOW,  IPT_CUSTOM  ) PORT_VBLANK("screen") // Related to d013. Must be 0
+	PORT_BIT( 0x01, IP_ACTIVE_HIGH, IPT_CUSTOM  )                       // Related to d013. Must be 0
+	PORT_BIT( 0x02, IP_ACTIVE_LOW,  IPT_CUSTOM  ) PORT_READ_LINE_DEVICE_MEMBER("screen", FUNC(screen_device::vblank)) // Related to d013. Must be 0
 	PORT_BIT( 0x04, IP_ACTIVE_LOW,  IPT_UNKNOWN )
 	PORT_BIT( 0x08, IP_ACTIVE_LOW,  IPT_UNKNOWN )
 	PORT_BIT( 0x10, IP_ACTIVE_LOW,  IPT_UNKNOWN )
 	PORT_BIT( 0x20, IP_ACTIVE_LOW,  IPT_UNKNOWN )
 	PORT_BIT( 0x40, IP_ACTIVE_LOW,  IPT_UNKNOWN )
-	PORT_BIT( 0x80, IP_ACTIVE_HIGH, IPT_CUSTOM )   PORT_READ_LINE_DEVICE_MEMBER("eeprom", eeprom_serial_93cxx_device, do_read)
+	PORT_BIT( 0x80, IP_ACTIVE_HIGH, IPT_CUSTOM  ) PORT_READ_LINE_DEVICE_MEMBER("eeprom", FUNC(eeprom_serial_93cxx_device::do_read))
 
 	PORT_START("BUTTON")
 	PORT_BIT( 0x01, IP_ACTIVE_LOW,  IPT_COIN2   ) PORT_IMPULSE(5)   // ? (coin error, pulses mask 4 of port c6)
 	PORT_BIT( 0x02, IP_ACTIVE_LOW,  IPT_COIN1   ) PORT_IMPULSE(5) PORT_NAME("Medal")    // coin/medal in (coin error)
-	PORT_BIT( 0x04, IP_ACTIVE_HIGH, IPT_CUSTOM ) PORT_READ_LINE_DEVICE_MEMBER("hopper", ticket_dispenser_device, line_r)
+	PORT_BIT( 0x04, IP_ACTIVE_LOW,  IPT_CUSTOM  ) PORT_READ_LINE_DEVICE_MEMBER("hopper", FUNC(ticket_dispenser_device::line_r))
 	PORT_SERVICE( 0x08, IP_ACTIVE_LOW )
 	PORT_BIT( 0x10, IP_ACTIVE_LOW,  IPT_GAMBLE_BET ) PORT_CODE(KEYCODE_1)  // bet / select in test menu
 	PORT_BIT( 0x20, IP_ACTIVE_LOW,  IPT_BUTTON1 )
@@ -1286,7 +1291,7 @@ static INPUT_PORTS_START( lufykzku )
 	PORT_BIT( 0x08, IP_ACTIVE_HIGH, IPT_UNKNOWN )
 	PORT_BIT( 0x10, IP_ACTIVE_HIGH, IPT_UNKNOWN )
 	PORT_BIT( 0x20, IP_ACTIVE_HIGH, IPT_UNKNOWN )
-	PORT_BIT( 0x40, IP_ACTIVE_HIGH, IPT_CUSTOM ) PORT_READ_LINE_DEVICE_MEMBER("hopper", ticket_dispenser_device, line_r)
+	PORT_BIT( 0x40, IP_ACTIVE_LOW,  IPT_CUSTOM  ) PORT_READ_LINE_DEVICE_MEMBER("hopper", FUNC(ticket_dispenser_device::line_r))
 	PORT_BIT( 0x80, IP_ACTIVE_HIGH, IPT_UNKNOWN )
 
 	PORT_START("PAYOUT")
@@ -1294,7 +1299,7 @@ static INPUT_PORTS_START( lufykzku )
 	PORT_BIT( 0x02, IP_ACTIVE_HIGH, IPT_SERVICE1      ) // service sw (service coin, press to go down in service mode)
 	PORT_BIT( 0x04, IP_ACTIVE_HIGH, IPT_GAMBLE_PAYOUT ) // payout sw
 	PORT_BIT( 0x08, IP_ACTIVE_HIGH, IPT_UNKNOWN       )
-	PORT_BIT( 0x10, IP_ACTIVE_LOW,  IPT_GAMBLE_DOOR   ) // "maintenance panel abnormality" when not high on boot (in that case, when it goes high the game resets)
+	PORT_BIT( 0x10, IP_ACTIVE_LOW,  IPT_DOOR          ) // "maintenance panel abnormality" when not high on boot (in that case, when it goes high the game resets)
 	PORT_BIT( 0x20, IP_ACTIVE_HIGH, IPT_UNKNOWN       )
 	PORT_BIT( 0x40, IP_ACTIVE_HIGH, IPT_UNKNOWN       )
 	PORT_BIT( 0x80, IP_ACTIVE_HIGH, IPT_UNKNOWN       )
@@ -1362,14 +1367,14 @@ static INPUT_PORTS_START( sammymdl )
 	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_UNKNOWN )
 
 	PORT_START("COIN")
-	PORT_BIT( 0x01, IP_ACTIVE_LOW,  IPT_COIN1   ) PORT_IMPULSE(5)   // coin1 in
-	PORT_BIT( 0x02, IP_ACTIVE_LOW,  IPT_COIN2   ) PORT_IMPULSE(5)   // coin2 in
-	PORT_BIT( 0x04, IP_ACTIVE_LOW,  IPT_COIN3   ) PORT_IMPULSE(5) PORT_NAME("Medal")    // medal in
-	PORT_BIT( 0x08, IP_ACTIVE_LOW,  IPT_SERVICE )   // test sw
-	PORT_BIT( 0x10, IP_ACTIVE_LOW,  IPT_UNKNOWN )
-	PORT_BIT( 0x20, IP_ACTIVE_HIGH, IPT_CUSTOM ) PORT_READ_LINE_DEVICE_MEMBER("hopper", ticket_dispenser_device, line_r)
-	PORT_BIT( 0x40, IP_ACTIVE_LOW,  IPT_UNKNOWN )
-	PORT_BIT( 0x80, IP_ACTIVE_LOW,  IPT_UNKNOWN )
+	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_COIN1   ) PORT_IMPULSE(5)   // coin1 in
+	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_COIN2   ) PORT_IMPULSE(5)   // coin2 in
+	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_COIN3   ) PORT_IMPULSE(5) PORT_NAME("Medal")    // medal in
+	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_SERVICE )   // test sw
+	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_UNKNOWN )
+	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_CUSTOM  ) PORT_READ_LINE_DEVICE_MEMBER("hopper", FUNC(ticket_dispenser_device::line_r))
+	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_UNKNOWN )
+	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_UNKNOWN )
 INPUT_PORTS_END
 
 static INPUT_PORTS_START( haekaka )
@@ -1384,22 +1389,22 @@ static INPUT_PORTS_START( haekaka )
 	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_UNKNOWN )
 
 	PORT_START("COIN")
-	PORT_BIT( 0x01, IP_ACTIVE_LOW,  IPT_UNKNOWN  )
-	PORT_BIT( 0x02, IP_ACTIVE_LOW,  IPT_UNKNOWN  )
-	PORT_BIT( 0x04, IP_ACTIVE_LOW,  IPT_UNKNOWN  )
-	PORT_BIT( 0x08, IP_ACTIVE_LOW,  IPT_SERVICE  )  // test sw
-	PORT_BIT( 0x10, IP_ACTIVE_LOW,  IPT_BUTTON1  )  // button
-	PORT_BIT( 0x20, IP_ACTIVE_HIGH, IPT_CUSTOM  ) PORT_READ_LINE_DEVICE_MEMBER("hopper", ticket_dispenser_device, line_r)
-	PORT_BIT( 0x40, IP_ACTIVE_LOW,  IPT_SERVICE1 )  // service coin / set in test mode
-	PORT_BIT( 0x80, IP_ACTIVE_LOW,  IPT_UNKNOWN  )
+	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_UNKNOWN  )
+	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_UNKNOWN  )
+	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_UNKNOWN  )
+	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_SERVICE  )  // test sw
+	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_BUTTON1  )  // button
+	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_CUSTOM   ) PORT_READ_LINE_DEVICE_MEMBER("hopper", FUNC(ticket_dispenser_device::line_r))
+	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_SERVICE1 )  // service coin / set in test mode
+	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_UNKNOWN  )
 INPUT_PORTS_END
 
 static INPUT_PORTS_START( gocowboy )
 	PORT_START("BUTTON")
 	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_BUTTON1  ) // shoot
 	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_COIN1    ) PORT_IMPULSE(20) // coin
-	PORT_BIT( 0x04, IP_ACTIVE_HIGH, IPT_CUSTOM ) PORT_READ_LINE_DEVICE_MEMBER("hopper_small", ticket_dispenser_device, line_r) // 1/2' pay sensor (small)
-	PORT_BIT( 0x08, IP_ACTIVE_HIGH, IPT_CUSTOM ) PORT_READ_LINE_DEVICE_MEMBER("hopper_large", ticket_dispenser_device, line_r) // 3/4' pay sensor (large)
+	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_CUSTOM   ) PORT_READ_LINE_DEVICE_MEMBER("hopper_small", FUNC(ticket_dispenser_device::line_r)) // 1/2' pay sensor (small)
+	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_CUSTOM   ) PORT_READ_LINE_DEVICE_MEMBER("hopper_large", FUNC(ticket_dispenser_device::line_r)) // 3/4' pay sensor (large)
 	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_SERVICE1 ) PORT_NAME("Meter Switch") // capsule test (pressed while booting) / next in test mode
 	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_SERVICE2 ) PORT_NAME("Reset Switch") // reset backup ram (pressed while booting) / previous in test mode
 	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_SERVICE  )                           // test mode (keep pressed in game) / select in test mode / service coin
@@ -1441,7 +1446,7 @@ void sigmab98_state::sigmab98(machine_config &config)
 	NVRAM(config, "nvram", nvram_device::DEFAULT_ALL_0);
 	EEPROM_93C46_16BIT(config, "eeprom");
 
-	TICKET_DISPENSER(config, m_hopper, attotime::from_msec(200), TICKET_MOTOR_ACTIVE_LOW, TICKET_STATUS_ACTIVE_LOW );
+	TICKET_DISPENSER(config, m_hopper, attotime::from_msec(200));
 
 	// video hardware
 	SCREEN(config, m_screen, SCREEN_TYPE_RASTER);
@@ -1459,12 +1464,11 @@ void sigmab98_state::sigmab98(machine_config &config)
 	BUFFERED_SPRITERAM8(config, m_buffered_spriteram);
 
 	// sound hardware
-	SPEAKER(config, "lspeaker").front_left();
-	SPEAKER(config, "rspeaker").front_right();
+	SPEAKER(config, "speaker", 2).front();
 
 	ymz280b_device &ymz(YMZ280B(config, "ymz", 16934400));    // clock @X2?
-	ymz.add_route(0, "lspeaker", 1.0);
-	ymz.add_route(1, "rspeaker", 1.0);
+	ymz.add_route(0, "speaker", 1.0, 0);
+	ymz.add_route(1, "speaker", 1.0, 1);
 }
 
 void sigmab98_state::dodghero(machine_config &config)
@@ -1484,11 +1488,22 @@ TIMER_DEVICE_CALLBACK_MEMBER(lufykzku_state::lufykzku_irq)
 	int scanline = param;
 
 	if (scanline == 240)
-		m_maincpu->set_input_line_and_vector(0, HOLD_LINE, m_vblank_vector); // Z80
+	{
+		if (m_vblank_vector) m_maincpu->set_input_line_and_vector(0, HOLD_LINE, m_vblank_vector); // Z80
+	}
 	else if (scanline == 128)
-		m_maincpu->set_input_line_and_vector(0, HOLD_LINE, m_timer0_vector); // Z80
+	{
+		if (m_timer0_vector) m_maincpu->set_input_line_and_vector(0, HOLD_LINE, m_timer0_vector); // Z80
+	}
 	else if ((scanline % 8) == 0)
-		m_maincpu->set_input_line_and_vector(0, HOLD_LINE, m_timer1_vector); // Z80 - this needs to be called often or the state of the door is not read at boot (at least 5 times before bb9 is called)
+	{
+		if (m_timer1_vector) m_maincpu->set_input_line_and_vector(0, HOLD_LINE, m_timer1_vector); // Z80 - this needs to be called often or the state of the door is not read at boot (at least 5 times before bb9 is called)
+	}
+}
+
+TIMER_DEVICE_CALLBACK_MEMBER(lufykzku_state::rockman_timer_irq)
+{
+	m_maincpu->set_input_line_and_vector(0, HOLD_LINE, 0xfc);
 }
 
 void lufykzku_state::lufykzku(machine_config &config)
@@ -1502,7 +1517,7 @@ void lufykzku_state::lufykzku(machine_config &config)
 	// No EEPROM
 
 	MB3773(config, m_watchdog, 0);
-	TICKET_DISPENSER(config, m_hopper, attotime::from_msec(200), TICKET_MOTOR_ACTIVE_LOW, TICKET_STATUS_ACTIVE_LOW );
+	TICKET_DISPENSER(config, m_hopper, attotime::from_msec(200));
 
 	// 2 x 8-bit parallel/serial converters
 	TTL165(config, m_dsw_shifter[0]);
@@ -1529,13 +1544,17 @@ void lufykzku_state::lufykzku(machine_config &config)
 	//BUFFERED_SPRITERAM8(config, m_buffered_spriteram); // same as sammymdl?
 
 	// sound hardware
-	SPEAKER(config, "lspeaker").front_left();
-	SPEAKER(config, "rspeaker").front_right();
+	SPEAKER(config, "speaker", 2).front();
 	okim9810_device &oki(OKIM9810(config, "oki", XTAL(4'096'000)));
-	oki.add_route(0, "lspeaker", 0.80);
-	oki.add_route(1, "rspeaker", 0.80);
+	oki.add_route(0, "speaker", 0.80, 0);
+	oki.add_route(1, "speaker", 0.80, 1);
 }
 
+void lufykzku_state::rockman(machine_config& config)
+{
+	lufykzku(config);
+	TIMER(config, "unktimer").configure_periodic(FUNC(lufykzku_state::rockman_timer_irq), attotime::from_hz(100));
+}
 
 /***************************************************************************
                              Sammy Medal Games
@@ -1574,7 +1593,7 @@ void sammymdl_state::sammymdl(machine_config &config)
 	NVRAM(config, "nvram", nvram_device::DEFAULT_ALL_0);   // battery backed RAM
 	EEPROM_93C46_8BIT(config, "eeprom");
 
-	TICKET_DISPENSER(config, m_hopper, attotime::from_msec(200), TICKET_MOTOR_ACTIVE_LOW, TICKET_STATUS_ACTIVE_LOW );
+	TICKET_DISPENSER(config, m_hopper, attotime::from_msec(200));
 
 	WATCHDOG_TIMER(config, "watchdog");
 
@@ -1595,12 +1614,11 @@ void sammymdl_state::sammymdl(machine_config &config)
 	//BUFFERED_SPRITERAM8(config, m_buffered_spriteram); // not on sammymdl?
 
 	// sound hardware
-	SPEAKER(config, "lspeaker").front_left();
-	SPEAKER(config, "rspeaker").front_right();
+	SPEAKER(config, "speaker", 2).front();
 
 	okim9810_device &oki(OKIM9810(config, "oki", XTAL(4'096'000)));
-	oki.add_route(0, "lspeaker", 0.80);
-	oki.add_route(1, "rspeaker", 0.80);
+	oki.add_route(0, "speaker", 0.80, 0);
+	oki.add_route(1, "speaker", 0.80, 1);
 }
 
 void sammymdl_state::animalc(machine_config &config)
@@ -1621,8 +1639,8 @@ void sammymdl_state::gocowboy(machine_config &config)
 	TIMER(config, "scantimer").configure_scanline(FUNC(sammymdl_state::gocowboy_int), "screen", 0, 1);
 
 	config.device_remove("hopper");
-	TICKET_DISPENSER(config, m_hopper_small, attotime::from_msec(200), TICKET_MOTOR_ACTIVE_LOW, TICKET_STATUS_ACTIVE_LOW );
-	TICKET_DISPENSER(config, m_hopper_large, attotime::from_msec(200), TICKET_MOTOR_ACTIVE_LOW, TICKET_STATUS_ACTIVE_LOW );
+	TICKET_DISPENSER(config, m_hopper_small, attotime::from_msec(200));
+	TICKET_DISPENSER(config, m_hopper_large, attotime::from_msec(200));
 
 	m_screen->screen_vblank().set_nop();
 }
@@ -2006,6 +2024,26 @@ void lufykzku_state::init_lufykzku()
 	m_gfxdecode->gfx(1)->set_granularity(16);
 }
 
+void lufykzku_state::init_rockman()
+{
+	m_vblank_vector = 0xfa; // nop
+	m_timer0_vector = 0;
+	m_timer1_vector = 0xfe;
+
+	m_gfxdecode->gfx(1)->set_granularity(16);
+}
+
+// Banpresto BPSC-2001M-A PCB, same as lufykzku
+ROM_START( mnrockman )
+	ROM_REGION( 0x20000, "maincpu", 0 )
+	ROM_LOAD( "ka-108_p1_ver1.02.ic1", 0x000000, 0x020000, CRC(727edf2f) SHA1(51a5f89a9ba64e16a1f46cc1145efa792ebb6401) )
+
+	ROM_REGION( 0x200000, "sprites", 0 )
+	ROM_LOAD( "ka-108_g1_ver1.00.ic3", 0x000000, 0x200000, CRC(ef79a6de) SHA1(50cbe7665e80b58a6bb0b20bae2deeca2e29c9da) )
+
+	ROM_REGION( 0x1000000, "oki", ROMREGION_ERASEFF )
+	ROM_LOAD( "ka-108_s1_ver1.00.ic2", 0x000000, 0x080000, CRC(828dd3bd) SHA1(9788a30199d81f6db54f5409fcb146098a29e6aa) )
+ROM_END
 
 /***************************************************************************
 
@@ -2055,10 +2093,14 @@ void lufykzku_state::init_lufykzku()
 
 ***************************************************************************/
 
-#define SAMMYMDL_BIOS                                                                                               \
-	ROM_REGION( 0x80000, "mainbios", 0 )                                                                            \
-	ROM_SYSTEM_BIOS( 0, "v5", "IPL Ver. 5.0" )                                                                      \
-	ROM_LOAD( "vm1211l01.u2", 0x000000, 0x080000, CRC(c3c74dc5) SHA1(07352e6dba7514214e778ba39e1ca773e4698858) )
+#define SAMMYMDL_BIOS                                                                                                           \
+	ROM_REGION( 0x80000, "mainbios", 0 )                                                                                        \
+	ROM_SYSTEM_BIOS( 0, "v5", "IPL Ver. 5.0" ) /* (c)2003 */                                                                    \
+	ROMX_LOAD( "vm1211l01.u2",   0x000000, 0x080000, CRC(c3c74dc5) SHA1(07352e6dba7514214e778ba39e1ca773e4698858), ROM_BIOS(0) ) \
+	ROM_SYSTEM_BIOS( 1, "v4", "IPL Ver. 4.0" ) /* (c)2000, ROM patches not correct for this BIOS */                             \
+	ROMX_LOAD( "mt201l04.u012",  0x000000, 0x080000, CRC(c8c6d25f) SHA1(5be39fa72b65f2e455ccc146dbab58d24ab46505), ROM_BIOS(1) ) \
+	ROM_SYSTEM_BIOS( 2, "v1", "IPL Ver. 1.0" ) /* (c)1999, ROM patches not correct for this BIOS */                             \
+	ROMX_LOAD( "vm1201l01.u012", 0x000000, 0x080000, CRC(21b2ccbd) SHA1(c3ceea196e5c1c6a9e03cf3a22e9f25f0099a38e), ROM_BIOS(2) )
 
 ROM_START( sammymdl )
 	SAMMYMDL_BIOS
@@ -2094,6 +2136,45 @@ ROM_START( animalc )
 	ROM_LOAD( "vx2301l01.u016", 0x00000, 0x200000, CRC(4ae14ff9) SHA1(1273d15ea642452fecacff572655cd3ab47a5884) )   // 1xxxxxxxxxxxxxxxxxxxx = 0x00
 ROM_END
 
+ROM_START( gunkids )
+	SAMMYMDL_BIOS
+
+	ROM_REGION( 0x1000000, "oki", ROMREGION_ERASEFF )
+	ROM_LOAD( "vx2302l01.u021", 0x00000, 0x200000, CRC(3d989a45) SHA1(f4e1dc13bfe9ef8bf733735b6647946dda6962f2) )
+
+	ROM_REGION( 0x40000, "maincpu", 0 )
+	ROM_COPY( "oki", 0x1c0000, 0x00000, 0x40000 )
+
+	ROM_REGION( 0x200000, "sprites", 0 )
+	ROM_LOAD( "vx2301l01.u016", 0x00000, 0x200000, CRC(5e356b68) SHA1(0e4e28b02dcb5ff7d2a7139c5cdf31cbd08167f4) )
+ROM_END
+
+ROM_START( wantouch ) // ワンタッチャブル
+	SAMMYMDL_BIOS
+
+	ROM_REGION( 0x1000000, "oki", ROMREGION_ERASEFF )
+	ROM_LOAD( "vx2302l01.u021", 0x00000, 0x200000, CRC(fe00ac87) SHA1(97c3defafd04f48984e5f2fe34528ada024355c5) )
+
+	ROM_REGION( 0x40000, "maincpu", 0 )
+	ROM_COPY( "oki", 0x1c0000, 0x00000, 0x40000 )
+
+	ROM_REGION( 0x200000, "sprites", 0 )
+	ROM_LOAD( "vx2301l01.u016", 0x00000, 0x200000, CRC(cd8d4478) SHA1(224e8e0e9a49f10515e0ebf9a5e97d8adc7eed13) )
+ROM_END
+
+ROM_START( kotekita ) // こてき隊（たい）スロット
+	SAMMYMDL_BIOS
+
+	ROM_REGION( 0x1000000, "oki", ROMREGION_ERASEFF )
+	ROM_LOAD( "vx2302l01.u021", 0x00000, 0x200000, CRC(37faa4a9) SHA1(619135d74052fb2272ce78976c2aa59a65244f12) )
+
+	ROM_REGION( 0x40000, "maincpu", 0 )
+	ROM_COPY( "oki", 0x1c0000, 0x00000, 0x40000 )
+
+	ROM_REGION( 0x200000, "sprites", 0 )
+	ROM_LOAD( "vx2301l01.u016", 0x00000, 0x200000, CRC(e471c1ef) SHA1(4ddf3261d8736888ebbdbd0d755aad75987b3b3d) )
+ROM_END
+
 void sammymdl_state::init_animalc()
 {
 	uint8_t *rom = memregion("mainbios")->base();
@@ -2277,6 +2358,32 @@ void sammymdl_state::init_haekaka()
 	rom[0x005ac] = 0xc3;
 }
 
+ROM_START( kingyosu )
+	SAMMYMDL_BIOS
+
+	ROM_REGION( 0x1000000, "oki", ROMREGION_ERASEFF )
+	ROM_LOAD( "vx1302l02.u021", 0x00000, 0x200000, CRC(45e43e3c) SHA1(71ada38c1f65738ca75ca75d2272f310ba43c72c) )
+
+	ROM_REGION( 0x40000, "maincpu", 0 )
+	ROM_COPY( "oki", 0x1c0000, 0x00000, 0x40000 )
+
+	ROM_REGION( 0x200000, "sprites", 0 )
+	ROM_LOAD( "vx1301l01.u016", 0x00000, 0x200000, CRC(25dfbf4c) SHA1(8f9552cf54bdf7490055ebdb23fa758f5cddafb5) )
+ROM_END
+
+ROM_START( shatekds )
+	SAMMYMDL_BIOS
+
+	ROM_REGION( 0x1000000, "oki", ROMREGION_ERASEFF )
+	ROM_LOAD( "29f016a.u015", 0x00000, 0x200000, CRC(7c779286) SHA1(db688ac2cad14c642e6da770212217a466e1bb64) )
+
+	ROM_REGION( 0x40000, "maincpu", 0 )
+	ROM_COPY( "oki", 0x1c0000, 0x00000, 0x40000 )
+
+	ROM_REGION( 0x200000, "sprites", 0 )
+	ROM_LOAD( "mx29f1610tc.u016", 0x00000, 0x200000, CRC(6e7d2c7d) SHA1(747d54eee26631966a91fd26b8b4beb32b67ddb7) )
+ROM_END
+
 } // anonymous namespace
 
 
@@ -2297,11 +2404,17 @@ GAME( 1997, ucytokyu, 0,        sigmab98, sigma_js, sigmab98_state, init_ucytoky
 GAME( 2000, dashhero, 0,        sigmab98, sigma_1b, sigmab98_state, init_dashhero, ROT0, "Sigma",             "Minna Ganbare! Dash Hero",             MACHINE_NOT_WORKING ) // 1999 in the rom
 // Banpresto Medal Games
 GAME( 2001, lufykzku, 0,        lufykzku, lufykzku, lufykzku_state, init_lufykzku, ROT0, "Banpresto / Eiichiro Oda / Shueisha - Fuji TV - Toho Animation", "Otakara Itadaki Luffy Kaizoku-Dan! (Japan, v1.02)", 0 )
+GAME( 2002, mnrockman,0,        rockman,  lufykzku, lufykzku_state, init_rockman,  ROT0, "Banpresto / Capcom / Shogakukan / ShoPro / TV Tokyo", "Medal Network: Rockman EXE", 0 )
 // Sammy Medal Games:
-GAME( 2000, sammymdl, 0,        sammymdl, sammymdl, sammymdl_state, init_animalc,  ROT0, "Sammy",             "Sammy Medal Game System Bios",         MACHINE_IS_BIOS_ROOT )
+GAME( 1999, sammymdl, 0,        sammymdl, sammymdl, sammymdl_state, init_animalc,  ROT0, "Sammy",             "Sammy Medal Game System BIOS",         MACHINE_IS_BIOS_ROOT )
+GAME( 1999, kingyosu, sammymdl, itazuram, sammymdl, sammymdl_state, init_animalc,  ROT0, "Sammy",             "Dokidoki Kingyo Sukui",                0 )
+GAME( 1999, shatekds, sammymdl, animalc,  sammymdl, sammymdl_state, init_animalc,  ROT0, "Sammy",             "Shatekids",                            0 )
 GAME( 2000, animalc,  sammymdl, animalc,  sammymdl, sammymdl_state, init_animalc,  ROT0, "Sammy",             "Animal Catch",                         0 )
 GAME( 2000, itazuram, sammymdl, itazuram, sammymdl, sammymdl_state, init_itazuram, ROT0, "Sammy",             "Itazura Monkey",                       0 )
 GAME( 2000, pyenaget, sammymdl, pyenaget, sammymdl, sammymdl_state, init_haekaka,  ROT0, "Sammy",             "Pye-nage Taikai",                      0 )
 GAME( 2000, tdoboon,  sammymdl, tdoboon,  haekaka,  sammymdl_state, init_haekaka,  ROT0, "Sammy",             "Taihou de Doboon",                     0 )
 GAME( 2001, haekaka,  sammymdl, haekaka,  haekaka,  sammymdl_state, init_haekaka,  ROT0, "Sammy",             "Hae Hae Ka Ka Ka",                     0 )
+GAME( 2001, kotekita, sammymdl, haekaka,  sammymdl, sammymdl_state, init_haekaka,  ROT0, "Sammy",             "Kotekitai Slot",                       0 )
+GAME( 2002, gunkids,  sammymdl, animalc,  sammymdl, sammymdl_state, init_animalc,  ROT0, "Sammy",             "Hayauchi Gun Kids",                    0 )
+GAME( 2003, wantouch, sammymdl, animalc,  sammymdl, sammymdl_state, init_animalc,  ROT0, "Sammy",             "Wantouchable",                         0 )
 GAME( 2003, gocowboy, 0,        gocowboy, gocowboy, sammymdl_state, empty_init,    ROT0, "Sammy",             "Go Go Cowboy (English, prize)",        0 )
