@@ -169,13 +169,11 @@ void spectrum_128_state::video_start()
 {
 	spectrum_state::video_start();
 	m_screen_location = m_ram->pointer() + (5 << 14);
-	m_border4t_render_at = 3;
 }
 
 uint8_t spectrum_128_state::spectrum_128_pre_opcode_fetch_r(offs_t offset)
 {
-	m_is_m1_rd_contended = false;
-	if (!machine().side_effects_disabled() && is_contended(offset)) content_early();
+	m_ula->m1(offset);
 
 	/* this allows expansion devices to act upon opcode fetches from MEM addresses
 	   for example, interface1 detection fetches requires fetches at 0008 / 0708 to
@@ -202,7 +200,7 @@ u8 spectrum_128_state::spectrum_128_rom_r(offs_t offset)
 template <u8 Bank> void spectrum_128_state::spectrum_128_ram_w(offs_t offset, u8 data)
 {
 	u16 addr = 0x4000 * Bank + offset;
-	if (is_contended(addr)) content_early();
+	m_ula->data_w(addr);
 	if (is_vram_write(addr)) m_screen->update_now();
 
 	((u8*)m_bank_ram[Bank]->base())[offset] = data;
@@ -214,7 +212,7 @@ template void spectrum_128_state::spectrum_128_ram_w<0>(offs_t offset, u8 data);
 template <u8 Bank> u8 spectrum_128_state::spectrum_128_ram_r(offs_t offset)
 {
 	u16 addr = 0x4000 * Bank + offset;
-	if (!machine().side_effects_disabled() && is_contended(addr)) content_early();
+	m_ula->data_r(addr);
 
 	return ((u8*)m_bank_ram[Bank]->base())[offset];
 }
@@ -225,8 +223,7 @@ template <u8 Bank> u8 spectrum_128_state::spectrum_128_ram_r(offs_t offset)
 //    D5: Disable paging
 void spectrum_128_state::spectrum_128_port_7ffd_w(offs_t offset, uint8_t data)
 {
-	if (is_contended(offset)) content_early();
-	content_early(1);
+	m_ula->ula_w(offset);
 
 	// disable paging?
 	if (m_port_7ffd_data & 0x20) return;
@@ -240,11 +237,19 @@ void spectrum_128_state::spectrum_128_port_7ffd_w(offs_t offset, uint8_t data)
 	m_exp->iorq_w(offset | 1, data);
 }
 
+void spectrum_128_state::bank3_set_page(u8 page)
+{
+	m_port_7ffd_data &= 0xf8;
+	m_port_7ffd_data |= page & 0x07;
+	spectrum_128_update_memory();
+}
+
 void spectrum_128_state::spectrum_128_update_memory()
 {
 	m_bank_rom[0]->set_entry(BIT(m_port_7ffd_data, 4));
 	// select ram at 0x0c000-0x0ffff
 	m_bank_ram[3]->set_entry(m_port_7ffd_data & 0x07);
+	m_ula->bank3_pg_w(m_port_7ffd_data & 0x07);
 
 	m_screen->update_now();
 	if (BIT(m_port_7ffd_data, 3))
@@ -255,11 +260,7 @@ void spectrum_128_state::spectrum_128_update_memory()
 
 uint8_t spectrum_128_state::spectrum_port_r(offs_t offset)
 {
-	if (!machine().side_effects_disabled() && is_contended(offset))
-	{
-		content_early();
-		content_late();
-	}
+	m_ula->io_r(offset);
 
 	// Pass through to expansion device if present
 	if (m_exp->get_card_device())
@@ -381,12 +382,6 @@ bool spectrum_128_state::is_vram_write(offs_t offset) {
 		: spectrum_state::is_vram_write(offset);
 }
 
-bool spectrum_128_state::is_contended(offs_t offset) {
-	u8 pg = m_bank_ram[3]->entry();
-	return spectrum_state::is_contended(offset)
-		|| ((offset >= 0xc000 && offset <= 0xffff) && (pg & 1)); // Memory pages 1,3,5 and 7 are contended
-}
-
 u8 *spectrum_128_state::snow_pattern1_base(u8 i_reg)
 {
 	const bool is_alt_scr_selected = BIT(m_port_7ffd_data, 3);
@@ -427,7 +422,7 @@ void spectrum_128_state::spectrum_128(machine_config &config)
 	m_maincpu->set_m1_map(&spectrum_128_state::spectrum_128_fetch);
 	m_maincpu->set_vblank_int("screen", FUNC(spectrum_128_state::spec_interrupt));
 	m_maincpu->refresh_cb().set(FUNC(spectrum_128_state::spectrum_refresh_w));
-	m_maincpu->nomreq_cb().set(FUNC(spectrum_128_state::spectrum_nomreq));
+	m_maincpu->nomreq_cb().set("ula", FUNC(spectrum_ula_device::nomem_rq));
 	m_maincpu->busack_cb().set("dma", FUNC(dma_slot_device::bai_w));
 
 	config.set_maximum_quantum(attotime::from_hz(60));
@@ -439,9 +434,13 @@ void spectrum_128_state::spectrum_128(machine_config &config)
 
 	subdevice<gfxdecode_device>("gfxdecode")->set_info(spec128);
 
+	SPECTRUM_ULA_128K(config.replace(), m_ula);
+	m_ula->set_z80(m_maincpu);
+	m_ula->set_screen(m_screen, get_screen_area());
+
 	// sound hardware
 	AY_SLOT(config, "ay_slot", X1_128_SINCLAIR / 20, default_ay_slot_devices, "ay_ay8912")
-		.add_route(ALL_OUTPUTS, "mono", 0.25);
+		.add_route(ALL_OUTPUTS, "speakers", 0.25);
 
 	// expansion port
 	SPECTRUM_EXPANSION_SLOT(config.replace(), m_exp, spec128_expansion_devices, nullptr);
@@ -451,7 +450,7 @@ void spectrum_128_state::spectrum_128(machine_config &config)
 
 	dma_slot_device &dma(DMA_SLOT(config.replace(), "dma", X1_128_SINCLAIR / 10, default_dma_slot_devices, nullptr));
 	dma.set_io_space(m_maincpu, AS_IO);
-	dma.out_busreq_callback().set_inputline(m_maincpu, Z80_INPUT_LINE_BUSRQ);
+	dma.out_busreq_callback().set_inputline(m_maincpu, Z80_INPUT_LINE_BUSREQ);
 	dma.out_int_callback().set_inputline(m_maincpu, INPUT_LINE_IRQ0);
 	dma.in_mreq_callback().set([this](offs_t offset) { return m_program.read_byte(offset); });
 	dma.out_mreq_callback().set([this](offs_t offset, u8 data) { m_program.write_byte(offset, data); });
