@@ -12,7 +12,7 @@
 #define ROM128 (BIT(m_regs[MEM_CONFIG], 0))
 #define OFFS_512(_rl) (((m_regs[_rl + 1] & 1) << 8) | m_regs[_rl])
 
-#define VM static_cast<v_mode>(BIT(m_regs[V_CONFIG], 0, 2))
+#define VM v_mode(BIT(m_regs[V_CONFIG], 0, 2))
 
 static constexpr rectangle screen_area[4] = {
 	rectangle(tsconf_state::with_hblank(52), tsconf_state::with_hblank(256 + 51), tsconf_state::with_vblank(48), tsconf_state::with_vblank(192 + 47)), // 52|256|52 x 48-192-48
@@ -29,11 +29,6 @@ enum v_mode : u8
 	VM_TXT
 };
 
-static constexpr u32 tmp_tile_oversized_to_code(u16 code)
-{
-	return code / 64 * 64 * 8 + (code % 64);
-}
-
 // https://github.com/tslabs/zx-evo/blob/master/pentevo/vdac/vdac1/cpld/top.v
 static constexpr u8 pwm_to_rgb[32] = {
 	0, 10, 21, 31, 42, 53, 63, 74,
@@ -54,25 +49,19 @@ rectangle tsconf_state::get_screen_area()
 	return info;
 }
 
-void tsconf_state::tsconf_palette(palette_device &palette) const
-{
-	rgb_t colors[256] = {0};
-	palette.set_pen_colors(0, colors);
-}
-
 void tsconf_state::tsconf_update_bank0()
 {
 	u8 page0 = m_regs[PAGE0];
 	if (!NW0_MAP)
 	{
 		/* ROM: 0-SYS, 1-DOS, 2-128, 3-48 */
-		page0 = m_beta->started() && m_beta->is_active() ? ROM128 : (0x02 | ROM128);
-		page0 |= (m_regs[PAGE0] & 0xfc);
+		page0 &= 0xfc;
+		page0 |= m_beta->dos_r() ? ROM128 : (0x02 | ROM128);
 	}
 
-	if (W0_RAM)
+	if (W0_RAM || m_beta->vdos_r())
 	{
-		m_bank_ram[0]->set_entry(page0);
+		m_bank_ram[0]->set_entry(m_beta->vdos_r() ? 0xff : page0);
 		m_bank0_rom.disable();
 	}
 	else
@@ -82,6 +71,12 @@ void tsconf_state::tsconf_update_bank0()
 	}
 }
 
+void tsconf_state::update_io(int dos)
+{
+	m_io_shadow_view.select(dos ? 1 : 0);
+	tsconf_update_bank0();
+}
+
 void tsconf_state::tsconf_update_video_mode()
 {
 	rectangle visarea = screen_area[3];
@@ -112,6 +107,33 @@ u32 tsconf_state::get_vpage_offset()
 	return PAGE4K(m_regs[V_PAGE] & ((VM == VM_16C) ? 0xf8 : 0xf0));
 }
 
+u32 tsconf_state::screen_update(screen_device &screen, bitmap_rgb32 &bitmap, const rectangle &cliprect)
+{
+	rectangle scr = get_screen_area();
+	rectangle vis = screen.visible_area();
+	if (vis != scr)
+	{
+		rectangle bsides[4] = {
+			rectangle(vis.left(),      vis.right(),    vis.top(),        scr.top() - 1),
+			rectangle(vis.left(),      scr.left() - 1, scr.top(),        scr.bottom()),
+			rectangle(scr.right() + 1, vis.right(),    scr.top(),        scr.bottom()),
+			rectangle(vis.left(),      vis.right(),    scr.bottom() + 1, vis.bottom())
+		};
+		for (auto i = 0; i < 4; i++)
+		{
+			rectangle border = bsides[i] & cliprect;
+			if (!border.empty())
+				bitmap.fill(m_palette->pen_color(get_border_color()), border);
+		}
+	}
+
+	scr &= cliprect;
+	if (!scr.empty())
+		tsconf_update_screen(screen, bitmap, scr);
+
+	return 0;
+}
+
 /*
 Layered as:
  + Border - already updated with screen_update_spectrum()
@@ -122,44 +144,53 @@ Layered as:
  + Tiles 1
  + Sprites 2
 */
-void tsconf_state::spectrum_update_screen(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect)
+void tsconf_state::tsconf_update_screen(screen_device &screen, bitmap_rgb32 &bitmap, const rectangle &cliprect)
 {
 	if (!BIT(m_regs[V_CONFIG], 5))
 	{
 		if (VM == VM_ZX)
-			tsconf_UpdateZxScreenBitmap(screen, bitmap, cliprect);
+		{
+			tsconf_draw_zx(screen, bitmap, cliprect);
+		}
 		else if (VM == VM_TXT)
-			tsconf_UpdateTxtBitmap(bitmap, cliprect);
+		{
+			tsconf_draw_txt(bitmap, cliprect);
+		}
 		else
-			tsconf_UpdateGfxBitmap(bitmap, cliprect);
+		{
+			tsconf_draw_gfx(bitmap, cliprect);
+		}
 	}
 	else
 	{
-		bitmap.fill(get_border_color(), cliprect);
+		bitmap.fill(m_palette->pen_color(get_border_color()), cliprect);
 	}
 
 	if (!BIT(m_regs[V_CONFIG], 4))
 	{
 		screen.priority().fill(0, cliprect);
 		if (BIT(m_regs[TS_CONFIG], 5))
-			m_ts_tilemap[TM_TILES0]->draw(screen, bitmap, cliprect,
-										  BIT(m_regs[TS_CONFIG], 2) ? TILEMAP_DRAW_ALL_CATEGORIES : TILEMAP_DRAW_CATEGORY(1), 1);
+		{
+			m_ts_tilemap[TM_TILES0]->draw(
+					screen, bitmap, cliprect,
+					BIT(m_regs[TS_CONFIG], 2) ? TILEMAP_DRAW_ALL_CATEGORIES : TILEMAP_DRAW_CATEGORY(1), 1, 0);
+		}
 
 		if (BIT(m_regs[TS_CONFIG], 6))
-			m_ts_tilemap[TM_TILES1]->draw(screen, bitmap, cliprect,
-										  BIT(m_regs[TS_CONFIG], 3) ? TILEMAP_DRAW_ALL_CATEGORIES : TILEMAP_DRAW_CATEGORY(1), 2);
+		{
+			m_ts_tilemap[TM_TILES1]->draw(
+					screen, bitmap, cliprect,
+					BIT(m_regs[TS_CONFIG], 3) ? TILEMAP_DRAW_ALL_CATEGORIES : TILEMAP_DRAW_CATEGORY(1), 2, 0);
+		}
 
 		if (BIT(m_regs[TS_CONFIG], 7))
 		{
-			// draw_sprites(screen, bitmap, cliprect);
-			//  Avoid frequent expensive updates for now. Currently once per frame
-			if (cliprect.bottom() == get_screen_area().bottom() && cliprect.right() == get_screen_area().right())
-				draw_sprites(screen, bitmap, get_screen_area());
+			draw_sprites(screen, bitmap, get_screen_area());
 		}
 	}
 }
 
-void tsconf_state::tsconf_UpdateZxScreenBitmap(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect)
+void tsconf_state::tsconf_draw_zx(screen_device &screen, bitmap_rgb32 &bitmap, const rectangle &cliprect)
 {
 	u8 pal_offset = m_regs[PAL_SEL] << 4;
 	u8 *screen_location = m_ram->pointer() + PAGE4K(m_regs[V_PAGE]);
@@ -172,7 +203,7 @@ void tsconf_state::tsconf_UpdateZxScreenBitmap(screen_device &screen, bitmap_ind
 		u16 y = vpos - get_screen_area().top();
 		u8 *scr = &screen_location[((y & 7) << 8) | ((y & 0x38) << 2) | ((y & 0xc0) << 5) | (x >> 3)];
 		u8 *attr = &attrs_location[((y & 0xf8) << 2) | (x >> 3)];
-		u16 *pix = &(bitmap.pix(vpos, hpos));
+		u32 *pix = &(bitmap.pix(vpos, hpos));
 		while (hpos <= cliprect.right())
 		{
 			u16 ink = pal_offset | ((*attr >> 3) & 0x08) | (*attr & 0x07);
@@ -180,14 +211,14 @@ void tsconf_state::tsconf_UpdateZxScreenBitmap(screen_device &screen, bitmap_ind
 			u8 pix8 = (invert_attrs && (*attr & 0x80)) ? ~*scr : *scr;
 
 			for (u8 b = 0x80 >> (x & 0x07); b != 0 && hpos <= cliprect.right(); b >>= 1, x++, hpos++)
-				*pix++ = (pix8 & b) ? ink : pap;
+				*pix++ = m_palette->pen_color((pix8 & b) ? ink : pap);
 			scr++;
 			attr++;
 		}
 	}
 }
 
-void tsconf_state::tsconf_UpdateTxtBitmap(bitmap_ind16 &bitmap, const rectangle &cliprect)
+void tsconf_state::tsconf_draw_txt(bitmap_rgb32 &bitmap, const rectangle &cliprect)
 {
 	u8 *font_location = m_ram->pointer() + PAGE4K(m_regs[V_PAGE] ^ 0x01);
 	u8 pal_offset = m_regs[PAL_SEL] << 4;
@@ -200,20 +231,20 @@ void tsconf_state::tsconf_UpdateTxtBitmap(bitmap_ind16 &bitmap, const rectangle
 
 		// TODO? u16 x_offset = OFFS_512(G_X_OFFS_L);
 		u8 *text_location = m_ram->pointer() + PAGE4K(m_regs[V_PAGE]) + (y_offset / 8 * 256 + x / 8);
-		u16 *pix = &(bitmap.pix(vpos, hpos));
+		u32 *pix = &(bitmap.pix(vpos, hpos));
 		while (hpos <= cliprect.right())
 		{
-			u8 font_color = *(text_location + 128) & 0x0f;
-			u8 bg_color = (*(text_location + 128) & 0xf0) >> 4;
-			u8 char_x = *(font_location + (*text_location * 8) + (y_offset % 8));
+			u8 font_color = text_location[128] & 0x0f;
+			u8 bg_color = (text_location[128] & 0xf0) >> 4;
+			u8 char_x = font_location[(text_location[0] * 8) + (y_offset % 8)];
 			for (u8 b = 0x80 >> (x & 0x07); b != 0 && hpos <= cliprect.right(); b >>= 1, x++, hpos++)
-				*pix++ = pal_offset | ((char_x & b) ? font_color : bg_color);
+				*pix++ = m_palette->pen_color(pal_offset | ((char_x & b) ? font_color : bg_color));
 			text_location++;
 		}
 	}
 }
 
-void tsconf_state::tsconf_UpdateGfxBitmap(bitmap_ind16 &bitmap, const rectangle &cliprect)
+void tsconf_state::tsconf_draw_gfx(bitmap_rgb32 &bitmap, const rectangle &cliprect)
 {
 	u8 pal_offset = m_regs[PAL_SEL] << 4;
 	for (u16 vpos = cliprect.top(); vpos <= cliprect.bottom(); vpos++)
@@ -221,13 +252,13 @@ void tsconf_state::tsconf_UpdateGfxBitmap(bitmap_ind16 &bitmap, const rectangle
 		u16 y_offset = (OFFS_512(G_Y_OFFS_L) + m_gfx_y_frame_offset + vpos) & 0x1ff;
 		u16 x_offset = (OFFS_512(G_X_OFFS_L) + (cliprect.left() - get_screen_area().left())) & 0x1ff;
 		u8 *video_location = m_ram->pointer() + get_vpage_offset() + ((y_offset * 512 + x_offset) >> (2 - VM));
-		u16 *bm = &(bitmap.pix(vpos, cliprect.left()));
+		u32 *bm = &(bitmap.pix(vpos, cliprect.left()));
 		s16 width = cliprect.width();
 		if (VM == VM_16C)
 		{
 			if (x_offset & 1)
 			{
-				*bm++ = pal_offset | (*video_location++ & 0x0f);
+				*bm++ = m_palette->pen_color(pal_offset | (*video_location++ & 0x0f));
 				x_offset++;
 				width--;
 			}
@@ -236,9 +267,13 @@ void tsconf_state::tsconf_UpdateGfxBitmap(bitmap_ind16 &bitmap, const rectangle
 				if (x_offset == 512)
 					video_location -= 256;
 				u8 pix = *video_location++;
-				*bm++ = pal_offset | (pix >> 4);
+				rgb_t pen = m_palette->pen_color(pal_offset | (pix >> 4));
+				*bm++ = pen;
 				if (width != 1)
-					*bm++ = pal_offset | (pix & 0x0f);
+				{
+					pen = m_palette->pen_color(pal_offset | (pix & 0x0f));
+					*bm++ = pen;
+				}
 			}
 		}
 		else // VM_256C
@@ -247,7 +282,8 @@ void tsconf_state::tsconf_UpdateGfxBitmap(bitmap_ind16 &bitmap, const rectangle
 			{
 				if (x_offset == 512)
 					video_location -= 512;
-				*bm++ = *video_location++;
+				rgb_t pen = m_palette->pen_color(*video_location++);
+				*bm++ = pen;
 			}
 		}
 	}
@@ -262,84 +298,158 @@ SFILE   Reg.16  7       6       5       4       3       2       1       0
 4       R2L     TNUM[7:0]
 5       R2H     SPAL[7:4]                       TNUM[11:8]
 */
-void tsconf_state::draw_sprites(screen_device &screen_d, bitmap_ind16 &bitmap, const rectangle &cliprect)
+void tsconf_state::draw_sprites(screen_device &screen_d, bitmap_rgb32 &bitmap, const rectangle &cliprect)
 {
-	rectangle screen = get_screen_area();
-
-	u8 layer = 0;
-	u8 *sinfo = m_sfile->pointer() + 1;
-	u8 sid = 1;
-	// Higher Sprite draws on top. Prepare to iterate backwards.
-	for (; sid < 85 && layer < 3; sid++)
+	if (m_sprites_cache.empty())
 	{
-		if (BIT(*sinfo, 6))
-			layer++;
-		sinfo += 6;
-	}
-	sinfo -= 1;
-
-	for (; sid; sid--)
-	{
-		s16 y = *sinfo++;
-		y |= BIT(*sinfo, 0) << 8;
-		y += screen.top() - (y >= screen.height() ? 512 : 0);
-		u8 height8 = BIT(*sinfo, 1, 3);
-		layer -= BIT(*sinfo, 6);
-		if (!BIT(*sinfo, 5))
+		const rectangle screen = get_screen_area();
+		u8 layer = 0;
+		u8 *sinfo = m_sfile->pointer();
+		for (u8 sid = 0; sid < 85 && layer < 3; sid++)
 		{
-			// sprite disabled -> move to previous
-			sinfo -= 7;
-		}
-		else
-		{
-			bool flipy = BIT(*sinfo++, 7);
-			s16 x = *sinfo++;
-			x |= BIT(*sinfo, 0) << 8;
-			x += screen.left() - (x >= screen.width() ? 512 : 0);
-			u8 width8 = BIT(*sinfo, 1, 3);
-			bool flipx = BIT(*sinfo++, 7);
-			u16 code = *sinfo++;
-			code |= BIT(*sinfo, 0, 4) << 8;
-			u8 pal = BIT(*sinfo, 4, 4);
-			sinfo -= 11;
-
-			u32 pmask = layer ? (layer == 1 ? GFX_PMASK_2 : 0) : (GFX_PMASK_1 | GFX_PMASK_2);
-
-			u8 tile_row = code / 64 + flipy * height8;
-			for (auto iy = y; iy <= y + height8 * 8; iy = iy + 8)
+			s16 y = *sinfo++;
+			y |= BIT(*sinfo, 0) << 8;
+			y += screen.top() - (y >= screen.height() ? 512 : 0);
+			const u8 height8 = BIT(*sinfo, 1, 3);
+			const bool leap_next = BIT(*sinfo, 6);
+			if (!BIT(*sinfo, 5))
 			{
-				u8 tile_col = (code % 64) + flipx * width8;
-				for (auto ix = x; ix <= x + width8 * 8; ix = ix + 8)
-				{
-					m_gfxdecode->gfx(TM_SPRITES)->prio_transpen(bitmap, cliprect, tmp_tile_oversized_to_code((tile_row % 64) * 64 + (tile_col % 64)), pal, flipx, flipy, ix, iy, screen_d.priority(), pmask, 0);
-					tile_col += flipx ? -1 : 1;
-				}
-				tile_row += flipy ? -1 : 1;
+				// sprite disabled -> move to next
+				sinfo += 5;
 			}
+			else
+			{
+				const bool flipy = BIT(*sinfo++, 7);
+				s16 x = *sinfo++;
+				x |= BIT(*sinfo, 0) << 8;
+				x += screen.left() - (x >= screen.width() ? 512 : 0);
+				const u8 width8 = BIT(*sinfo, 1, 3);
+				const bool flipx = BIT(*sinfo++, 7);
+				u16 code = *sinfo++;
+				code |= BIT(*sinfo, 0, 4) << 8;
+				const u8 pal = BIT(*sinfo++, 4, 4);
+
+				u8 tile_row = code / 64 + flipy * height8;
+				const u32 pmask = (~1) << layer;
+				for (auto iy = y; iy <= y + height8 * 8; iy = iy + 8)
+				{
+					u8 tile_col = (code % 64) + flipx * width8;
+					for (auto ix = x; ix <= x + width8 * 8; ix = ix + 8)
+					{
+						sprite_data spr = {};
+						spr.code = (tile_row % 64) * 64 + (tile_col % 64);
+						spr.color = pal;
+						spr.flipx = flipx;
+						spr.flipy = flipy;
+						spr.destx = ix;
+						spr.desty = iy;
+						spr.pmask = pmask;
+
+						m_sprites_cache.push_back(spr);
+						tile_col += flipx ? -1 : 1;
+					}
+					tile_row += flipy ? -1 : 1;
+				}
+			}
+			layer += leap_next;
 		}
 	}
+
+	// Higher Sprite draws on top. Iterate backwards.
+	for (auto spr = m_sprites_cache.rbegin(); spr != m_sprites_cache.rend(); ++spr)
+	{
+		m_gfxdecode->gfx(TM_SPRITES)->prio_transpen(bitmap, cliprect,
+			spr->code, spr->color, spr->flipx, spr->flipy, spr->destx, spr->desty,
+			screen_d.priority(), spr->pmask, 0);
+	}
+
+}
+
+attotime tsconf_state::copper_until_pos_r(u16 pos)
+{
+	const u8 mode = pos >> 14;
+	switch(mode)
+	{
+		case 0b11: // line
+		case 0b10: // x
+			{
+				const int vpos = m_screen->vpos();
+				const int x = (mode == 0b11) ? screen_area[3].right() + 1 : ((pos & 0xff) << 1);
+				if (m_screen->hpos() < x)
+					return m_screen->time_until_pos(vpos, x);
+				else
+					return m_screen->time_until_pos((vpos + 1) % m_screen->height(), x);
+			}
+			break;
+		case 0b01: // y
+			{
+				const u8 y = pos & 0x1ff;
+				if (m_screen->vpos() == y)
+					return attotime::zero;
+				else
+					return m_screen->time_until_pos(y);
+			}
+			break;
+		case 0b00: // frame
+		default:
+			return m_screen->time_until_pos(0, 0);
+			break;
+	}
+}
+
+u8 tsconf_state::ram_bank_read(u8 bank, offs_t offset)
+{
+	if (!machine().side_effects_disabled() && ((m_regs[SYS_CONFIG] & 3) == 2)) // 14Mhz
+	{
+		const u16 cpu_hi_addr = ((m_regs[PAGE0 + bank]) << 5) | BIT(offset, 9, 5);
+		if (!(m_regs[CACHE_CONFIG] & (1 << bank)) || (cpu_hi_addr != m_cache_line_addr))
+		{
+			m_cache_line_addr = cpu_hi_addr;
+			m_maincpu->adjust_icount(-2);
+		}
+	}
+
+	return reinterpret_cast<u8 *>(m_bank_ram[bank]->base())[offset];
 }
 
 void tsconf_state::ram_bank_write(u8 bank, offs_t offset, u8 data)
 {
+	const u16 cpu_hi_addr = ((m_regs[PAGE0 + bank]) << 5) | BIT(offset, 9, 5);
+	if (cpu_hi_addr == m_cache_line_addr)
+		m_cache_line_addr = -1; // invalidate cache line
+
 	if (BIT(m_regs[FMAPS], 4))
 	{
 		offs_t machine_addr = PAGE4K(bank) + offset;
 		offs_t fmap_addr = BIT(m_regs[FMAPS], 0, 4) << 12;
-		if ((machine_addr >= fmap_addr) && (machine_addr < (fmap_addr + 256 * 5)))
+		if ((machine_addr >= fmap_addr) && (machine_addr < (fmap_addr + 0x800)))
 		{
 			u16 addr_w = machine_addr - fmap_addr;
-			if (addr_w < 512)
+			if (addr_w < 0x200)
 				cram_write(addr_w, data);
-			else if (addr_w < 1024)
-				m_sfile->write(addr_w - 512, data);
-			else
-				tsconf_port_xxaf_w((addr_w - 1024) << 8, data);
+			else if (addr_w < 0x400)
+			{
+				m_sprites_cache.clear();
+				m_sfile->write(addr_w - 0x200, data);
+			}
+			else if (addr_w < 0x500)
+				tsconf_port_xxaf_w((addr_w - 0x400) << 8, data);
+			else if (addr_w < 0x600)
+				; // reserved
+			else if (m_copper != nullptr)
+				m_copper->cl_data_w(addr_w - 0x600, data);
 		}
 	}
 
-	if (bank > 0 || (W0_WE && W0_RAM))
+	if (bank > 0 || W0_WE)
 		ram_page_write(m_regs[PAGE0 + bank], offset, data);
+	else if (!bank && m_beta->vdos_r())
+		ram_page_write(0xff, offset, data);
+}
+
+static int tiles_offset_to_raw(int t_offset)
+{
+	return bitswap<17>(t_offset, 16, 15, 14, 13, 12, 11, 7, 6, 5, 4, 3, 2, 10, 9, 8, 1, 0) << 1;
 }
 
 void tsconf_state::ram_page_write(u8 page, offs_t offset, u8 data)
@@ -352,11 +462,23 @@ void tsconf_state::ram_page_write(u8 page, offs_t offset, u8 data)
 	}
 	else
 	{
-		if (ram_addr >= PAGE4K(m_regs[T0_G_PAGE] & 0xf8) && ram_addr < PAGE4K((m_regs[T0_G_PAGE] & 0xf8) + 8))
-			m_gfxdecode->gfx(TM_TILES0)->mark_all_dirty();
+		const int t0_offset = ram_addr - PAGE4K(m_regs[T0_G_PAGE] & 0xf8);
+		if ((t0_offset >= 0) && (t0_offset < PAGE4K(8)))
+		{
+			const int raw_offset = tiles_offset_to_raw(t0_offset);
+			m_tiles_raw[0][raw_offset] = data >> 4;
+			m_tiles_raw[0][raw_offset + 1] = data & 0x0f;
+			m_ts_tilemap[TM_TILES0]->mark_all_dirty();
+		}
 
-		if (ram_addr >= PAGE4K(m_regs[T1_G_PAGE] & 0xf8) && ram_addr < PAGE4K((m_regs[T1_G_PAGE] & 0xf8) + 8))
-			m_gfxdecode->gfx(TM_TILES1)->mark_all_dirty();
+		const int t1_offset = ram_addr - PAGE4K(m_regs[T1_G_PAGE] & 0xf8);
+		if ((t1_offset >= 0) && (t1_offset < PAGE4K(8)))
+		{
+			const int raw_offset = tiles_offset_to_raw(t1_offset);
+			m_tiles_raw[1][raw_offset] = data >> 4;
+			m_tiles_raw[1][raw_offset + 1] = data & 0x0f;
+			m_ts_tilemap[TM_TILES1]->mark_all_dirty();
+		}
 	}
 
 	if (ram_addr >= get_vpage_offset() && ram_addr < get_vpage_offset() + PAGE4K((VM == VM_16C) ? 8 : 16))
@@ -365,15 +487,20 @@ void tsconf_state::ram_page_write(u8 page, offs_t offset, u8 data)
 	if (ram_addr >= PAGE4K(m_regs[m_regs[V_PAGE] ^ 0x01]) && ram_addr < PAGE4K(m_regs[m_regs[V_PAGE] ^ 0x01] + 1))
 		m_gfxdecode->gfx(TM_TS_CHAR)->mark_all_dirty();
 
-	if (ram_addr >= PAGE4K(m_regs[SG_PAGE] & 0xf8) && ram_addr < PAGE4K((m_regs[SG_PAGE] & 0xf8) + 8))
-		m_gfxdecode->gfx(TM_SPRITES)->mark_all_dirty();
+	const int spr_offset = ram_addr - PAGE4K(m_regs[SG_PAGE] & 0xf8);
+	if ((spr_offset >= 0) && (spr_offset < PAGE4K(8)))
+	{
+		const int raw_offset = tiles_offset_to_raw(spr_offset);
+		m_sprites_raw[raw_offset] = data >> 4;
+		m_sprites_raw[raw_offset + 1] = data & 0x0f;
+	}
 
-	m_ram->write(ram_addr, data);
+	m_ram->pointer()[ram_addr] = data;
 }
 
 u16 tsconf_state::ram_read16(offs_t offset)
 {
-	return (m_ram->read(offset & ~offs_t(1)) << 8) | m_ram->read(offset | 1);
+	return (m_ram->pointer()[offset & ~offs_t(1)] << 8) | m_ram->pointer()[offset | 1];
 }
 
 void tsconf_state::ram_write16(offs_t offset, u16 data)
@@ -390,30 +517,36 @@ u16 tsconf_state::spi_read16()
 
 void tsconf_state::cram_write(u16 offset, u8 data)
 {
+	m_screen->update_now();
 	u16 dest = offset & 0x1ff;
 	m_cram->write(dest, data);
 	u8 pen = dest >> 1;
 	rgb_t rgb = from_pwm((m_cram->read(dest | 1) << 8 | m_cram->read(dest & 0x1fe)));
 	m_palette->set_pen_color(pen, rgb);
-};
+}
 
 void tsconf_state::cram_write16(offs_t offset, u16 data)
 {
 	cram_write(offset & 0x1fe, data >> 8);
 	cram_write(offset | 1, data & 0xff);
-};
+}
 
 void tsconf_state::sfile_write16(offs_t offset, u16 data)
 {
+	m_sprites_cache.clear();
+
 	u16 dest = offset & 0x1fe;
 	m_sfile->write(dest, data >> 8);
 	m_sfile->write(dest | 1, data & 0xff);
-};
+}
 
 u8 tsconf_state::tsconf_port_xx1f_r(offs_t offset) {
-	return m_beta->started() && m_beta->is_active()
-		? m_beta->status_r()
-		: 0x00; // TODO kempston read
+	return 0x00; // TODO kempston read
+}
+
+void tsconf_state::bank3_set_page(u8 page)
+{
+	tsconf_port_xxaf_w(PAGE3 << 8, page);
 }
 
 void tsconf_state::tsconf_port_7ffd_w(u8 data)
@@ -439,7 +572,7 @@ void tsconf_state::tsconf_port_7ffd_w(u8 data)
 void tsconf_state::tsconf_ula_w(offs_t offset, u8 data)
 {
 	spectrum_ula_w(offset, data);
-	tsconf_port_xxaf_w(BORDER << 8, (data & 0x07) | (m_regs[PAL_SEL] << 4));
+	tsconf_port_xxaf_w(BORDER << 8, 0xf0 | (data & 0x07));
 }
 
 u8 tsconf_state::tsconf_port_xxaf_r(offs_t port)
@@ -473,6 +606,20 @@ u8 tsconf_state::tsconf_port_xxaf_r(offs_t port)
 	return data;
 }
 
+void tsconf_state::copy_tiles_to_raw(const u8 *tiles_src, u8 *raw_target)
+{
+	for (u32 ln = 0; ln < PAGE4K(8); ln += 4)
+	{
+		int targ = tiles_offset_to_raw(ln);
+		for (u8 x = 0; x < 4; ++x)
+		{
+			const u8 data = tiles_src[ln + x];
+			raw_target[targ + (x << 1)] = data >> 4;
+			raw_target[targ + (x << 1) + 1] = data & 0x0f;
+		}
+	}
+}
+
 void tsconf_state::tsconf_port_xxaf_w(offs_t port, u8 data)
 {
 	u8 nreg = port >> 8;
@@ -494,7 +641,7 @@ void tsconf_state::tsconf_port_xxaf_w(offs_t port, u8 data)
 	case T0_X_OFFSET_H:
 	case T1_X_OFFSET_L:
 	case T1_X_OFFSET_H:
-		m_scanline_delayed_regs_update[static_cast<tsconf_regs>(nreg)] = data;
+		m_scanline_delayed_regs_update[tsconf_regs(nreg)] = data;
 		break;
 
 	default:
@@ -539,6 +686,11 @@ void tsconf_state::tsconf_port_xxaf_w(offs_t port, u8 data)
 		m_bank_ram[3]->set_entry(data);
 		break;
 
+	case COPPER:
+		if (m_copper != nullptr)
+			m_copper->copper_en_w(data);
+		break;
+
 	case DMAS_ADDRESS_L:
 		m_dma->set_saddr_l(data);
 		break;
@@ -605,7 +757,7 @@ void tsconf_state::tsconf_port_xxaf_w(offs_t port, u8 data)
 		break;
 
 	case SG_PAGE:
-		m_gfxdecode->gfx(TM_SPRITES)->set_source(m_ram->pointer() + PAGE4K(data & 0xf8));
+		copy_tiles_to_raw(m_ram->pointer() + PAGE4K(data & 0xf8), m_sprites_raw.target());
 		break;
 
 	case SYS_CONFIG:
@@ -620,11 +772,14 @@ void tsconf_state::tsconf_port_xxaf_w(offs_t port, u8 data)
 		update_frame_timer();
 		break;
 
+	case FDD_VIRT:
+		m_beta->io_forced_w(BIT(data, 7));
+		m_beta->fddvirt_w(data & 0x0f);
+		break;
+
 	case FMAPS:
 	case TS_CONFIG:
 	case INT_MASK:
-	// TODO
-	case FDD_VIRT:
 	case CACHE_CONFIG:
 		break;
 
@@ -637,8 +792,8 @@ u8 tsconf_state::tsconf_port_f7_r(offs_t offset)
 {
 	// BFF7
 	return  (m_port_f7_ext == PS2KEYBOARDS_LOG && m_glukrs->address_r() == 0xf0)
-		? m_keyboard->read()
-		: m_glukrs->data_r();
+			? m_keyboard->read()
+			: m_glukrs->data_r();
 }
 
 void tsconf_state::tsconf_port_f7_w(offs_t offset, u8 data)
@@ -665,14 +820,14 @@ void tsconf_state::tsconf_port_f7_w(offs_t offset, u8 data)
 			if (m_glukrs->address_r() == 0xf0)
 			{
 				u8 m_fx[0xf] = {0xff};
-				m_port_f7_ext = static_cast<gluk_ext>(data);
+				m_port_f7_ext = gluk_ext(data);
 				switch (m_port_f7_ext)
 				{
 				case CONF_VERSION:
 				{
-					strcpy((char *)m_fx, "M.A.M.E.");
+					strcpy((char *)m_fx, "MAME");
 					PAIR16 m_ver;
-					m_ver.w = ((22 << 9) | (02 << 5) | 8); // y.m.d
+					m_ver.w = ((25 << 9) | (07 << 5) | 31); // y.m.d
 					m_fx[0x0c] = m_ver.b.l;
 					m_fx[0x0d] = m_ver.b.h;
 					break;
@@ -686,10 +841,8 @@ void tsconf_state::tsconf_port_f7_w(offs_t offset, u8 data)
 				}
 				for (u8 i = 0; i < 0xf; i++)
 				{
-					m_glukrs->address_w(0xf0 + i);
-					m_glukrs->data_w(m_fx[i]);
+					m_glukrs->write_direct(0xf0 + i, m_fx[i]);
 				}
-				m_glukrs->address_w(0xf0);
 			}
 			else
 			{
@@ -741,19 +894,13 @@ void tsconf_state::tsconf_spi_miso_w(u8 data)
 	m_zctl_di |= data;
 }
 
-void tsconf_state::tsconf_ay_address_w(u8 data)
-{
-	if ((m_mod_ay->read() == 1) && ((data & 0xfe) == 0xfe))
-		m_ay_selected = data & 1;
-	else
-		m_ay[m_ay_selected]->address_w(data);
-}
-
 IRQ_CALLBACK_MEMBER(tsconf_state::irq_vector)
 {
 	u8 vector = 0xff;
 	if (m_int_mask & 1)
+	{
 		m_int_mask &= ~1;
+	}
 	else if (m_int_mask & 2)
 	{
 		m_int_mask &= ~2;
@@ -766,7 +913,7 @@ IRQ_CALLBACK_MEMBER(tsconf_state::irq_vector)
 	}
 
 	if (!m_int_mask)
-		m_maincpu->set_input_line(0, CLEAR_LINE);
+		m_maincpu->set_input_line(INPUT_LINE_IRQ0, CLEAR_LINE);
 
 	return vector;
 }
@@ -774,8 +921,6 @@ IRQ_CALLBACK_MEMBER(tsconf_state::irq_vector)
 TIMER_CALLBACK_MEMBER(tsconf_state::irq_off)
 {
 	m_int_mask &= ~1;
-	if (!m_int_mask)
-		m_maincpu->set_input_line(0, CLEAR_LINE);
 }
 
 void tsconf_state::update_frame_timer()
@@ -787,10 +932,14 @@ void tsconf_state::update_frame_timer()
 	{
 		next = m_screen->time_until_pos(vpos, hpos << 1);
 		if (next >= m_screen->frame_period())
+		{
 			next = attotime::zero;
+		}
 	}
 	else
+	{
 		next = attotime::never;
+	}
 
 	m_frame_irq_timer->adjust(next);
 }
@@ -804,20 +953,18 @@ INTERRUPT_GEN_MEMBER(tsconf_state::tsconf_vblank_interrupt)
 
 void tsconf_state::dma_ready(int line)
 {
-	if (BIT(m_regs[INT_MASK], 2))
+	if (BIT(m_regs[INT_MASK], 2) && !(m_beta->vdos_r() || m_update_on_m1))
 	{
-		if (!m_int_mask)
-			m_maincpu->set_input_line(INPUT_LINE_IRQ0, ASSERT_LINE);
+		m_maincpu->set_input_line(INPUT_LINE_IRQ0, ASSERT_LINE);
 		m_int_mask |= 4;
 	}
 }
 
 TIMER_CALLBACK_MEMBER(tsconf_state::irq_frame)
 {
-	if (BIT(m_regs[INT_MASK], 0))
+	if (BIT(m_regs[INT_MASK], 0) && !(m_beta->vdos_r() || m_update_on_m1))
 	{
-		if (!m_int_mask)
-			m_maincpu->set_input_line(INPUT_LINE_IRQ0, ASSERT_LINE);
+		m_maincpu->set_input_line(INPUT_LINE_IRQ0, ASSERT_LINE);
 		m_irq_off_timer->adjust(attotime::from_ticks(32, m_maincpu->unscaled_clock()));
 		m_int_mask |= 1;
 	}
@@ -825,10 +972,9 @@ TIMER_CALLBACK_MEMBER(tsconf_state::irq_frame)
 
 TIMER_CALLBACK_MEMBER(tsconf_state::irq_scanline)
 {
-	if (BIT(m_regs[INT_MASK], 1))
+	if (BIT(m_regs[INT_MASK], 1) && !(m_beta->vdos_r() || m_update_on_m1))
 	{
-		if (!m_int_mask)
-			m_maincpu->set_input_line(INPUT_LINE_IRQ0, ASSERT_LINE);
+		m_maincpu->set_input_line(INPUT_LINE_IRQ0, ASSERT_LINE);
 		m_int_mask |= 2;
 	}
 
@@ -849,16 +995,18 @@ TIMER_CALLBACK_MEMBER(tsconf_state::irq_scanline)
 		case G_Y_OFFS_L:
 		case G_Y_OFFS_H:
 			m_gfx_y_frame_offset = screen_vpos < get_screen_area().top()
-				? -get_screen_area().top()
-				: -screen_vpos;
+					? -get_screen_area().top()
+					: -screen_vpos;
 			break;
 
 		case T0_G_PAGE:
-			m_gfxdecode->gfx(TM_TILES0)->set_source(m_ram->pointer() + PAGE4K(val & 0xf8));
+			copy_tiles_to_raw(m_ram->pointer() + PAGE4K(val & 0xf8), m_tiles_raw[0].target());
+			m_ts_tilemap[TM_TILES0]->mark_all_dirty();
 			break;
 
 		case T1_G_PAGE:
-			m_gfxdecode->gfx(TM_TILES1)->set_source(m_ram->pointer() + PAGE4K(val & 0xf8));
+			copy_tiles_to_raw(m_ram->pointer() + PAGE4K(val & 0xf8), m_tiles_raw[1].target());
+			m_ts_tilemap[TM_TILES1]->mark_all_dirty();
 			break;
 
 		case T0_X_OFFSET_L:
@@ -883,6 +1031,11 @@ TIMER_CALLBACK_MEMBER(tsconf_state::irq_scanline)
 
 u8 tsconf_state::beta_neutral_r(offs_t offset)
 {
+	if (!machine().side_effects_disabled() && m_update_on_m1)
+	{
+		m_update_on_m1 = false;
+		m_beta->on_m1_w();
+	}
 	return m_program.read_byte(offset);
 }
 
@@ -890,13 +1043,14 @@ u8 tsconf_state::beta_enable_r(offs_t offset)
 {
 	if (!machine().side_effects_disabled())
 	{
-		if (!W0_RAM && m_bank_rom[0]->entry() == 3)
+		if (m_update_on_m1)
 		{
-			if (m_beta->started() && !m_beta->is_active())
-			{
-				m_beta->enable();
-				tsconf_update_bank0();
-			}
+			m_update_on_m1 = false;
+			m_beta->on_m1_w();
+		}
+		if (ROM128 && !NW0_MAP)
+		{
+			m_beta->enable_w(1);
 		}
 	}
 	return m_program.read_byte(offset + 0x3d00);
@@ -906,11 +1060,12 @@ u8 tsconf_state::beta_disable_r(offs_t offset)
 {
 	if (!machine().side_effects_disabled())
 	{
-		if (m_beta->started() && m_beta->is_active())
+		if (m_update_on_m1)
 		{
-			m_beta->disable();
-			tsconf_update_bank0();
+			m_update_on_m1 = false;
+			m_beta->on_m1_w();
 		}
+		m_beta->enable_w(0);
 	}
 	return m_program.read_byte(offset + 0x4000);
 }
