@@ -11,11 +11,6 @@
     - f1gp2's hardware is very similar to Lethal Crash Race, main difference
       being an extra 68000.
 
-    TODO:
-    - Hook up link for Multi Player game mode. Currently they boot with
-      link set to multiple, but the ID changes every boot (and will black
-      out if a multiplayer game is started).
-
     f1gp:
     - gfxctrl register not understood - handling of fg/sprite priority to fix
       "continue" screen is just a kludge.
@@ -34,6 +29,8 @@
 #include "vsystem_spr.h"
 #include "vsystem_spr2.h"
 
+#include "bus/rs232/rs232.h"
+
 #include "cpu/m68000/m68000.h"
 #include "cpu/z80/z80.h"
 #include "machine/6850acia.h"
@@ -84,6 +81,8 @@ public:
 		m_palette(*this, "palette"),
 		m_soundlatch(*this, "soundlatch"),
 		m_acia(*this, "acia"),
+		m_rs232_out(*this, "com_out"),
+		m_rs232_in(*this, "com_in"),
 		m_rozgfxram(*this, "rozgfxram"),
 		m_spr_old(*this, "vsystem_spr_old%u", 1U)
 	{ }
@@ -120,22 +119,25 @@ protected:
 	required_device<palette_device> m_palette;
 	optional_device<generic_latch_8_device> m_soundlatch; // not f1gpbl
 	required_device<acia6850_device> m_acia;
+	required_device<rs232_port_device> m_rs232_out;
+	required_device<rs232_port_device> m_rs232_in;
 
 	void sh_bankswitch_w(uint8_t data);
-	uint8_t command_pending_r();
+	uint8_t soundlatch_pending_r();
+	void soundlatch_pending_w(int state);
 	void rozvideoram_w(offs_t offset, uint16_t data, uint16_t mem_mask = ~0);
 	void fgvideoram_w(offs_t offset, uint16_t data, uint16_t mem_mask = ~0);
 	void fgscroll_w(offs_t offset, uint16_t data, uint16_t mem_mask = ~0);
 	void gfxctrl_w(uint8_t data);
 	TILE_GET_INFO_MEMBER(get_fg_tile_info);
 
-	virtual void machine_start() override;
-	virtual void machine_reset() override;
-	virtual void video_start() override;
+	virtual void machine_start() override ATTR_COLD;
+	virtual void machine_reset() override ATTR_COLD;
+	virtual void video_start() override ATTR_COLD;
 
-	void f1gp_cpu2_map(address_map &map);
-	void sound_io_map(address_map &map);
-	void sound_map(address_map &map);
+	void f1gp_cpu2_map(address_map &map) ATTR_COLD;
+	void sound_io_map(address_map &map) ATTR_COLD;
+	void sound_map(address_map &map) ATTR_COLD;
 
 private:
 	// memory pointers
@@ -151,9 +153,9 @@ private:
 	uint32_t screen_update_f1gp(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect);
 	uint32_t screen_update_f1gpbl(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect);
 	void f1gpbl_draw_sprites(screen_device &screen, bitmap_ind16 &bitmap,const rectangle &cliprect);
-	void f1gp_cpu1_map(address_map &map);
-	void f1gpbl_cpu1_map(address_map &map);
-	void f1gpbl_cpu2_map(address_map &map);
+	void f1gp_cpu1_map(address_map &map) ATTR_COLD;
+	void f1gpbl_cpu1_map(address_map &map) ATTR_COLD;
+	void f1gpbl_cpu2_map(address_map &map) ATTR_COLD;
 };
 
 class f1gp2_state : public f1gp_state
@@ -167,8 +169,8 @@ public:
 	void f1gp2(machine_config &config);
 
 protected:
-	virtual void machine_reset() override;
-	virtual void video_start() override;
+	virtual void machine_reset() override ATTR_COLD;
+	virtual void video_start() override ATTR_COLD;
 
 private:
 	// video-related
@@ -182,12 +184,10 @@ private:
 	TILE_GET_INFO_MEMBER(get_roz_tile_info);
 
 	uint32_t screen_update(screen_device &screen, bitmap_ind16 &bitmap, const rectangle &cliprect);
-	void f1gp2_cpu1_map(address_map &map);
+	void f1gp2_cpu1_map(address_map &map) ATTR_COLD;
 };
 
 
-// video
-
 /***************************************************************************
 
   Callbacks for the TileMap code
@@ -205,14 +205,14 @@ TILE_GET_INFO_MEMBER(f1gp_state::get_roz_tile_info)
 {
 	int const code = m_rozvideoram[tile_index];
 
-	tileinfo.set(3, code & 0x7ff, code >> 12, 0);
+	tileinfo.set(1, code & 0x7ff, code >> 12, 0);
 }
 
 TILE_GET_INFO_MEMBER(f1gp2_state::get_roz_tile_info)
 {
 	int const code = m_rozvideoram[tile_index];
 
-	tileinfo.set(2, (code & 0x7ff) + (m_roz_bank << 11), code >> 12, 0);
+	tileinfo.set(1, (code & 0x7ff) + (m_roz_bank << 11), code >> 12, 0);
 }
 
 
@@ -222,7 +222,6 @@ TILE_GET_INFO_MEMBER(f1gp2_state::get_roz_tile_info)
 
 ***************************************************************************/
 
-
 void f1gp_state::video_start()
 {
 	m_roz_tilemap = &machine().tilemap().create(*m_gfxdecode, tilemap_get_info_delegate(*this, FUNC(f1gp_state::get_roz_tile_info)), TILEMAP_SCAN_ROWS, 16, 16, 64, 64);
@@ -263,7 +262,7 @@ void f1gp2_state::video_start()
 void f1gp_state::rozgfxram_w(offs_t offset, uint16_t data, uint16_t mem_mask)
 {
 	COMBINE_DATA(&m_rozgfxram[offset]);
-	m_gfxdecode->gfx(3)->mark_dirty(offset / 64);
+	m_gfxdecode->gfx(1)->mark_dirty(offset / 64);
 }
 
 void f1gp_state::rozvideoram_w(offs_t offset, uint16_t data, uint16_t mem_mask)
@@ -320,13 +319,13 @@ uint32_t f1gp_state::screen_update_f1gp(screen_device &screen, bitmap_ind16 &bit
 	// quick kludge for "continue" screen priority
 	if (m_gfxctrl == 0x00)
 	{
-		m_spr_old[0]->turbofrc_draw_sprites(m_sprvram[0], m_sprvram[0].bytes(), 0, bitmap, cliprect, screen.priority(), 0x02);
-		m_spr_old[1]->turbofrc_draw_sprites(m_sprvram[1], m_sprvram[1].bytes(), 0, bitmap, cliprect, screen.priority(), 0x02);
+		m_spr_old[0]->draw_sprites(m_sprvram[0], m_sprvram[0].bytes(), 0, bitmap, cliprect, screen.priority(), 0x02);
+		m_spr_old[1]->draw_sprites(m_sprvram[1], m_sprvram[1].bytes(), 0, bitmap, cliprect, screen.priority(), 0x02);
 	}
 	else
 	{
-		m_spr_old[0]->turbofrc_draw_sprites(m_sprvram[0], m_sprvram[0].bytes(), 0, bitmap, cliprect, screen.priority(), 0x00);
-		m_spr_old[1]->turbofrc_draw_sprites(m_sprvram[1], m_sprvram[1].bytes(), 0, bitmap, cliprect, screen.priority(), 0x02);
+		m_spr_old[0]->draw_sprites(m_sprvram[0], m_sprvram[0].bytes(), 0, bitmap, cliprect, screen.priority(), 0x00);
+		m_spr_old[1]->draw_sprites(m_sprvram[1], m_sprvram[1].bytes(), 0, bitmap, cliprect, screen.priority(), 0x02);
 	}
 	return 0;
 }
@@ -362,6 +361,7 @@ uint32_t f1gp2_state::screen_update(screen_device &screen, bitmap_ind16 &bitmap,
 	return 0;
 }
 
+
 /***************************************************************************
 
   BOOTLEG SUPPORT
@@ -387,8 +387,8 @@ void f1gp_state::f1gpbl_draw_sprites(screen_device &screen,bitmap_ind16 &bitmap,
 	{
 		int const x = (m_spriteram[attr_start + 2] & 0x03ff) - 48;
 		int const y = (256 - (m_spriteram[attr_start + 3 - 4] & 0x03ff)) - 15;
-		int const flipx = m_spriteram[attr_start + 1] & 0x0800;
-		int const flipy = m_spriteram[attr_start + 1] & 0x8000;
+		bool const flipx = BIT(m_spriteram[attr_start + 1], 11);
+		bool const flipy = BIT(m_spriteram[attr_start + 1], 15);
 		int const color = m_spriteram[attr_start + 1] & 0x000f;
 		int code = m_spriteram[attr_start + 0] & 0x3fff;
 		int const pri = 0; //?
@@ -414,7 +414,7 @@ void f1gp_state::f1gpbl_draw_sprites(screen_device &screen,bitmap_ind16 &bitmap,
 			gfx = 0;
 		}
 
-		m_gfxdecode->gfx(1 + gfx)->prio_transpen(bitmap, cliprect,
+		m_gfxdecode->gfx(2 + gfx)->prio_transpen(bitmap, cliprect,
 			code,
 			color,
 			flipx, flipy,
@@ -423,7 +423,7 @@ void f1gp_state::f1gpbl_draw_sprites(screen_device &screen,bitmap_ind16 &bitmap,
 			pri ? 0 : 0x2, 15);
 
 		// wrap around x
-		m_gfxdecode->gfx(1 + gfx)->prio_transpen(bitmap, cliprect,
+		m_gfxdecode->gfx(2 + gfx)->prio_transpen(bitmap, cliprect,
 			code,
 			color,
 			flipx, flipy,
@@ -460,19 +460,26 @@ uint32_t f1gp_state::screen_update_f1gpbl(screen_device &screen, bitmap_ind16 &b
 }
 
 
-// machine
-
 void f1gp_state::sh_bankswitch_w(uint8_t data)
 {
 	m_z80bank->set_entry(data & 0x01);
 }
 
-
-uint8_t f1gp_state::command_pending_r()
+uint8_t f1gp_state::soundlatch_pending_r()
 {
 	return (m_soundlatch->pending_r() ? 0xff : 0);
 }
 
+void f1gp_state::soundlatch_pending_w(int state)
+{
+	m_audiocpu->set_input_line(INPUT_LINE_NMI, state ? ASSERT_LINE : CLEAR_LINE);
+
+	// sound comms is 2-way (see soundlatch_pending_r),
+	// NMI routine is very short, so briefly set perfect_quantum to make sure that the timing is right
+	if (state)
+		machine().scheduler().perfect_quantum(attotime::from_usec(100));
+}
+
 
 void f1gp_state::f1gp_cpu1_map(address_map &map)
 {
@@ -495,7 +502,7 @@ void f1gp_state::f1gp_cpu1_map(address_map &map)
 	map(0xfff004, 0xfff005).portr("DSW1");
 	map(0xfff002, 0xfff005).w(FUNC(f1gp_state::fgscroll_w));
 	map(0xfff006, 0xfff007).portr("DSW2");
-	map(0xfff009, 0xfff009).r(FUNC(f1gp_state::command_pending_r)).w(m_soundlatch, FUNC(generic_latch_8_device::write)).umask16(0x00ff);
+	map(0xfff009, 0xfff009).r(FUNC(f1gp_state::soundlatch_pending_r)).w(m_soundlatch, FUNC(generic_latch_8_device::write)).umask16(0x00ff);
 	map(0xfff020, 0xfff023).w("gga", FUNC(vsystem_gga_device::write)).umask16(0x00ff);
 	map(0xfff040, 0xfff05f).w(m_k053936, FUNC(k053936_device::ctrl_w));
 	map(0xfff050, 0xfff051).portr("DSW3");
@@ -518,7 +525,7 @@ void f1gp2_state::f1gp2_cpu1_map(address_map &map)
 	map(0xfff002, 0xfff003).portr("WHEEL");
 	map(0xfff004, 0xfff005).portr("DSW1");
 	map(0xfff006, 0xfff007).portr("DSW2");
-	map(0xfff009, 0xfff009).r(FUNC(f1gp2_state::command_pending_r)).w(m_soundlatch, FUNC(generic_latch_8_device::write)).umask16(0x00ff);
+	map(0xfff009, 0xfff009).r(FUNC(f1gp2_state::soundlatch_pending_r)).w(m_soundlatch, FUNC(generic_latch_8_device::write)).umask16(0x00ff);
 	map(0xfff00a, 0xfff00b).portr("DSW3");
 	map(0xfff020, 0xfff03f).w(m_k053936, FUNC(k053936_device::ctrl_w));
 	map(0xfff044, 0xfff047).w(FUNC(f1gp2_state::fgscroll_w));
@@ -736,19 +743,43 @@ static INPUT_PORTS_START( f1gp2 )
 	PORT_DIPUNUSED( 0x001e, 0x001e )
 INPUT_PORTS_END
 
+static DEVICE_INPUT_DEFAULTS_START( linkplay )
+	DEVICE_INPUT_DEFAULTS( "RS232_RXBAUD", 0xff, RS232_BAUD_78125 )
+	DEVICE_INPUT_DEFAULTS( "RS232_TXBAUD", 0xff, RS232_BAUD_78125 )
+	DEVICE_INPUT_DEFAULTS( "RS232_DATABITS", 0xff, RS232_DATABITS_8 )
+	DEVICE_INPUT_DEFAULTS( "RS232_PARITY", 0xff, RS232_PARITY_EVEN )
+	DEVICE_INPUT_DEFAULTS( "RS232_STOPBITS", 0xff, RS232_STOPBITS_1 )
+DEVICE_INPUT_DEFAULTS_END
+
 
 
 static GFXDECODE_START( gfx_f1gp )
 	GFXDECODE_ENTRY( "fgtiles",  0, gfx_8x8x8_raw,          0x000,  1 )
+	GFXDECODE_RAM( "rozgfxram",  0, gfx_16x16x4_packed_msb, 0x300, 16 )
+GFXDECODE_END
+
+static GFXDECODE_START( gfx_f1gp_spr1 )
+	GFXDECODE_ENTRY( "sprites1", 0, gfx_16x16x4_packed_lsb, 0x100, 16 )
+GFXDECODE_END
+
+static GFXDECODE_START( gfx_f1gp_spr2 )
+	GFXDECODE_ENTRY( "sprites2", 0, gfx_16x16x4_packed_lsb, 0x200, 16 )
+GFXDECODE_END
+
+static GFXDECODE_START( gfx_f1gpbl )
+	GFXDECODE_ENTRY( "fgtiles",  0, gfx_8x8x8_raw,          0x000,  1 )
+	GFXDECODE_RAM( "rozgfxram",  0, gfx_16x16x4_packed_msb, 0x300, 16 )
 	GFXDECODE_ENTRY( "sprites1", 0, gfx_16x16x4_packed_lsb, 0x100, 16 )
 	GFXDECODE_ENTRY( "sprites2", 0, gfx_16x16x4_packed_lsb, 0x200, 16 )
-	GFXDECODE_RAM( "rozgfxram",  0, gfx_16x16x4_packed_msb, 0x300, 16 )
 GFXDECODE_END
 
 static GFXDECODE_START( gfx_f1gp2 )
 	GFXDECODE_ENTRY( "fgtiles",  0, gfx_8x8x8_raw,          0x000,  1 )
+	GFXDECODE_ENTRY( "roztiles", 0, gfx_16x16x4_packed_msb, 0x100, 16 )
+GFXDECODE_END
+
+static GFXDECODE_START( gfx_f1gp2_spr )
 	GFXDECODE_ENTRY( "sprites1", 0, gfx_16x16x4_packed_lsb, 0x200, 32 )
-	GFXDECODE_ENTRY( "sprites2", 0, gfx_16x16x4_packed_msb, 0x100, 16 )
 GFXDECODE_END
 
 
@@ -804,9 +835,18 @@ void f1gp_state::f1gp(machine_config &config)
 
 	ACIA6850(config, m_acia, 0);
 	m_acia->irq_handler().set_inputline("sub", M68K_IRQ_3);
-	m_acia->txd_handler().set("acia", FUNC(acia6850_device::write_rxd)); // loopback for now
+	m_acia->txd_handler().set("com_out", FUNC(rs232_port_device::write_txd));
 
-	clock_device &acia_clock(CLOCK(config, "acia_clock", 1'000'000)); // guessed
+	// dual DE-9 ports
+	// COM-IN (inner) and COM-OUT (outer) according to manual
+	rs232_port_device &rs232out(RS232_PORT(config, "com_out", default_rs232_devices, nullptr));
+	rs232out.set_option_device_input_defaults("null_modem", DEVICE_INPUT_DEFAULTS_NAME(linkplay));
+
+	rs232_port_device &rs232in(RS232_PORT(config, "com_in", default_rs232_devices, nullptr));
+	rs232in.rxd_handler().set("acia", FUNC(acia6850_device::write_rxd));
+	rs232in.set_option_device_input_defaults("null_modem", DEVICE_INPUT_DEFAULTS_NAME(linkplay));
+
+	clock_device &acia_clock(CLOCK(config, "acia_clock", 20_MHz_XTAL / 16)); // 78125 baud
 	acia_clock.signal_handler().set(m_acia, FUNC(acia6850_device::write_txc));
 	acia_clock.signal_handler().append(m_acia, FUNC(acia6850_device::write_rxc));
 
@@ -823,36 +863,31 @@ void f1gp_state::f1gp(machine_config &config)
 
 	VSYSTEM_GGA(config, "gga", XTAL(14'318'181) / 2); // divider not verified
 
-	VSYSTEM_SPR2(config, m_spr_old[0], 0);
+	VSYSTEM_SPR2(config, m_spr_old[0], 0, m_palette, gfx_f1gp_spr1);
 	m_spr_old[0]->set_tile_indirect_cb(FUNC(f1gp2_state::tile_callback<0>));
-	m_spr_old[0]->set_gfx_region(1);
 	m_spr_old[0]->set_pritype(2);
-	m_spr_old[0]->set_gfxdecode_tag(m_gfxdecode);
 
-	VSYSTEM_SPR2(config, m_spr_old[1], 0);
+	VSYSTEM_SPR2(config, m_spr_old[1], 0, m_palette, gfx_f1gp_spr2);
 	m_spr_old[1]->set_tile_indirect_cb(FUNC(f1gp2_state::tile_callback<1>));
-	m_spr_old[1]->set_gfx_region(2);
 	m_spr_old[1]->set_pritype(2);
-	m_spr_old[1]->set_gfxdecode_tag(m_gfxdecode);
 
 	K053936(config, m_k053936, 0);
 	m_k053936->set_wrap(1);
 	m_k053936->set_offsets(-58, -2);
 
 	// sound hardware
-	SPEAKER(config, "lspeaker").front_left();
-	SPEAKER(config, "rspeaker").front_right();
+	SPEAKER(config, "speaker", 2).front();
 
 	GENERIC_LATCH_8(config, m_soundlatch);
-	m_soundlatch->data_pending_callback().set_inputline(m_audiocpu, INPUT_LINE_NMI);
+	m_soundlatch->data_pending_callback().set(FUNC(f1gp_state::soundlatch_pending_w));
 	m_soundlatch->set_separate_acknowledge(true);
 
 	ym2610_device &ymsnd(YM2610(config, "ymsnd", XTAL(8'000'000)));
 	ymsnd.irq_handler().set_inputline(m_audiocpu, 0);
-	ymsnd.add_route(0, "lspeaker", 0.25);
-	ymsnd.add_route(0, "rspeaker", 0.25);
-	ymsnd.add_route(1, "lspeaker", 1.0);
-	ymsnd.add_route(2, "rspeaker", 1.0);
+	ymsnd.add_route(0, "speaker", 0.75, 0);
+	ymsnd.add_route(0, "speaker", 0.75, 1);
+	ymsnd.add_route(1, "speaker", 1.0, 0);
+	ymsnd.add_route(2, "speaker", 1.0, 1);
 }
 
 void f1gp_state::f1gpbl(machine_config &config)
@@ -871,12 +906,22 @@ void f1gp_state::f1gpbl(machine_config &config)
 
 	ACIA6850(config, m_acia, 0);
 	m_acia->irq_handler().set_inputline("sub", M68K_IRQ_3);
-	m_acia->txd_handler().set("acia", FUNC(acia6850_device::write_rxd)); // loopback for now
+	m_acia->txd_handler().set("com_out", FUNC(rs232_port_device::write_txd));
 
-	clock_device &acia_clock(CLOCK(config, "acia_clock", 1'000'000)); // guessed
+	// dual DE-9 ports
+	// COM-IN (inner) and COM-OUT (outer) according to manual
+	rs232_port_device &rs232out(RS232_PORT(config, "com_out", default_rs232_devices, nullptr));
+	rs232out.set_option_device_input_defaults("null_modem", DEVICE_INPUT_DEFAULTS_NAME(linkplay));
+
+	rs232_port_device &rs232in(RS232_PORT(config, "com_in", default_rs232_devices, nullptr));
+	rs232in.rxd_handler().set("acia", FUNC(acia6850_device::write_rxd));
+	rs232in.set_option_device_input_defaults("null_modem", DEVICE_INPUT_DEFAULTS_NAME(linkplay));
+
+	clock_device &acia_clock(CLOCK(config, "acia_clock", 20_MHz_XTAL / 16)); // 78125 baud
 	acia_clock.signal_handler().set(m_acia, FUNC(acia6850_device::write_txc));
 	acia_clock.signal_handler().append(m_acia, FUNC(acia6850_device::write_rxc));
 
+
 	// video hardware
 	screen_device &screen(SCREEN(config, "screen", SCREEN_TYPE_RASTER));
 	screen.set_refresh_hz(60);
@@ -885,7 +930,7 @@ void f1gp_state::f1gpbl(machine_config &config)
 	screen.set_screen_update(FUNC(f1gp_state::screen_update_f1gpbl));
 	screen.set_palette(m_palette);
 
-	GFXDECODE(config, m_gfxdecode, m_palette, gfx_f1gp);
+	GFXDECODE(config, m_gfxdecode, m_palette, gfx_f1gpbl);
 	PALETTE(config, m_palette).set_format(palette_device::xRGB_555, 2048);
 
 	//VSYSTEM_GGA(config, "gga", 0);
@@ -914,10 +959,8 @@ void f1gp2_state::f1gp2(machine_config &config)
 	config.device_remove("vsystem_spr_old1");
 	config.device_remove("vsystem_spr_old2");
 
-	VSYSTEM_SPR(config, m_spr, 0);
+	VSYSTEM_SPR(config, m_spr, 0, m_palette, gfx_f1gp2_spr);
 	m_spr->set_tile_indirect_cb(FUNC(f1gp2_state::tile_callback<0>));
-	m_spr->set_gfx_region(1);
-	m_spr->set_gfxdecode_tag(m_gfxdecode);
 
 	m_k053936->set_offsets(-48, -21);
 }
@@ -987,7 +1030,7 @@ ROM_START( f1gpa )
 	ROM_REGION( 0x20000, "sub", 0 ) // 68000 code
 	ROM_LOAD16_WORD_SWAP( "rom4-a.4", 0x00000, 0x20000, CRC(8e811d36) SHA1(2b806b50a3a307a21894687f16485ace287a7c4c) )
 
-	ROM_REGION( 0x20000, "audiocpu", 0 )    /* 64k for the audio CPU + banks */
+	ROM_REGION( 0x20000, "audiocpu", 0 )    // 64k for the audio CPU + banks
 	ROM_LOAD( "rom5-a.8", 0x00000, 0x20000, CRC(9ea36e35) SHA1(9254dea8362318d8cfbd5e36e476e0e235e6326a) )
 
 	ROM_REGION( 0x200000, "fgtiles", 0 )
@@ -1030,7 +1073,7 @@ ROM_START( f1gpb ) // 0F17-A-04 PCB
 	ROM_REGION( 0x20000, "sub", 0 ) // 68000 code
 	ROM_LOAD16_WORD_SWAP( "rom4.4", 0x00000, 0x20000, CRC(8e811d36) SHA1(2b806b50a3a307a21894687f16485ace287a7c4c) )
 
-	ROM_REGION( 0x20000, "audiocpu", 0 )    /* 64k for the audio CPU + banks */
+	ROM_REGION( 0x20000, "audiocpu", 0 )    // 64k for the audio CPU + banks
 	ROM_LOAD( "rom5.8", 0x00000, 0x20000, CRC(9ea36e35) SHA1(9254dea8362318d8cfbd5e36e476e0e235e6326a) )
 
 	ROM_REGION( 0x200000, "fgtiles", 0 )
@@ -1084,17 +1127,9 @@ ROM_START( f1gpbl )
 	ROM_LOAD( "15.ic153", 0x100000, 0x080000, CRC(c2867d7f) SHA1(86b1be9672cf9f610e1d7efff90d6a73dc1cdb90) )
 	ROM_LOAD( "14.ic154", 0x180000, 0x080000, CRC(0cd20423) SHA1(cddad02247b898c0a5a2fe061c41f68ecdf04d5c) )
 
-	/*
-	Roms 20 and 21 were missing from the PCB, however the others match perfectly (just with a different data layout)
-	I've reconstructed what should be the correct data for this bootleg.
-
-	Note, the bootleg combines 2 GFX regions into a single set of 4-way interleaved ROMs, so we load them in a user
-	region and use ROM_COPY.
-	*/
-
 	ROM_REGION( 0x200000, "user3", 0 )
-	ROM_LOAD32_BYTE( "rom21",    0x000003, 0x80000, CRC(7a08c3b7) SHA1(369123348a88513c066c239ed6aa4db5ae4ef0ac) )
-	ROM_LOAD32_BYTE( "rom20",    0x000001, 0x80000, CRC(bd1273d0) SHA1(cc7caee231fe3bd87d8403d34059e1292c7f7a00) )
+	ROM_LOAD32_BYTE( "21.ic143", 0x000003, 0x80000, CRC(7a08c3b7) SHA1(369123348a88513c066c239ed6aa4db5ae4ef0ac) )
+	ROM_LOAD32_BYTE( "20.ic142", 0x000001, 0x80000, CRC(bd1273d0) SHA1(cc7caee231fe3bd87d8403d34059e1292c7f7a00) )
 	ROM_LOAD32_BYTE( "19.ic141", 0x000002, 0x80000, CRC(aa4ebdfe) SHA1(ed117e6a84554c5ed2ad4379b834898a4c40d51e) )
 	ROM_LOAD32_BYTE( "18.ic140", 0x000000, 0x80000, CRC(9b2a4325) SHA1(b2020e08251366686c4c0045f3fd523fa327badf) )
 
@@ -1105,7 +1140,10 @@ ROM_START( f1gpbl )
 	ROM_COPY("user3", 0x100000, 0, 0x80000)
 
 	ROM_REGION( 0x90000, "oki", 0 )
-	ROM_LOAD( "6.ic13", 0x00000, 0x030000, CRC(6e83ffd8) SHA1(618fd6cd6c0844a4be96f77ff22cd41364718d16) ) // a second dump has 0x40 instead of 0x44 at 0x54cbc. Which one is bad?
+	// 0x40 instead at 0x54cbc has been confirmed by multiple dumps from different PCBs.
+	// there is a dump with 0x44 but given it was on only one PCB and with the only difference a single bit of a single byte,
+	// it is almost surely a bad read
+	ROM_LOAD( "6.ic13", 0x00000, 0x030000, CRC(469f3ee1) SHA1(9d5f0cd6463abf82e74bf4641fc1bcf478084c16) )
 	ROM_CONTINUE(       0x40000, 0x050000 )
 ROM_END
 
@@ -1131,7 +1169,7 @@ ROM_START( f1gp2 )
 	ROM_REGION( 0x200000, "sprites1", 0 )
 	ROM_LOAD( "rom15", 0x000000, 0x200000, CRC(1ac03e2e) SHA1(9073d0ae24364229a993046bd71e403988692993) )
 
-	ROM_REGION( 0x400000, "sprites2", 0 )
+	ROM_REGION( 0x400000, "roztiles", 0 )
 	ROM_LOAD16_WORD_SWAP( "rom11", 0x000000, 0x100000, CRC(b22a2c1f) SHA1(b5e67726be5a8561cd04c3c07895b8518b73b89c) )
 	ROM_LOAD16_WORD_SWAP( "rom10", 0x100000, 0x100000, CRC(43fcbe23) SHA1(54ab58d904890a0b907e674f855092e974c45edc) )
 	ROM_LOAD16_WORD_SWAP( "rom9",  0x200000, 0x100000, CRC(1bede8a1) SHA1(325ecc3afb30d281c2c8a56719e83e4dc20545bb) )
@@ -1147,9 +1185,9 @@ ROM_END
 } // anonymous namespace
 
 
-GAME( 1991, f1gp,   0,    f1gp,   f1gp,  f1gp_state,  empty_init, ROT90, "Video System Co.",   "F-1 Grand Prix (set 1)",            MACHINE_NO_COCKTAIL | MACHINE_NODEVICE_LAN | MACHINE_SUPPORTS_SAVE ) // censored banners, US McO'River release?
-GAME( 1991, f1gpa,  f1gp, f1gp,   f1gp,  f1gp_state,  empty_init, ROT90, "Video System Co.",   "F-1 Grand Prix (set 2)",            MACHINE_NO_COCKTAIL | MACHINE_NODEVICE_LAN | MACHINE_SUPPORTS_SAVE )
-GAME( 1991, f1gpb,  f1gp, f1gp,   f1gp,  f1gp_state,  empty_init, ROT90, "Video System Co.",   "F-1 Grand Prix (set 3)",            MACHINE_NOT_WORKING | MACHINE_NO_COCKTAIL | MACHINE_NODEVICE_LAN | MACHINE_SUPPORTS_SAVE ) // supposed to be the earliest version dumped and only work with steering wheel
-GAME( 1991, f1gpbl, f1gp, f1gpbl, f1gp,  f1gp_state,  empty_init, ROT90, "bootleg (Playmark)", "F-1 Grand Prix (Playmark bootleg)", MACHINE_NOT_WORKING | MACHINE_NODEVICE_LAN | MACHINE_SUPPORTS_SAVE ) // PCB marked 'Super Formula II', manufactured by Playmark.
+GAME( 1991, f1gp,   0,    f1gp,   f1gp,  f1gp_state,  empty_init, ROT90, "Video System Co.",   "F-1 Grand Prix (set 1)",            MACHINE_NO_COCKTAIL | MACHINE_SUPPORTS_SAVE ) // censored banners, US McO'River release?
+GAME( 1991, f1gpa,  f1gp, f1gp,   f1gp,  f1gp_state,  empty_init, ROT90, "Video System Co.",   "F-1 Grand Prix (set 2)",            MACHINE_NO_COCKTAIL | MACHINE_SUPPORTS_SAVE )
+GAME( 1991, f1gpb,  f1gp, f1gp,   f1gp,  f1gp_state,  empty_init, ROT90, "Video System Co.",   "F-1 Grand Prix (set 3)",            MACHINE_NOT_WORKING | MACHINE_NO_COCKTAIL | MACHINE_SUPPORTS_SAVE ) // supposed to be the earliest version dumped and only work with steering wheel
+GAME( 1991, f1gpbl, f1gp, f1gpbl, f1gp,  f1gp_state,  empty_init, ROT90, "bootleg (Playmark)", "F-1 Grand Prix (Playmark bootleg)", MACHINE_NOT_WORKING | MACHINE_SUPPORTS_SAVE ) // PCB marked 'Super Formula II', manufactured by Playmark.
 
-GAME( 1992, f1gp2,  0,    f1gp2,  f1gp2, f1gp2_state, empty_init, ROT90, "Video System Co.",   "F-1 Grand Prix Part II",            MACHINE_NO_COCKTAIL | MACHINE_NODEVICE_LAN | MACHINE_SUPPORTS_SAVE )
+GAME( 1992, f1gp2,  0,    f1gp2,  f1gp2, f1gp2_state, empty_init, ROT90, "Video System Co.",   "F-1 Grand Prix Part II",            MACHINE_NO_COCKTAIL | MACHINE_SUPPORTS_SAVE )
