@@ -14,7 +14,7 @@
 
 #include "modules/osdmodule.h"
 
-#if defined(OSD_SDL)
+#if defined(OSD_SDL) && !defined(SDLMAME_SDL3)
 
 // OSD headers
 #include "sdlopts.h"
@@ -365,7 +365,7 @@ void renderer_sdl2::render_quad(texture_info *texture, const render_primitive &p
 			copyinfo->time += (m_last_blit_time * (int64_t) (texture->raw_width() * texture->raw_height())) / (int64_t) m_last_blit_pixels;
 		}
 		copyinfo->samples++;
-		copyinfo->perf = ( texture->m_copyinfo->pixel_count * (osd_ticks_per_second()/1000)) / texture->m_copyinfo->time;
+		copyinfo->perf = (texture->m_copyinfo->pixel_count * (osd_ticks_per_second()/1000)) / std::max<int64_t>(texture->m_copyinfo->time, 1);
 	}
 	else
 	{
@@ -1181,12 +1181,10 @@ copy_info_t const video_sdl2::s_blit_info_default[] =
 
 } // namespace osd
 
-
-#else // defined(OSD_SDL)
+#else // defined(OSD_SDL) && !defined(SDLMAME_SDL3)
 
 namespace osd { namespace { MODULE_NOT_SUPPORTED(video_sdl2, OSD_RENDERER_PROVIDER, "accel") } }
 
-#endif // defined(OSD_SDL)
-
+#endif // defined(OSD_SDL) && !defined(SDLMAME_SDL3)
 
 MODULE_DEFINITION(RENDERER_SDL2, osd::video_sdl2)
